@@ -49,7 +49,7 @@ use nix::{
         signal::{kill, sigprocmask, SigmaskHow, Signal},
         signalfd::SigSet,
         socket::{SockaddrLike, SockaddrStorage},
-        stat::{fstat, Mode},
+        stat::Mode,
     },
     unistd::{pipe2, Pid},
 };
@@ -71,7 +71,8 @@ const PKG_DESCRIPTION: &str = env!("CARGO_PKG_DESCRIPTION");
 const PKG_AUTHORS: &str = env!("CARGO_PKG_AUTHORS");
 const PKG_LICENSE: &str = env!("CARGO_PKG_LICENSE");
 
-const CAPSET_RPATH: &[&str] = &["stat", "read", "chdir", "readdir"];
+const CAPSET_LPATH: &[&str] = &["walk", "stat", "chdir"];
+const CAPSET_RPATH: &[&str] = &["read", "readdir"];
 const CAPSET_WPATH: &[&str] = &["write", "truncate"];
 const CAPSET_CPATH: &[&str] = &["create", "delete", "rename"];
 const CAPSET_DPATH: &[&str] = &["mkbdev", "mkcdev"];
@@ -81,8 +82,12 @@ const CAPSET_FOWN: &[&str] = &["chown", "chgrp"];
 const CAPSET_FATTR: &[&str] = &["chmod", "chattr", "utime"];
 const CAPSET_NET: &[&str] = &["net/bind", "net/connect", "net/sendfd"];
 const CAPSET_INET: &[&str] = &["net/bind", "net/connect"];
+const CAPSET_BNET: &[&str] = &["net/bind"];
+const CAPSET_CNET: &[&str] = &["net/connect"];
+const CAPSET_SNET: &[&str] = &["net/sendfd"];
 
 const CAPSETS: &[(&str, &[&str])] = &[
+    ("lpath", CAPSET_LPATH),
     ("rpath", CAPSET_RPATH),
     ("wpath", CAPSET_WPATH),
     ("cpath", CAPSET_CPATH),
@@ -93,10 +98,14 @@ const CAPSETS: &[(&str, &[&str])] = &[
     ("fattr", CAPSET_FATTR),
     ("net", CAPSET_NET),
     ("inet", CAPSET_INET),
+    ("bnet", CAPSET_BNET),
+    ("cnet", CAPSET_CNET),
+    ("snet", CAPSET_SNET),
 ];
 
 const CAP_ORDER: &[&str] = &[
     // aliases (keep before base)
+    "lpath",
     "rpath",
     "wpath",
     "cpath",
@@ -107,7 +116,11 @@ const CAP_ORDER: &[&str] = &[
     "fattr",
     "net",
     "inet",
+    "bnet",
+    "cnet",
+    "snet",
     // base capabilities
+    "walk",
     "stat",
     "read",
     "write",
@@ -495,7 +508,7 @@ fn lookup_addr(addr: IpAddr) -> Result<String, Errno> {
 ///
 /// Replicated from `syd::hash::HashAlgorithm` to avoid depending on Syd.
 #[derive(Debug, Clone, Copy)]
-pub enum HashAlgorithm {
+enum HashAlgorithm {
     /// Crc32
     Crc32,
     /// Crc64
@@ -803,7 +816,7 @@ fn command_profile(opts: ProfileOpts) -> Result<ExitCode, Errno> {
     let log_fd = fd_rw.as_raw_fd().to_string();
     syd.env("SYD_LOG_FD", &log_fd);
     if env::var_os("PANDORA_DEBUG").is_some() {
-        let ino = fstat(&fd_rw).map(|st| st.st_ino)?;
+        let ino = fstatx(&fd_rw, STATX_INO).map(|stx| stx.stx_ino)?;
         eprintln!(
             "{} syd log fd set to pipe {} with inode {}.",
             style("pandora:").bold().magenta(),
@@ -1343,7 +1356,7 @@ fn do_inspect(
         if caps.contains("net/bind") {
             if ['/', '@', '!'].iter().any(|&c| elem.starts_with(c)) {
                 // UNIX socket (domain, abstract or unnamed).
-                wln!(&mut output, "allow/net/bind+{}", elem)?;
+                wln!(&mut output, "allow/bnet+{}", elem)?;
             } else {
                 // IPv{4,6} address
                 let ip = elem.split('!').next().ok_or(Errno::EINVAL)?;
@@ -1351,30 +1364,30 @@ fn do_inspect(
                 if let Ok(host) = lookup_addr(ip) {
                     wln!(&mut output, "# {host}")?;
                 }
-                wln!(&mut output, "allow/net/bind+{}", elem)?;
+                wln!(&mut output, "allow/bnet+{}", elem)?;
             }
             done = true;
         }
         if caps.contains("net/connect") {
             if ['/', '@', '!'].iter().any(|&c| elem.starts_with(c)) {
                 // UNIX socket (domain, abstract or unnamed).
-                wln!(&mut output, "allow/net/connect+{}", elem)?;
+                wln!(&mut output, "allow/cnet+{}", elem)?;
             } else {
                 let ip = elem.split('!').next().ok_or(Errno::EINVAL)?;
                 let ip = ip.parse::<IpAddr>().or(Err(Errno::EINVAL))?;
                 if let Ok(host) = lookup_addr(ip) {
                     wln!(&mut output, "# {host}")?;
                 }
-                wln!(&mut output, "allow/net/connect+{}", elem)?;
+                wln!(&mut output, "allow/cnet+{}", elem)?;
             }
             done = true;
         }
         if caps.contains("net/sendfd") {
             if ['/', '@', '!'].iter().any(|&c| elem.starts_with(c)) {
                 // UNIX socket (domain, abstract or unnamed).
-                wln!(&mut output, "allow/net/sendfd+{elem}")?;
+                wln!(&mut output, "allow/snet+{elem}")?;
             } else {
-                unreachable!("BUG: invalid net/sendfd entry {:?}", entry);
+                unreachable!("BUG: invalid snet entry {:?}", entry);
             }
             caps.remove("net/sendfd");
             done = true;
@@ -1385,12 +1398,23 @@ fn do_inspect(
         }
 
         // Perform alias expansion.
-        for (alias, members) in CAPSETS {
-            if members.iter().all(|m| caps.contains(*m)) {
-                for m in *members {
-                    caps.remove(*m);
+        loop {
+            let mut changed = false;
+
+            for (alias, members) in CAPSETS {
+                if caps.contains(*alias) {
+                    continue;
                 }
-                caps.insert((*alias).to_string());
+                if members.iter().all(|m| caps.contains(*m)) {
+                    for m in *members {
+                        changed |= caps.remove(*m);
+                    }
+                    changed |= caps.insert((*alias).to_string());
+                }
+            }
+
+            if !changed {
+                break;
             }
         }
 
@@ -2220,3 +2244,102 @@ fn set_cloexec<Fd: AsFd>(fd: Fd, state: bool) -> Result<(), Errno> {
     )
     .map(drop)
 }
+
+// Keep in sync with syd/src/compat.rs
+//
+// This structure represents the Linux data structure `struct statx_timestamp`
+#[repr(C)]
+#[derive(Copy, Clone, Debug, Default)]
+struct FileStatxTimestamp {
+    tv_sec: i64,
+    tv_nsec: u32,
+    __statx_timestamp_pad1: [i32; 1],
+}
+
+impl PartialEq for FileStatxTimestamp {
+    fn eq(&self, other: &Self) -> bool {
+        self.tv_sec == other.tv_sec && self.tv_nsec == other.tv_nsec
+    }
+}
+
+impl Eq for FileStatxTimestamp {}
+
+impl PartialOrd for FileStatxTimestamp {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Ord for FileStatxTimestamp {
+    fn cmp(&self, other: &Self) -> Ordering {
+        match self.tv_sec.cmp(&other.tv_sec) {
+            Ordering::Equal => self.tv_nsec.cmp(&other.tv_nsec),
+            ord => ord,
+        }
+    }
+}
+
+// This structure represents the Linux data structure `struct statx`
+#[repr(C)]
+#[derive(Copy, Clone, Debug)]
+struct FileStatx {
+    stx_mask: u32,       // What results were written [uncond]
+    stx_blksize: u32,    // Preferred general I/O size [uncond]
+    stx_attributes: u64, // Flags conveying information about the file [uncond]
+
+    stx_nlink: u32, // Number of hard links
+    stx_uid: u32,   // User ID of owner
+    stx_gid: u32,   // Group ID of owner
+    stx_mode: u16,  // File mode
+    __statx_pad1: [u16; 1],
+
+    stx_ino: u64,             // Inode number
+    stx_size: u64,            // File size
+    stx_blocks: u64,          // Number of 512-byte blocks allocated
+    stx_attributes_mask: u64, // Mask to show what's supported in stx_attributes
+
+    stx_atime: FileStatxTimestamp, // Last access time
+    stx_btime: FileStatxTimestamp, // File creation time
+    stx_ctime: FileStatxTimestamp, // Last attribute change time
+    stx_mtime: FileStatxTimestamp, // Last data modification time
+
+    stx_rdev_major: u32, // Device ID of special file [if bdev/cdev]
+    stx_rdev_minor: u32,
+
+    // Note, these are not not public on purpose
+    // as they return inconsistent values on filesytems
+    // such as btrfs and overlayfs. `stx_mnt_id` should
+    // be used instead.
+    stx_dev_major: u32, // ID of device containing file [uncond]
+    stx_dev_minor: u32,
+
+    stx_mnt_id: u64,
+    stx_dio_mem_align: u32,    // Memory buffer alignment for direct I/O
+    stx_dio_offset_align: u32, // File offset alignment for direct I/O
+
+    __statx_pad2: [u64; 12], // Spare space for future expansion
+}
+
+// Safe statx() wrapper to use with a FD only.
+fn fstatx<Fd: AsFd>(fd: Fd, mask: libc::c_uint) -> Result<FileStatx, Errno> {
+    let fd = fd.as_fd().as_raw_fd();
+    let mut dst = std::mem::MaybeUninit::uninit();
+
+    // SAFETY: Neither nix nor libc has a wrapper for statx.
+    Errno::result(unsafe {
+        libc::syscall(
+            libc::SYS_statx,
+            fd,
+            b"\0".as_ptr(),
+            libc::AT_EMPTY_PATH,
+            mask,
+            dst.as_mut_ptr(),
+        )
+    })?;
+
+    // SAFETY: statx returned success.
+    Ok(unsafe { dst.assume_init() })
+}
+
+// Want/got stx_ino.
+const STATX_INO: libc::c_uint = 0x00000100;
