@@ -17,6 +17,7 @@ use std::{
     cmp::Ordering,
     os::fd::{AsFd, AsRawFd, RawFd},
     ptr::NonNull,
+    sync::LazyLock,
 };
 
 use bitflags::bitflags;
@@ -33,7 +34,6 @@ use nix::{
     unistd::Pid,
     NixPath,
 };
-use once_cell::sync::Lazy;
 use serde::{ser::SerializeMap, Serialize, Serializer};
 
 use crate::{config::*, fs::FileType, XPath};
@@ -370,7 +370,7 @@ pub struct XattrArgs {
 // Note getxattrat may not be available,
 // and libc::SYS_getxattrat may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_GETXATTRAT: Lazy<libc::c_long> = Lazy::new(|| {
+static SYS_GETXATTRAT: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("getxattrat")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -421,7 +421,7 @@ pub unsafe fn getxattrat<Fd: AsFd, P: ?Sized + NixPath>(
 // Note setxattrat may not be available,
 // and libc::SYS_setxattrat may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_SETXATTRAT: Lazy<libc::c_long> = Lazy::new(|| {
+static SYS_SETXATTRAT: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("setxattrat")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -468,7 +468,7 @@ pub unsafe fn setxattrat<Fd: AsFd, P: ?Sized + NixPath>(
 // Note listxattrat may not be available,
 // and libc::SYS_listxattrat may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_LISTXATTRAT: Lazy<libc::c_long> = Lazy::new(|| {
+static SYS_LISTXATTRAT: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("listxattrat")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -518,7 +518,7 @@ pub unsafe fn listxattrat<Fd: AsFd, P: ?Sized + NixPath>(
 // Note removexattrat may not be available,
 // and libc::SYS_removexattrat may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_REMOVEXATTRAT: Lazy<libc::c_long> = Lazy::new(|| {
+static SYS_REMOVEXATTRAT: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("removexattrat")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -633,7 +633,7 @@ pub struct FileStatx {
     pub stx_rdev_minor: u32,
 
     // Note, these are not not public on purpose
-    // as they return inconsistent values on filesytems
+    // as they return inconsistent values on filesystems
     // such as btrfs and overlayfs. `stx_mnt_id` should
     // be used instead.
     pub(crate) stx_dev_major: u32, // ID of device containing file [uncond]
@@ -1120,7 +1120,7 @@ pub fn waitid(id: Id, flags: WaitPidFlag) -> Result<WaitStatus, Errno> {
         // Memory is zeroed rather than uninitialized, as not all platforms
         // initialize the memory in the StillAlive case
         let mut siginfo: libc::siginfo_t = std::mem::zeroed();
-        Errno::result(libc::waitid(idtype, idval, &mut siginfo, flags.bits()))?;
+        Errno::result(libc::waitid(idtype, idval, &raw mut siginfo, flags.bits()))?;
         siginfo
     };
 
@@ -1183,6 +1183,8 @@ pub(crate) const PF_INET6: libc::sa_family_t = libc::AF_INET6 as libc::sa_family
 pub(crate) const PF_ALG: libc::sa_family_t = libc::AF_ALG as libc::sa_family_t;
 #[allow(clippy::cast_possible_truncation)]
 pub(crate) const PF_NETLINK: libc::sa_family_t = libc::AF_NETLINK as libc::sa_family_t;
+/* From <bits/socket.h>, expect this to be updated regularly. */
+pub(crate) const PF_MAX: libc::sa_family_t = 46;
 
 /// nix' SockAddrLike.family() function does not support AF_ALG.
 /// This is a workaround that accesses the underlying sockaddr directly.
@@ -1359,7 +1361,7 @@ pub fn epoll_ctl_safe<E: AsFd>(
                     epoll.as_fd().as_raw_fd(),
                     EpollOp::EpollCtlAdd as libc::c_int,
                     fd,
-                    &mut event,
+                    &raw mut event,
                 )
             }),
             Errno::EEXIST,
@@ -1398,7 +1400,7 @@ pub fn epoll_ctl_mod_safe<E: AsFd>(
             epoll.as_fd().as_raw_fd(),
             EpollOp::EpollCtlMod as libc::c_int,
             fd,
-            &mut event,
+            &raw mut event,
         )
     })
     .map(drop)
@@ -1501,7 +1503,7 @@ pub fn getsockdomain<Fd: AsFd>(fd: Fd) -> Result<libc::c_int, Errno> {
             libc::SOL_SOCKET,
             libc::SO_DOMAIN,
             std::ptr::addr_of_mut!(fml) as *mut _,
-            &mut len,
+            &raw mut len,
         )
     })?;
 
@@ -1575,8 +1577,8 @@ impl std::fmt::Display for LsmId {
     }
 }
 
-/// Lazy-initialized system call number for `lsm_list_modules`.
-pub static SYS_LSM_LIST_MODULES: Lazy<libc::c_long> = Lazy::new(|| {
+/// LazyLock-initialized system call number for `lsm_list_modules`.
+pub static SYS_LSM_LIST_MODULES: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("lsm_list_modules")
         .map(i32::from)
         .map(libc::c_long::from)
