@@ -6,7 +6,6 @@
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
-    arch::asm,
     fmt::Display,
     os::{
         fd::{AsFd, AsRawFd, RawFd},
@@ -15,6 +14,7 @@ use std::{
     path::Path,
     process::{exit, Command},
     str::FromStr,
+    sync::atomic::Ordering,
 };
 
 use btoi::btoi;
@@ -49,9 +49,15 @@ use crate::{
         Scope, ABI,
     },
     path::{mask_path, XPathBuf},
-    sandbox::IoctlMap,
+    sandbox::{IoctlMap, Sandbox},
 };
 
+// Used as dummy errno(3) after SECCOMP_IOCTL_NOTIF_ADDFD with SECCOMP_ADDFD_FLAG_SEND flag.
+pub(crate) const EIDRM: i32 = -libc::EIDRM;
+
+// Used as dummy errno(3) to initiate Ghost Mode.
+pub(crate) const EOWNERDEAD: i32 = -libc::EOWNERDEAD;
+
 /// Confine current process using MDWE prctl(2).
 ///
 /// Use `no_inherit` to prevent inheriting the restriction to children.
@@ -355,6 +361,104 @@ pub fn confine_scmp_pwritev2(ssb: bool) -> SydResult<()> {
     Ok(())
 }
 
+/// Allow clone(2) operations without namespaces.
+pub fn confine_scmp_clone(ctx: &mut ScmpFilterContext) -> SydResult<()> {
+    let syscall = match ScmpSyscall::from_name("clone") {
+        Ok(s) => s,
+        Err(_) => {
+            info!("ctx": "confine", "op": "allow_syscall",
+                "msg": "invalid or unsupported syscall clone");
+            return Ok(());
+        }
+    };
+
+    let ns_mask = CloneFlags::CLONE_NEWNS
+        | CloneFlags::CLONE_NEWUTS
+        | CloneFlags::CLONE_NEWIPC
+        | CloneFlags::CLONE_NEWUSER
+        | CloneFlags::CLONE_NEWNET
+        | CloneFlags::CLONE_NEWPID
+        | CloneFlags::CLONE_NEWCGROUP
+        | CLONE_NEWTIME;
+    #[allow(clippy::cast_sign_loss)]
+    let ns_mask = ns_mask.bits() as u64;
+    ctx.add_rule_conditional(
+        ScmpAction::Allow,
+        syscall,
+        &[scmp_cmp!($arg0 & ns_mask == 0)],
+    )?;
+
+    Ok(())
+}
+
+/// Deny clone3(2) with ENOSYS for compatibility.
+pub fn confine_scmp_clone3(ctx: &mut ScmpFilterContext) -> SydResult<()> {
+    let syscall = match ScmpSyscall::from_name("clone3") {
+        Ok(s) => s,
+        Err(_) => {
+            info!("ctx": "confine", "op": "allow_syscall",
+                "msg": "invalid or unsupported syscall clone3");
+            return Ok(());
+        }
+    };
+
+    ctx.add_rule(ScmpAction::Errno(libc::ENOSYS), syscall)?;
+
+    Ok(())
+}
+
+/// Allow writes to sandbox `SYD_LOG_FD` only.
+///
+/// If logging is disabled:
+///
+/// a. If `max` is `None`, deny write(2) completely.
+/// b. If `max` is `Some(limit)`, allow writes up to `max` bytes.
+///
+/// It is OK for the `SYD_LOG_FD` to be negative,
+/// in which case no rule will be inserted
+/// for the fd.
+///
+/// # Exceptions
+///
+/// 1. Allow write(2) globally if profiling is enabled.
+/// 2. Allow write(2) globally if `chk_mem` is true,
+///    and `Sandbox::memory_access` is less than 2.
+///    This is required for proc_pid_mem(5) access.
+pub fn confine_scmp_write(
+    ctx: &mut ScmpFilterContext,
+    max: Option<u64>,
+    chk_mem: bool,
+) -> SydResult<()> {
+    let syscall = match ScmpSyscall::from_name("write") {
+        Ok(syscall) => syscall,
+        Err(_) => {
+            info!("ctx": "confine", "op": "allow_syscall",
+                "msg": "invalid or unsupported syscall write");
+            return Ok(());
+        }
+    };
+
+    if cfg!(feature = "prof") && (chk_mem && Sandbox::memory_access() < 2) {
+        ctx.add_rule(ScmpAction::Allow, syscall)?;
+        return Ok(());
+    }
+
+    if let Ok(log_fd) = u64::try_from(crate::log::LOG_FD.load(Ordering::Relaxed)) {
+        ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg0 == log_fd)])?;
+        if let Some(max) = max {
+            ctx.add_rule_conditional(
+                ScmpAction::Allow,
+                syscall,
+                &[scmp_cmp!($arg0 != log_fd), scmp_cmp!($arg2 <= max)],
+            )?;
+        }
+    } else if let Some(max) = max {
+        ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg2 <= max)])?;
+    } // else deny write(2) completely.
+
+    Ok(())
+}
+
 /// Convenience `Command` run wrapper which returns:
 ///
 /// - Same exit code as the process on clean exit.
@@ -740,11 +844,12 @@ pub(crate) fn op2name(op: u8) -> &'static str {
         0x2 => "bind",
         0x3 => "connect",
         0x5 => "accept",
+        0x6 => "getsockname",
         0xb => "sendto",
         0x10 => "sendmsg",
         0x12 => "accept4",
         0x14 => "sendmmsg",
-        _ => unreachable!(),
+        _ => unreachable!("BUG: op2name called with unsupported op:{op:#x}, report a bug!"),
     }
 }
 
@@ -1592,21 +1697,9 @@ pub(crate) const NAMESPACE_FLAGS_ALL: libc::c_int = libc::CLONE_NEWNS
     | CLONE_NEWTIME.bits();
 
 pub(crate) const NAMESPACE_NAMES: &[&str] = &[
-    "mount", "ipc", "net", "pid", "uts", "user", "cgroup", "time",
+    "user", "mount", "ipc", "net", "pid", "uts", "cgroup", "time",
 ];
 
-/// Convert CLONE namespace flags to a Vector of Strings.
-pub fn nsflags_name(flags: libc::c_int) -> Vec<String> {
-    let mut names = Vec::with_capacity(NAMESPACE_FLAGS.len());
-    for &flag in NAMESPACE_FLAGS {
-        if flags & flag != 0 {
-            names.push(nsflag_name(flag));
-        }
-    }
-
-    names
-}
-
 /// Convert a CLONE namespace flag to its String representation.
 pub fn nsflag_name(flag: libc::c_int) -> String {
     match flag {
@@ -1850,7 +1943,7 @@ impl ExportMode {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "mov eax, 0x2", // 0x2 is the syscall number for fork on x86
         "int 0x80",     // Interrupt to make the syscall
         out("eax") _,
@@ -1866,7 +1959,7 @@ pub unsafe fn fork_fast() {
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
     // Inline assembly for x86-64
-    asm!(
+    std::arch::asm!(
         "mov rax, 57", // 57 is the syscall number for fork on x86-64
         "syscall",
         out("rax") _,
@@ -1881,7 +1974,7 @@ pub unsafe fn fork_fast() {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "mov x0, 17",  // SIGCHLD
         "mov x1, 0",   // child_stack (null, not recommended)
         "mov x8, 220", // syscall number for clone
@@ -1898,7 +1991,7 @@ pub unsafe fn fork_fast() {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "mov r7, #2", // 2 is the syscall number for fork on ARM
         "swi #0",     // Software interrupt to make the syscall
         out("r0") _,
@@ -1916,7 +2009,7 @@ pub unsafe fn fork_fast() {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "li 0, 2",     // Load immediate 2 into register r0 (syscall number for fork)
         "sc",          // System call
         out("r3") _,   // Output from r3 (return value of fork)
@@ -1932,7 +2025,7 @@ pub unsafe fn fork_fast() {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "li a7, 220",  // syscall number for clone on riscv64
         "li a0, 17",   // SIGCHLD
         "li a1, 0",    // child_stack (null, not recommended)
@@ -1952,7 +2045,7 @@ pub unsafe fn fork_fast() {
 ///
 /// Unsafe to be fast!
 pub unsafe fn fork_fast() {
-    asm!(
+    std::arch::asm!(
         "lgr %r1, 2", // Load syscall number for fork (2) directly into %r1.
         "svc 0",      // Supervisor Call to invoke the syscall.
     );
