@@ -19,6 +19,7 @@ use std::{
     thread::JoinHandle,
 };
 
+use btoi::ParseIntegerError;
 #[cfg(feature = "oci")]
 use libcgroups::common::AnyManagerError;
 #[cfg(feature = "oci")]
@@ -71,6 +72,8 @@ pub enum SydError {
     /// This error type represents integer parse errors.
     ParseInt(ParseIntError),
     /// This error type represents integer parse errors.
+    ParseInteger(ParseIntegerError),
+    /// This error type represents integer parse errors.
     TryInt(TryFromIntError),
     /// This error type represents a slice conversion error.
     TrySlice(TryFromSliceError),
@@ -86,7 +89,7 @@ pub enum SydError {
     /// This error type represents a Cgroup setup error.
     CgSetup(CreateCgroupSetupError),
     #[cfg(feature = "oci")]
-    /// This error type represents a miscallenous Cgroup error.
+    /// This error type represents a miscellaneous Cgroup error.
     CgMisc(AnyManagerError),
     #[cfg(feature = "oci")]
     /// This error type represents container errors.
@@ -113,6 +116,7 @@ impl SydError {
             Self::Nix(errno) => Some(*errno),
             Self::Proc(error) => proc_error_to_errno(error),
             Self::Scmp(error) => scmp2no(error),
+            Self::ParseInt(_) | Self::ParseInteger(_) | Self::ParseSize(_) => Some(Errno::EINVAL),
             _ => None,
         }
     }
@@ -126,9 +130,10 @@ impl fmt::Debug for SydError {
             Self::Caps(error) => write!(f, "CapsError: {error:?}"),
             Self::Elf(error) => write!(f, "ElfError: {error:?}"),
             Self::Env(error) => write!(f, "LookupError<VarError>: {error:?}"),
-            Self::Nix(errno) => write!(f, "NixError: {errno:?}"),
+            Self::Nix(errno) => write!(f, "LinuxError: {errno:?}"),
             Self::Json(error) => write!(f, "JsonError: {error:?}"),
             Self::ParseInt(error) => write!(f, "ParseIntError: {error:?}"),
+            Self::ParseInteger(error) => write!(f, "ParseIntegerError: {error:?}"),
             Self::Scmp(error) => write!(f, "SeccompError: {error:?}"),
             Self::TryInt(error) => write!(f, "TryFromIntError: {error:?}"),
             Self::TrySlice(error) => write!(f, "TryFromSliceError: {error:?}"),
@@ -161,9 +166,10 @@ impl fmt::Display for SydError {
             Self::Caps(error) => write!(f, "CapsError: {error}"),
             Self::Elf(error) => write!(f, "ElfError: {error}"),
             Self::Env(error) => write!(f, "LookupError<VarError>: {error}"),
-            Self::Nix(errno) => write!(f, "NixError: {errno}"),
+            Self::Nix(errno) => write!(f, "LinuxError: {errno}"),
             Self::Json(error) => write!(f, "JsonError: {error}"),
             Self::ParseInt(error) => write!(f, "ParseIntError: {error}"),
+            Self::ParseInteger(error) => write!(f, "ParseIntegerError: {error}"),
             Self::Scmp(error) => write!(f, "SeccompError: {error}"),
             Self::TryInt(error) => write!(f, "TryFromIntError: {error}"),
             Self::TrySlice(error) => write!(f, "TryFromSliceError: {error}"),
@@ -195,6 +201,7 @@ impl std::error::Error for SydError {
             Self::Args(error) => Some(error),
             Self::Nix(errno) => Some(errno),
             Self::ParseInt(error) => Some(error),
+            Self::ParseInteger(error) => Some(error),
             Self::TryInt(error) => Some(error),
             Self::TrySlice(error) => Some(error),
             // TODO: Change to Some(error) when
@@ -354,6 +361,13 @@ impl From<ParseIntError> for SydError {
     }
 }
 
+// Conversions from ParseIntegerError to SydError.
+impl From<ParseIntegerError> for SydError {
+    fn from(err: ParseIntegerError) -> SydError {
+        Self::ParseInteger(err)
+    }
+}
+
 // Conversions from TryFromIntError to SydError.
 impl From<TryFromIntError> for SydError {
     fn from(err: TryFromIntError) -> SydError {
