@@ -11,7 +11,7 @@ use std::os::fd::AsRawFd;
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
-    fcntl::OFlag,
+    fcntl::{OFlag, ResolveFlag},
     sys::signal::{kill, Signal},
     unistd::Pid,
 };
@@ -20,7 +20,7 @@ use crate::{
     compat::fstatx,
     config::PROC_FILE,
     debug, error,
-    fs::{readlinkat, safe_open_magicsym, CanonicalPath, FileInfo, FsFlags},
+    fs::{readlinkat, safe_open_path_msym, CanonicalPath, FileInfo, FsFlags},
     hook::{RemoteProcess, SysArg, SysFlags, UNotifyEventRequest},
     kernel::{sandbox_path, syscall_path_handler},
     path::XPathBuf,
@@ -107,7 +107,20 @@ pub(crate) fn sysexit_chdir(
     // SAFETY: Validate /proc/$pid/cwd against TOCTTOU!
     let mut pfd = XPathBuf::from_pid(pid);
     pfd.push(b"cwd");
-    let cwd = safe_open_magicsym(PROC_FILE(), &pfd, OFlag::O_PATH | OFlag::O_DIRECTORY)?;
+    let cwd = match safe_open_path_msym(PROC_FILE(), &pfd, OFlag::O_DIRECTORY, ResolveFlag::empty())
+    {
+        Ok(cwd) => cwd,
+        Err(errno) => {
+            // SAFETY: Failed to open FD,
+            // assume TOCTTOU: terminate the process.
+            error!("ctx": "chdir", "op": "open_dir_fd",
+                "msg": format!("failed to open dir-fd for `{path}': {errno}"),
+                "err": errno as i32, "pid": pid.as_raw(), "path": &path,
+                "tip": "check with SYD_LOG=debug and/or submit a bug report");
+            let _ = kill(pid, Some(Signal::SIGKILL));
+            return Err(Errno::ESRCH);
+        }
+    };
 
     #[allow(clippy::disallowed_methods)]
     let fd = path.dir.as_ref().unwrap();
@@ -118,10 +131,9 @@ pub(crate) fn sysexit_chdir(
             // SAFETY: Failed to stat FD,
             // assume TOCTTOU: terminate the process.
             error!("ctx": "chdir", "op": "fstat_dir_fd",
-                "err": format!("failed to fstat dir-fd for `{path}': {errno}"),
-                "pid": pid.as_raw(),
-                "path": &path,
-                "errno": errno as i32);
+                "msg": format!("failed to fstat dir-fd for `{path}': {errno}"),
+                "err": errno as i32, "pid": pid.as_raw(), "path": &path,
+                "tip": "check with SYD_LOG=debug and/or submit a bug report");
             let _ = kill(pid, Some(Signal::SIGKILL));
             return Err(Errno::ESRCH);
         }
@@ -133,10 +145,9 @@ pub(crate) fn sysexit_chdir(
             // SAFETY: Failed to stat CWD,
             // assume TOCTTOU: terminate the process.
             error!("ctx": "chdir", "op": "stat_cwd_symlink",
-                "err": format!("failed to stat cwd-symlink for `{path}': {errno}"),
-                "pid": pid.as_raw(),
-                "path": &path,
-                "errno": errno as i32);
+                "msg": format!("failed to stat cwd-symlink for `{path}': {errno}"),
+                "err": errno as i32, "pid": pid.as_raw(), "path": &path,
+                "tip": "check with SYD_LOG=debug and/or submit a bug report");
             let _ = kill(pid, Some(Signal::SIGKILL));
             return Err(Errno::ESRCH);
         }
@@ -163,10 +174,8 @@ pub(crate) fn sysexit_chdir(
             .ok()
             .unwrap_or_else(|| XPathBuf::from("?"));
         error!("ctx": "chdir", "op": "dir_mismatch",
-            "err": format!("dir mismatch detected for directory `{path}' -> `{cwd}': assume TOCTTOU!"),
-            "pid": pid.as_raw(),
-            "path": &path,
-            "real": cwd,
+            "msg": format!("dir mismatch detected for directory `{path}' -> `{cwd}': assume TOCTTOU!"),
+            "pid": pid.as_raw(), "path": &path, "real": cwd,
             "cwd_mount_id": stx_cwd.stx_mnt_id,
             "dir_mount_id": stx_fd.stx_mnt_id,
             "cwd_inode": stx_cwd.stx_ino,
