@@ -6,7 +6,10 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::os::fd::{AsFd, AsRawFd};
+use std::{
+    os::fd::{AsFd, AsRawFd},
+    sync::LazyLock,
+};
 
 use libseccomp::{ScmpNotifResp, ScmpSyscall};
 use nix::{
@@ -15,7 +18,6 @@ use nix::{
     sys::stat::{fchmod, Mode},
     NixPath,
 };
-use once_cell::sync::Lazy;
 
 use crate::{
     config::PROC_FD,
@@ -29,7 +31,7 @@ use crate::{
 // Note fchmodat2 may not be available,
 // and libc::SYS_fchmodat2 may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_FCHMODAT2: Lazy<libc::c_long> = Lazy::new(|| {
+static SYS_FCHMODAT2: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("fchmodat2")
         .map(i32::from)
         .map(libc::c_long::from)
