@@ -6,21 +6,25 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::{
-    fs::File,
-    io::{Seek, SeekFrom},
-};
+use std::{fs::File, io::Seek, os::fd::AsRawFd};
 
 use libseccomp::ScmpNotifResp;
-use nix::{errno::Errno, fcntl::AtFlags, NixPath};
+use nix::{
+    errno::Errno,
+    fcntl::{AtFlags, OFlag, ResolveFlag},
+    sys::signal::{kill, Signal},
+    unistd::Pid,
+    NixPath,
+};
 
 use crate::{
-    config::{PTRACE_DATA_EXECVE, PTRACE_DATA_EXECVEAT},
+    config::{PROC_FILE, PTRACE_DATA_EXECVE, PTRACE_DATA_EXECVEAT},
     elf::{ElfError, ElfFileType, ElfType, ExecutableFile, LinkingType},
     err2no, error,
-    fs::{is_executable, set_nonblock, FileType, FsFlags, MaybeFd, AT_EXECVE_CHECK},
+    fs::{is_executable, safe_open_msym, FileType, FsFlags, MaybeFd, AT_EXECVE_CHECK},
     hook::{RemoteProcess, SysArg, SysFlags, UNotifyEventRequest},
     kernel::{sandbox_path, to_atflags},
+    path::XPathBuf,
     ptrace::ptrace_syscall_info,
     sandbox::{Action, Capability, IntegrityError, SandboxGuard},
     warn,
@@ -30,7 +34,7 @@ use crate::{
 // The seccomp hooks are only used with trace/allow_unsafe_ptrace:1.
 #[allow(clippy::cognitive_complexity)]
 pub(crate) fn sysenter_exec(
-    process: &RemoteProcess,
+    pid: Pid,
     sandbox: &SandboxGuard,
     info: ptrace_syscall_info,
 ) -> Result<(File, ExecutableFile), Errno> {
@@ -46,7 +50,7 @@ pub(crate) fn sysenter_exec(
             "execve",
             SysArg {
                 path: Some(0),
-                fsflags: FsFlags::MUST_PATH | FsFlags::WANT_READ,
+                fsflags: FsFlags::MUST_PATH,
                 ..Default::default()
             },
             false,
@@ -58,7 +62,7 @@ pub(crate) fn sysenter_exec(
                 AtFlags::AT_SYMLINK_NOFOLLOW | AtFlags::AT_EMPTY_PATH | AT_EXECVE_CHECK,
             )?;
 
-            let mut fsflags = FsFlags::MUST_PATH | FsFlags::WANT_READ;
+            let mut fsflags = FsFlags::MUST_PATH;
             if flags.contains(AtFlags::AT_SYMLINK_NOFOLLOW) {
                 fsflags.insert(FsFlags::NO_FOLLOW_LAST);
             }
@@ -84,12 +88,11 @@ pub(crate) fn sysenter_exec(
     };
 
     // Read remote path.
+    let process = RemoteProcess::new(pid);
+
+    // SAFETY: This is a ptrace hook, the PID cannot be validated.
     let (mut path, _, _, empty_path) =
-        // SAFETY: We will validate the PidFd afterwards.
         unsafe { process.read_path(sandbox, data.args, arg, false, None) }?;
-    if !process.is_alive() {
-        return Err(Errno::ESRCH);
-    }
 
     // Call sandbox access checker if Exec sandboxing is on.
     let caps = sandbox.getcaps(Capability::CAP_EXEC | Capability::CAP_TPE);
@@ -99,7 +102,7 @@ pub(crate) fn sysenter_exec(
         sandbox_path(
             None,
             sandbox,
-            process,
+            pid,
             path.abs(),
             Capability::CAP_EXEC,
             hide,
@@ -123,10 +126,14 @@ pub(crate) fn sysenter_exec(
     // SAFETY: Return EACCES without any more processing
     // if the file is not a regular file or a memory fd.
     // Mfd check depends on trace/allow_unsafe_memfd option.
+    //
+    // Note, attempting to execute directories on Linux
+    // return EACCES, not EISDIR like the manual page
+    // claims. GNU make has a test checking this errno.
     match path.typ.as_ref() {
         Some(FileType::Reg) => {}
-        Some(FileType::Mfd) if sandbox.allow_unsafe_memfd() => {}
-        Some(FileType::Dir) => return Err(Errno::EISDIR),
+        Some(FileType::Mfd) if sandbox.flags.allow_unsafe_memfd() => {}
+        //Some(FileType::Dir) => return Err(Errno::EISDIR),
         _ => return Err(Errno::EACCES),
     };
 
@@ -145,10 +152,10 @@ pub(crate) fn sysenter_exec(
         if action != Action::Filter {
             let (_, bin) = path.abs().split();
             error!("ctx": "segvguard",
-                "err": format!("max crashes {} exceeded, execution of `{bin}' denied",
+                "msg": format!("max crashes {} exceeded, execution of `{bin}' denied",
                     sandbox.segvguard_maxcrashes),
                 "tip": "increase `segvguard/maxcrashes'",
-                "pid": process.pid.as_raw(), "path": path.abs());
+                "pid": pid.as_raw(), "path": path.abs());
         }
 
         match action {
@@ -157,15 +164,15 @@ pub(crate) fn sysenter_exec(
             Action::Panic => panic!(),
             Action::Exit => std::process::exit(libc::EACCES),
             Action::Stop => {
-                let _ = process.pidfd_kill(libc::SIGSTOP);
+                let _ = kill(process.pid, Some(Signal::SIGSTOP));
                 return Err(Errno::EACCES);
             }
             Action::Abort => {
-                let _ = process.pidfd_kill(libc::SIGABRT);
+                let _ = kill(process.pid, Some(Signal::SIGABRT));
                 return Err(Errno::EACCES);
             }
             Action::Kill => {
-                let _ = process.pidfd_kill(libc::SIGKILL);
+                let _ = kill(process.pid, Some(Signal::SIGKILL));
                 return Err(Errno::EACCES);
             }
         }
@@ -173,12 +180,16 @@ pub(crate) fn sysenter_exec(
 
     // Trusted Path Execution.
     if caps.contains(Capability::CAP_TPE) {
-        let action = sandbox.check_tpe(path.abs());
+        // MUST_PATH ensures path.dir is Some.
+        #[allow(clippy::disallowed_methods)]
+        let file = path.dir.as_ref().unwrap();
+        let (action, msg) = sandbox.check_tpe(file, path.abs());
         if !matches!(action, Action::Allow | Action::Filter) {
             // TODO: Fix proc_mmap to work in ptrace hooks.
+            let msg = msg.as_deref().unwrap_or("?");
             error!("ctx": "trusted_path_execution",
-                "err": "exec from untrusted path blocked",
-                "pid": process.pid.as_raw(), "path": path.abs(),
+                "msg": format!("exec from untrusted path blocked: {msg}"),
+                "pid": pid.as_raw(), "path": path.abs(),
                 "sys": syscall_name, "arch": info.arch, "args": data.args,
                 "tip": "move the binary to a safe location or use `sandbox/tpe:off'");
         }
@@ -188,87 +199,71 @@ pub(crate) fn sysenter_exec(
             Action::Panic => panic!(),
             Action::Exit => std::process::exit(libc::EACCES),
             Action::Stop => {
-                let _ = process.pidfd_kill(libc::SIGSTOP);
+                let _ = kill(pid, Some(Signal::SIGSTOP));
                 return Err(Errno::EACCES);
             }
             Action::Abort => {
-                let _ = process.pidfd_kill(libc::SIGABRT);
+                let _ = kill(pid, Some(Signal::SIGABRT));
                 return Err(Errno::EACCES);
             }
             Action::Kill => {
-                let _ = process.pidfd_kill(libc::SIGKILL);
+                let _ = kill(pid, Some(Signal::SIGKILL));
                 return Err(Errno::EACCES);
             }
         }
     }
 
+    // Open a new instance of the file for read.
+    // We prefer this over WANT_READ to ensure
+    // we do not share OFD with the sandbox process.
     assert!(
         path.base.is_empty(),
         "BUG: MUST_PATH returned a directory for exec, report a bug!"
     );
     let mut file = match path.dir.take() {
         Some(MaybeFd::Owned(fd)) => {
-            set_nonblock(&fd, false)?;
-            File::from(fd)
+            let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
+            safe_open_msym(PROC_FILE(), &pfd, OFlag::O_RDONLY, ResolveFlag::empty())
+                .map(File::from)?
         }
         _ => return Err(Errno::ENOEXEC),
     };
 
     // Parse ELF as necessary for restrictions.
-    let deny_script = sandbox.deny_script();
-    let restrict_32 = sandbox.deny_elf32();
-    let restrict_dyn = sandbox.deny_elf_dynamic();
-    let restrict_sta = sandbox.deny_elf_static();
-    let restrict_pie = !sandbox.allow_unsafe_nopie();
-    let restrict_xs = !sandbox.allow_unsafe_stack();
-
-    // Shared library execution depends on trace/allow_unsafe_exec:1.
-    let restrict_ldd = !sandbox.allow_unsafe_exec();
+    let deny_script = sandbox.flags.deny_script();
+    let restrict_32 = sandbox.flags.deny_elf32();
+    let restrict_dyn = sandbox.flags.deny_elf_dynamic();
+    let restrict_sta = sandbox.flags.deny_elf_static();
+    let restrict_ldd = !sandbox.flags.allow_unsafe_ldso();
+    let restrict_pie = !sandbox.flags.allow_unsafe_nopie();
+    let restrict_xs = !sandbox.flags.allow_unsafe_stack();
 
     let check_linking = restrict_ldd || restrict_dyn || restrict_sta || restrict_pie || restrict_xs;
 
-    // Ensure the file offset is maintained,
-    // as the file might be sharing the OFD
-    // with the sandbox process.
-    //
-    // SAFETY: Deny non-seekable files such as FIFOs with EACCES.
-    let offset = file.stream_position().or(Err(Errno::EACCES))?;
-
-    let result = (|| -> Result<ExecutableFile, ElfError> {
-        // Parse ELF and reset the file offset.
-        if offset != 0 {
-            file.rewind().map_err(ElfError::IoError)?;
-        }
-        let result = ExecutableFile::parse(&file, check_linking);
-        file.seek(SeekFrom::Start(offset))
-            .map_err(ElfError::IoError)?;
-        result
-    })();
-
-    let exe = match result {
+    let exe = match ExecutableFile::parse(&file, check_linking) {
         Ok(exe) => exe,
         Err(ElfError::IoError(err)) => {
             let errno = err2no(&err);
             if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
                 error!("ctx": "parse_elf",
-                    "err": format!("io error: {errno}"),
-                    "pid": process.pid.as_raw(), "path": path.abs());
+                    "msg": format!("io error: {errno}"),
+                    "pid": pid.as_raw(), "path": path.abs());
             }
             return Err(errno);
         }
         Err(ElfError::BadMagic) => {
             if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
                 error!("ctx": "parse_elf",
-                    "err": "invalid ELF file",
-                    "pid": process.pid.as_raw(), "path": path.abs());
+                    "msg": "invalid ELF file",
+                    "pid": pid.as_raw(), "path": path.abs());
             }
             return Err(Errno::ENOEXEC);
         }
         Err(ElfError::Malformed) => {
             if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
                 error!("ctx": "parse_elf",
-                    "err": "malformed ELF file",
-                    "pid": process.pid.as_raw(), "path": path.abs());
+                    "msg": "malformed ELF file",
+                    "pid": pid.as_raw(), "path": path.abs());
             }
             return Err(Errno::ENOEXEC);
         }
@@ -278,8 +273,8 @@ pub(crate) fn sysenter_exec(
     if is_script && deny_script {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
             error!("ctx": "deny_script",
-                "err": "script execution denied",
-                "pid": process.pid.as_raw(), "path": path.abs(),
+                "msg": "script execution denied",
+                "pid": pid.as_raw(), "path": path.abs(),
                 "exe": format!("{exe}"));
         }
         return Err(Errno::EACCES);
@@ -297,8 +292,9 @@ pub(crate) fn sysenter_exec(
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
             error!("ctx": "check_elf",
-                "err": "ld.so exec-indirection",
-                "pid": process.pid.as_raw(), "path": path.abs(),
+                "msg": "ld.so(8) exec-indirection prevented",
+                "pid": pid.as_raw(), "path": path.abs(),
+                "tip": "configure `trace/allow_unsafe_ldso:1'",
                 "exe": format!("{exe}"));
         }
         return Err(Errno::EACCES);
@@ -306,8 +302,8 @@ pub(crate) fn sysenter_exec(
 
     if !is_script && restrict_pie && matches!(exe, ExecutableFile::Elf { pie: false, .. }) {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "err": "not PIE",
-                "pid": process.pid.as_raw(), "path": path.abs(),
+            error!("ctx": "check_elf", "msg": "ELF is not Position Independent Executable (PIE)",
+                "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/allow_unsafe_nopie:1'",
                 "exe": format!("{exe}"));
         }
@@ -316,8 +312,8 @@ pub(crate) fn sysenter_exec(
 
     if !is_script && restrict_xs && matches!(exe, ExecutableFile::Elf { xs: true, .. }) {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "err": "execstack",
-                "pid": process.pid.as_raw(), "path": path.abs(),
+            error!("ctx": "check_elf", "msg": "ELF has executable stack",
+                "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/allow_unsafe_stack:1'",
                 "exe": format!("{exe}"));
         }
@@ -335,8 +331,8 @@ pub(crate) fn sysenter_exec(
         )
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "err": "32-bit",
-                "pid": process.pid.as_raw(), "path": path.abs(),
+            error!("ctx": "check_elf", "msg": "32-bit ELF execution prevented",
+                "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/deny_elf32:0'",
                 "exe": format!("{exe}"));
         }
@@ -354,8 +350,8 @@ pub(crate) fn sysenter_exec(
         )
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "err": "dynamic-link",
-                "pid": process.pid.as_raw(), "path": path.abs(),
+            error!("ctx": "check_elf", "msg": "dynamic linked ELF execution prevented",
+                "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/deny_elf_dynamic:0'",
                 "exe": format!("{exe}"));
         }
@@ -373,8 +369,8 @@ pub(crate) fn sysenter_exec(
         )
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "err": "static-link",
-                "pid": process.pid.as_raw(), "path": path,
+            error!("ctx": "check_elf", "msg": "static linked ELF execution prevented",
+                "pid": pid.as_raw(), "path": path,
                 "tip": "configure `trace/deny_elf_static:0'",
                 "exe": format!("{exe}"));
         }
@@ -383,59 +379,55 @@ pub(crate) fn sysenter_exec(
 
     // Check for Force sandboxing.
     if sandbox.enabled(Capability::CAP_FORCE) {
-        // Calculate checksum and reset the file offset.
-        if offset != 0 {
-            file.rewind().or(Err(Errno::EACCES))?;
-        }
-        let result = sandbox.check_force2(path.abs(), &mut file);
-        file.seek(SeekFrom::Start(offset)).or(Err(Errno::EACCES))?;
+        // Reset the file offset and calculate checksum.
+        file.rewind().or(Err(Errno::EACCES))?;
 
-        match result {
+        match sandbox.check_force2(path.abs(), &mut file) {
             Ok(Action::Allow) => {}
             Ok(Action::Warn) => {
                 warn!("ctx": "verify_elf", "act": Action::Warn,
-                    "pid": process.pid.as_raw(), "path": path.abs(),
+                    "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
             }
             Ok(Action::Filter) => return Err(Errno::EACCES),
             Ok(Action::Deny) => {
                 warn!("ctx": "verify_elf", "act": Action::Deny,
-                    "pid": process.pid.as_raw(), "path": path.abs(),
+                    "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 return Err(Errno::EACCES);
             }
             Ok(Action::Stop) => {
                 warn!("ctx": "verify_elf", "act": Action::Stop,
-                    "pid": process.pid.as_raw(), "path": path.abs(),
+                    "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
-                let _ = process.pidfd_kill(libc::SIGSTOP);
+                let _ = kill(pid, Some(Signal::SIGSTOP));
                 return Err(Errno::EACCES);
             }
             Ok(Action::Abort) => {
                 warn!("ctx": "verify_elf", "act": Action::Abort,
-                    "pid": process.pid.as_raw(), "path": path.abs(),
+                    "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
-                let _ = process.pidfd_kill(libc::SIGABRT);
+                let _ = kill(pid, Some(Signal::SIGABRT));
                 return Err(Errno::EACCES);
             }
             Ok(Action::Kill) => {
                 warn!("ctx": "verify_elf", "act": Action::Kill,
-                    "pid": process.pid.as_raw(), "path": path.abs(),
+                    "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
-                let _ = process.pidfd_kill(libc::SIGKILL);
+                let _ = kill(pid, Some(Signal::SIGKILL));
                 return Err(Errno::EACCES);
             }
             Ok(Action::Exit) => {
                 error!("ctx": "verify_elf", "act": Action::Exit,
-                    "pid": process.pid.as_raw(), "path": path.abs(),
+                    "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 std::process::exit(libc::EACCES);
             }
             Ok(Action::Panic) => panic!(),
             Err(IntegrityError::Sys(errno)) => {
                 error!("ctx": "verify_elf",
-                    "err": format!("system error during ELF checksum calculation: {errno}"),
-                    "pid": process.pid.as_raw(), "path": path.abs(),
+                    "msg": format!("system error during ELF checksum calculation: {errno}"),
+                    "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 return Err(Errno::EACCES);
             }
@@ -446,19 +438,25 @@ pub(crate) fn sysenter_exec(
             }) => {
                 if !matches!(action, Action::Allow | Action::Filter) {
                     error!("ctx": "verify_elf", "act": action,
-                        "err": format!("ELF checksum mismatch: {found} is not {expected}"),
-                        "pid": process.pid.as_raw(), "path": path.abs(),
+                        "msg": format!("ELF checksum mismatch: {found} is not {expected}"),
+                        "pid": pid.as_raw(), "path": path.abs(),
                         "tip": format!("configure `force+{path}:<checksum>'"));
                 }
                 match action {
                     Action::Allow | Action::Warn => {}
                     Action::Filter | Action::Deny => return Err(Errno::EACCES),
                     Action::Stop | Action::Abort | Action::Kill => {
-                        let _ = process.pidfd_kill(
-                            action
-                                .signal()
-                                .map(|sig| sig as i32)
-                                .unwrap_or(libc::SIGKILL),
+                        let _ = kill(
+                            pid,
+                            Some(
+                                Signal::try_from(
+                                    action
+                                        .signal()
+                                        .map(|sig| sig as i32)
+                                        .unwrap_or(libc::SIGKILL),
+                                )
+                                .unwrap_or(Signal::SIGKILL),
+                            ),
                         );
                         return Err(Errno::EACCES);
                     }
@@ -538,7 +536,7 @@ fn syscall_exec_handler(
         // and only the file executability check (aka AT_EXECVE_CHECK)
         // will be performed.
         let sandbox = request.get_sandbox();
-        let restrict_memfd = !sandbox.allow_unsafe_memfd();
+        let restrict_memfd = !sandbox.flags.allow_unsafe_memfd();
 
         // Read remote path.
         let (path, _) = request.read_path(&sandbox, arg, false)?;
@@ -548,7 +546,7 @@ fn syscall_exec_handler(
             sandbox_path(
                 Some(&request),
                 &sandbox,
-                &RemoteProcess::new(request.scmpreq.pid()), // Unused when request.is_some()
+                request.scmpreq.pid(), // Unused when request.is_some()
                 path.abs(),
                 Capability::CAP_EXEC,
                 sandbox.enabled(Capability::CAP_STAT),
@@ -573,10 +571,14 @@ fn syscall_exec_handler(
         // SAFETY: Return EACCES without any more processing
         // if the file is not a regular file or a memory fd.
         // Mfd check depends on trace/allow_unsafe_memfd option.
+        //
+        // Note, attempting to execute directories on Linux
+        // return EACCES, not EISDIR like the manual page
+        // claims. GNU make has a test checking this errno.
         match path.typ.as_ref() {
             Some(FileType::Reg) => {}
             Some(FileType::Mfd) if !restrict_memfd => {}
-            Some(FileType::Dir) => return Err(Errno::EISDIR),
+            //Some(FileType::Dir) => return Err(Errno::EISDIR),
             _ => return Err(Errno::EACCES),
         };
 
