@@ -14,14 +14,17 @@ use std::{
 };
 
 use libseccomp::ScmpNotifResp;
-use nix::{errno::Errno, fcntl::OFlag};
+use nix::{
+    errno::Errno,
+    fcntl::{OFlag, ResolveFlag},
+};
 use serde::{Serialize, Serializer};
 
 use crate::{
     config::{PAGE_SIZE, PROC_FILE},
     elf::ExecutableFile,
     error,
-    fs::{fd_status_flags, safe_open_magicsym, CanonicalPath},
+    fs::{fd_status_flags, safe_open_msym, CanonicalPath},
     hook::UNotifyEventRequest,
     kernel::sandbox_path,
     path::XPathBuf,
@@ -229,13 +232,13 @@ fn syscall_mem_handler(request: UNotifyEventRequest, syscall: MemSyscall) -> Scm
                 let msg = msg.as_deref().unwrap_or("?");
                 if verbose {
                     error!("ctx": "trusted_path_execution",
-                        "err": format!("library load from untrusted path blocked: {msg}"),
+                        "msg": format!("library load from untrusted path blocked: {msg}"),
                         "sys": &name, "path": &path,
                         "req": &request,
                         "tip": "move the library to a safe location or use `sandbox/tpe:off'");
                 } else {
                     error!("ctx": "trusted_path_execution",
-                        "err": format!("library load from untrusted path blocked: {msg}"),
+                        "msg": format!("library load from untrusted path blocked: {msg}"),
                         "sys": &name, "path": &path,
                         "pid": request.scmpreq.pid,
                         "tip": "move the library to a safe location or use `sandbox/tpe:off'");
@@ -264,7 +267,9 @@ fn syscall_mem_handler(request: UNotifyEventRequest, syscall: MemSyscall) -> Scm
             let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
 
             let mut file =
-                match safe_open_magicsym(PROC_FILE(), &pfd, OFlag::O_RDONLY).map(File::from) {
+                match safe_open_msym(PROC_FILE(), &pfd, OFlag::O_RDONLY, ResolveFlag::empty())
+                    .map(File::from)
+                {
                     Ok(file) => file,
                     Err(_) => {
                         return request.fail_syscall(Errno::EBADF);
@@ -284,14 +289,14 @@ fn syscall_mem_handler(request: UNotifyEventRequest, syscall: MemSyscall) -> Scm
                         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
                             if verbose {
                                 error!("ctx": "check_lib",
-                                    "err": "library load with executable stack blocked",
+                                    "msg": "library load with executable stack blocked",
                                     "sys": &name, "path": path.abs(),
                                     "tip": "configure `trace/allow_unsafe_stack:1'",
                                     "lib": format!("{exe}"),
                                     "req": &request);
                             } else {
                                 error!("ctx": "check_lib",
-                                    "err": "library load with executable stack blocked",
+                                    "msg": "library load with executable stack blocked",
                                     "sys": &name, "path": path.abs(),
                                     "tip": "configure `trace/allow_unsafe_stack:1'",
                                     "lib": format!("{exe}"),
@@ -347,13 +352,13 @@ fn syscall_mem_handler(request: UNotifyEventRequest, syscall: MemSyscall) -> Scm
                     Err(IntegrityError::Sys(errno)) => {
                         if verbose {
                             error!("ctx": "verify_lib",
-                                "err": format!("system error during library checksum calculation: {errno}"),
+                                "msg": format!("system error during library checksum calculation: {errno}"),
                                 "sys": &name, "path": path.abs(),
                                 "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                 "req": &request);
                         } else {
                             error!("ctx": "verify_lib",
-                                "err": format!("system error during library checksum calculation: {errno}"),
+                                "msg": format!("system error during library checksum calculation: {errno}"),
                                 "sys": &name, "path": path.abs(),
                                 "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                 "pid": request.scmpreq.pid);
@@ -368,13 +373,13 @@ fn syscall_mem_handler(request: UNotifyEventRequest, syscall: MemSyscall) -> Scm
                         if action != Action::Filter {
                             if sandbox.verbose {
                                 error!("ctx": "verify_lib", "act": action,
-                                    "err": format!("library checksum mismatch: {found} is not {expected}"),
+                                    "msg": format!("library checksum mismatch: {found} is not {expected}"),
                                     "sys": &name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                     "req": &request);
                             } else {
                                 error!("ctx": "verify_lib", "act": action,
-                                    "err": format!("library checksum mismatch: {found} is not {expected}"),
+                                    "msg": format!("library checksum mismatch: {found} is not {expected}"),
                                     "sys": &name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
                                     "pid": request.scmpreq.pid);
