@@ -93,9 +93,6 @@ pub(crate) mod shm;
 /// {,rt_}sigaction(2) handler
 pub(crate) mod sigaction;
 
-/// {,rt_}sigreturn(2) handler
-pub(crate) mod sigreturn;
-
 /// Signal syscall handlers
 pub(crate) mod signal;
 
@@ -136,15 +133,20 @@ use memchr::memmem;
 use nix::{
     errno::Errno,
     fcntl::AtFlags,
-    sys::stat::{Mode, SFlag},
+    sys::{
+        signal::{kill, Signal},
+        stat::{Mode, SFlag},
+    },
+    unistd::Pid,
 };
 
 use crate::{
     fs::{to_valid_fd, CanonicalPath, FileInfo, FileType},
-    hook::{PathArgs, RemoteProcess, SysArg, UNotifyEventRequest},
+    hook::{PathArgs, SysArg, UNotifyEventRequest},
+    ioctl::{ioctl_names_get, Ioctl},
     notice,
     path::{XPath, XPathBuf},
-    sandbox::{Action, Capability, SandboxGuard},
+    sandbox::{Action, Capability, Sandbox, SandboxGuard},
     warn,
 };
 
@@ -153,8 +155,8 @@ use crate::{
 #[allow(clippy::too_many_arguments)]
 pub(crate) fn sandbox_path(
     request: Option<&UNotifyEventRequest>,
-    sandbox: &SandboxGuard,
-    process: &RemoteProcess,
+    sandbox: &Sandbox,
+    pid: Pid,
     path: &XPath,
     caps: Capability,
     hide: bool,
@@ -170,7 +172,7 @@ pub(crate) fn sandbox_path(
     let mut caps = sandbox.getcaps(caps);
     let stat = sandbox.enabled(Capability::CAP_STAT);
     if caps.is_empty() && (!hide || !stat) {
-        return if caps_old.intersects(Capability::CAP_WRSET) && sandbox.is_append(path) {
+        return if caps_old.can_write() && sandbox.is_append(path) {
             // SAFETY: Protect append-only paths against writes.
             // We use ECANCELED which will result in a no-op.
             Err(Errno::ECANCELED)
@@ -182,7 +184,7 @@ pub(crate) fn sandbox_path(
     // Convert /proc/${pid} to /proc/self as necessary.
     let path = if let Some(p) = path.split_prefix(b"/proc") {
         let mut buf = itoa::Buffer::new();
-        let pid = buf.format(process.pid.as_raw());
+        let pid = buf.format(pid.as_raw());
         if let Some(p) = p.split_prefix(pid.as_bytes()) {
             let mut pdir = XPathBuf::from("/proc/self");
             pdir.push(p.as_bytes());
@@ -196,17 +198,20 @@ pub(crate) fn sandbox_path(
 
     let mut action = Action::Allow;
     let mut filter = false;
-    let mut deny_errno = Errno::EACCES;
+    let check_caps = caps & Capability::CAP_PATH;
+    let mut deny_errno = if check_caps == Capability::CAP_WALK {
+        Errno::ENOENT
+    } else {
+        Errno::EACCES
+    };
 
     // Sandboxing.
-    for cap in caps & Capability::CAP_PATH {
+    for cap in check_caps {
         let (new_action, new_filter) = sandbox.check_path(cap, &path);
 
         if new_action >= action {
             action = new_action;
-        }
-        if !filter && new_filter {
-            filter = true;
+            filter = new_filter;
         }
     }
 
@@ -237,29 +242,43 @@ pub(crate) fn sandbox_path(
 
     if !filter && action >= Action::Warn {
         // Log warn for normal cases.
-        // Log info for path hiding unless explicitly specified to warn.
-        let is_warn = if caps != Capability::CAP_STAT {
-            true
-        } else {
-            !matches!(
+        // Log info for path hiding/walking unless explicitly specified to warn.
+        let is_warn = match caps {
+            Capability::CAP_STAT => !matches!(
                 sandbox.default_action(Capability::CAP_STAT),
                 Action::Filter | Action::Deny
-            )
+            ),
+            Capability::CAP_WALK => !matches!(
+                sandbox.default_action(Capability::CAP_WALK),
+                Action::Filter | Action::Deny
+            ),
+            _ => true,
         };
 
         if let Some(request) = request {
+            let arch = request.scmpreq.data.arch;
             let args = request.scmpreq.data.args;
+
+            // Enrich log data with ioctl(2) request names.
+            let ctl = if caps.contains(Capability::CAP_IOCTL) {
+                ioctl_names_get(args[1] as Ioctl, arch)
+            } else {
+                None
+            };
+
             if sandbox.verbose {
                 if is_warn {
                     warn!("ctx": "access", "cap": caps, "act": action,
-                        "sys": syscall_name, "path": &path, "args": args,
+                        "sys": syscall_name, "ctl": ctl,
+                        "path": &path, "args": args,
                         "tip": format!("configure `allow/{}+{}'",
                             caps.to_string().to_ascii_lowercase(),
                             path),
                         "req": request);
                 } else {
                     notice!("ctx": "access", "cap": caps, "act": action,
-                        "sys": syscall_name, "path": &path, "args": args,
+                        "sys": syscall_name, "ctl": ctl,
+                        "path": &path, "args": args,
                         "tip": format!("configure `allow/{}+{}'",
                             caps.to_string().to_ascii_lowercase(),
                             path),
@@ -267,14 +286,16 @@ pub(crate) fn sandbox_path(
                 }
             } else if is_warn {
                 warn!("ctx": "access", "cap": caps, "act": action,
-                    "sys": syscall_name, "path": &path, "args": args,
+                    "sys": syscall_name, "ctl": ctl,
+                    "path": &path, "args": args,
                     "tip": format!("configure `allow/{}+{}'",
                         caps.to_string().to_ascii_lowercase(),
                         path),
                     "pid": request.scmpreq.pid);
             } else {
                 notice!("ctx": "access", "cap": caps, "act": action,
-                    "sys": syscall_name, "path": &path, "args": args,
+                    "sys": syscall_name, "ctl": ctl,
+                    "path": &path, "args": args,
                     "tip": format!("configure `allow/{}+{}'",
                         caps.to_string().to_ascii_lowercase(),
                         path),
@@ -286,20 +307,20 @@ pub(crate) fn sandbox_path(
                 "tip": format!("configure `allow/{}+{}'",
                     caps.to_string().to_ascii_lowercase(),
                     path),
-                "pid": process.pid.as_raw());
+                "pid": pid.as_raw());
         } else {
             notice!("ctx": "access", "cap": caps, "act": action,
                 "sys": syscall_name, "path": &path,
                 "tip": format!("configure `allow/{}+{}'",
                     caps.to_string().to_ascii_lowercase(),
                     path),
-                "pid": process.pid.as_raw());
+                "pid": pid.as_raw());
         }
     }
 
     match action {
         Action::Allow | Action::Warn => {
-            if caps.intersects(Capability::CAP_WRSET) && sandbox.is_append(&path) {
+            if caps.can_write() && sandbox.is_append(&path) {
                 // SAFETY: Protect append-only paths against writes.
                 // We use ECANCELED which will result in a no-op.
                 Err(Errno::ECANCELED)
@@ -314,7 +335,7 @@ pub(crate) fn sandbox_path(
             if let Some(request) = request {
                 let _ = request.pidfd_kill(libc::SIGSTOP);
             } else {
-                let _ = process.pidfd_kill(libc::SIGSTOP);
+                let _ = kill(pid, Some(Signal::SIGSTOP));
             }
             Err(deny_errno)
         }
@@ -322,7 +343,7 @@ pub(crate) fn sandbox_path(
             if let Some(request) = request {
                 let _ = request.pidfd_kill(libc::SIGABRT);
             } else {
-                let _ = process.pidfd_kill(libc::SIGABRT);
+                let _ = kill(pid, Some(Signal::SIGABRT));
             }
             Err(deny_errno)
         }
@@ -330,7 +351,7 @@ pub(crate) fn sandbox_path(
             if let Some(request) = request {
                 let _ = request.pidfd_kill(libc::SIGKILL);
             } else {
-                let _ = process.pidfd_kill(libc::SIGKILL);
+                let _ = kill(pid, Some(Signal::SIGKILL));
             }
             Err(deny_errno)
         }
@@ -430,7 +451,7 @@ where
                             #[allow(clippy::disallowed_methods)]
                             let files = request.crypt_map.as_ref().unwrap();
                             for (path, map) in
-                                &files.read().unwrap_or_else(|err| err.into_inner()).0
+                                files.read().unwrap_or_else(|err| err.into_inner()).iter()
                             {
                                 if info == map.info {
                                     found = Some(path.clone());
@@ -449,13 +470,12 @@ where
                         // SAFETY: Only regular files are encrypted.
                         CanonicalPath::new(path, FileType::Reg, arg.fsflags)?
                     } else {
-                        CanonicalPath::new_fd(fd.into(), req.pid(), dirfd)?
+                        CanonicalPath::new_fd(fd.into(), req.pid())?
                     };
 
                     paths[idx] = Some(path);
                 } else {
-                    let path =
-                        CanonicalPath::new_fd(libc::AT_FDCWD.into(), req.pid(), libc::AT_FDCWD)?;
+                    let path = CanonicalPath::new_fd(libc::AT_FDCWD.into(), req.pid())?;
 
                     paths[idx] = Some(path);
                 }
@@ -465,9 +485,6 @@ where
         }
 
         if !magic {
-            // Unused when request.is_some()
-            let process = RemoteProcess::new(request.scmpreq.pid());
-
             // Call sandbox access checker, skip magic paths.
             match (&paths[0], &paths[1]) {
                 (Some(path), None) => {
@@ -488,7 +505,7 @@ where
                     sandbox_path(
                         Some(&request),
                         &sandbox,
-                        &process,
+                        request.scmpreq.pid(), // Unused when request.is_some()
                         path.abs(),
                         caps,
                         hide,
@@ -503,7 +520,7 @@ where
                     sandbox_path(
                         Some(&request),
                         &sandbox,
-                        &process,
+                        request.scmpreq.pid(), // Unused when request.is_some()
                         path_0.abs(),
                         Capability::CAP_RENAME,
                         hide,
@@ -515,7 +532,7 @@ where
                         sandbox_path(
                             Some(&request),
                             &sandbox,
-                            &process,
+                            request.scmpreq.pid(), // Unused when request.is_some()
                             path_1.abs(),
                             Capability::CAP_CREATE,
                             hide,
@@ -544,7 +561,7 @@ where
                 }
 
                 if let Some(file_type) = &path.typ {
-                    if !matches!(file_type, FileType::Dir | FileType::MagicLnk(_, _))
+                    if !matches!(file_type, FileType::Dir | FileType::MagicLnk(_))
                         && path.abs().last() == Some(b'/')
                     {
                         return Err(Errno::ENOTDIR);
