@@ -17,6 +17,7 @@ use std::{
     },
 };
 
+use ipnet::IpNet;
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
@@ -26,7 +27,7 @@ use nix::{
 };
 
 use crate::{
-    compat::{addr_family, PF_ALG, PF_INET, PF_INET6, PF_NETLINK, PF_UNIX, PF_UNSPEC},
+    compat::{addr_family, PF_ALG, PF_INET, PF_INET6, PF_MAX, PF_NETLINK, PF_UNIX, PF_UNSPEC},
     confine::{op2errno, op2name, scmp_arch_bits},
     fs::{fd_status_flags, file_type, safe_canonicalize, CanonicalPath, FileType, FsFlags},
     hook::UNotifyEventRequest,
@@ -40,7 +41,7 @@ use crate::{
         socket::handle_socket,
     },
     path::{XPath, XPathBuf},
-    sandbox::{Action, Capability, SandboxGuard},
+    sandbox::{Action, AddressPattern, Capability, CidrRule, SandboxGuard},
     warn,
 };
 
@@ -165,17 +166,7 @@ pub(crate) fn sys_sendmmsg(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_network_handler(request, &req.data.args, 0x14)
 }
 
-/// A helper function to handle network-related syscalls.
-///
-/// This function abstracts the common logic involved in handling network syscalls such as `bind`,
-/// `connect`, `and `sendto` in a seccomp-based sandboxing environment. It reduces code duplication
-/// across different syscall handler functions.
-///
-/// # Returns
-///
-/// Returns `ScmpNotifResp` indicating the result of the syscall handling:
-/// - If successful, it contains a continued syscall.
-/// - If an error occurs, it contains a failed syscall with an `EACCES` error code.
+// A helper function to handle network-related syscalls.
 #[allow(clippy::cognitive_complexity)]
 fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
@@ -188,16 +179,23 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
 
         let cap = match op {
             0x1 => {
-                // a. socket(2) system call.
-                // b. socketcall(2) -> socket(2) indirection.
+                // a. socket system call.
+                // b. socketcall -> socket indirection.
                 let flags = *sandbox.flags;
                 let nlfam = sandbox.netlink_families;
                 drop(sandbox); // drop read-lock before emulation.
 
                 return handle_socket(&request, args, flags, nlfam);
             }
-            0x2 | 0x6 => Capability::CAP_NET_BIND,
-            _ => Capability::CAP_NET_CONNECT,
+            0x5 | 0x6 | 0x12 => {
+                // accept, accept4, getsockname:
+                //
+                // accept{,4} are IP blocklist only.
+                // getsockname is informational.
+                Capability::empty()
+            }
+            0x2 /* bind */ => Capability::CAP_NET_BIND,
+            _ /* connect, send{to,{m,}msg} */ => Capability::CAP_NET_CONNECT,
         };
         drop(sandbox); // release the read-lock before get-fd.
 
@@ -221,7 +219,8 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
 
         match op {
             0x5 | 0x12 => {
-                // accept{,4} uses a different data structure, so we handle it in its own branch.
+                // accept{,4} uses a different data structure,
+                // so we handle it in its own branch.
                 return handle_accept(fd, &request, args, op);
             }
             0x6 => {
@@ -294,14 +293,15 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                 // SAFETY: We do not check Netlink address for access.
                 // We apply filtering on netlink families at socket level.
             }
-            PF_ALG if allow_safe_kcapi && op == 0x2 => {
+            PF_ALG if allow_safe_kcapi && op == 0x2 /*bind*/ => {
                 // SAFETY: Admin requested access to KCAPI.
             }
             PF_ALG => {
                 // a. SAFETY: Access to KCAPI is disabled by default.
-                // b. Non-bind() call is not supported for AF_ALG socket.
+                // b. Non-bind(2) call is not supported for AF_ALG socket.
                 return Err(Errno::EOPNOTSUPP);
             }
+            n if n >= PF_MAX => return Err(Errno::EINVAL),
             _ if allow_unsupp_socket => {
                 // SAFETY: This is also restricted at socket(2) boundary
                 // where there is no pointer-dereference in access
@@ -378,10 +378,10 @@ fn canon_addr<'a>(
         let null = memchr::memchr(0, path).unwrap_or(path.len());
         let path = XPathBuf::from(&path[..null]);
 
-        // If bind, the path may or may not exist depending on SO_REUSEADDR
-        // Else, the path must exist.
+        // For bind(2), the path must be missing or we return EADDRINUSE.
+        // For connect family, the path must exist or we return ENOENT.
         let fsflags = if cap == Capability::CAP_NET_BIND {
-            FsFlags::empty()
+            FsFlags::MISS_LAST
         } else {
             FsFlags::MUST_PATH
         };
@@ -496,10 +496,17 @@ pub(crate) fn sandbox_addr(
     op: u8,
     caps: Capability,
 ) -> Result<(), Errno> {
+    // bind, connect, accept, sendto, sendmsg, accept4, sendmmsg
+    assert!(
+        matches!(op, 0x2 | 0x3 | 0x5 | 0xb | 0x10 | 0x12 | 0x14),
+        "BUG: sandbox_addr called with invalid op:{op:#x}, report a bug!"
+    );
+
     match addr.family() {
         Some(AddressFamily::Unix) => sandbox_addr_unix(request, sandbox, addr, root, op, caps),
-        Some(AddressFamily::Inet) => sandbox_addr_inet(request, sandbox, addr, op, caps),
-        Some(AddressFamily::Inet6) => sandbox_addr_inet6(request, sandbox, addr, op, caps),
+        Some(AddressFamily::Inet | AddressFamily::Inet6) => {
+            sandbox_addr_inet(request, sandbox, addr, op, caps)
+        }
         Some(_) | None => sandbox_addr_notsup(sandbox),
     }
 }
@@ -514,6 +521,12 @@ pub(crate) fn sandbox_addr_unix(
     op: u8,
     caps: Capability,
 ) -> Result<(), Errno> {
+    // bind, connect, sendto, sendmsg, sendmmsg
+    assert!(
+        matches!(op, 0x2 | 0x3 | 0xb | 0x10 | 0x14),
+        "BUG: sandbox_addr_unix called with invalid op:{op:#x}, report a bug!"
+    );
+
     if sandbox.getcaps(caps).is_empty() {
         // Sandboxing is off.
         return Ok(());
@@ -569,16 +582,15 @@ pub(crate) fn sandbox_addr_unix(
 
     if !filter {
         let sys = op2name(op);
-        let grp = if sys == "bind" { "bind" } else { "connect" };
         if sandbox.verbose {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "unix": &path, "abs": abs,
-                "tip": format!("configure `allow/net/{grp}+{path}'"),
+                "tip": format!("configure `allow/{caps}+{path}'"),
                 "req": request);
         } else {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "unix": &path, "abs": abs,
-                "tip": format!("configure `allow/net/{grp}+{path}'"),
+                "tip": format!("configure `allow/{caps}+{path}'"),
                 "pid": request.scmpreq.pid);
         }
     }
@@ -596,7 +608,7 @@ pub(crate) fn sandbox_addr_unix(
     }
 }
 
-/// Process an `AddressFamily::Inet` socket address.
+/// Process an IPv4 or IPv6 address.
 #[allow(clippy::cognitive_complexity)]
 pub(crate) fn sandbox_addr_inet(
     request: &UNotifyEventRequest,
@@ -605,97 +617,76 @@ pub(crate) fn sandbox_addr_inet(
     op: u8,
     caps: Capability,
 ) -> Result<(), Errno> {
-    if sandbox.getcaps(caps).is_empty() {
+    // accept(2) and accept4(2) are treated specially:
+    // No ACL is done, only IP blocklist check.
+    //
+    // First branch is accept, accept4.
+    // Second branch is bind, connect, send{to,{m,}msg}.
+    if matches!(op, 0x5 | 0x12) {
+        assert!(
+            caps.is_empty(),
+            "BUG: sandbox_addr_inet called with op:{op:#x} and {caps}, report a bug!"
+        );
+    } else if matches!(op, 0x2 | 0x3 | 0xb | 0x10 | 0x14) {
+        assert!(
+            !caps.is_empty(),
+            "BUG: sandbox_addr_inet called with op:{op:#x} and without caps, report a bug!"
+        );
+    } else {
+        unreachable!("BUG: sandbox_addr_inet called with op:{op:#x}, report a bug!");
+    }
+
+    if !caps.is_empty() && sandbox.getcaps(caps).is_empty() {
         // Sandboxing is off.
         return Ok(());
     }
 
-    let addr = addr.as_sockaddr_in().ok_or(Errno::EINVAL)?;
-    let port = addr.port();
-    let addr = IpAddr::V4(addr.ip());
-
-    let (action, filter) = if matches!(op, 0x5 | 0x12) {
-        // accept{,4}: Check for IP blocklist.
-        sandbox.check_block(addr)
+    let (addr, port) = if let Some(sin) = addr.as_sockaddr_in() {
+        (IpAddr::V4(sin.ip()), sin.port())
+    } else if let Some(sa6) = addr.as_sockaddr_in6() {
+        if let Some(v4addr) = sa6.ip().to_ipv4_mapped() {
+            // Mapped IPv4 address, convert to IPv4.
+            (IpAddr::V4(v4addr), sa6.port())
+        } else {
+            // Regular IPv6 address, check as is.
+            (IpAddr::V6(sa6.ip()), sa6.port())
+        }
     } else {
-        // Check for access.
-        sandbox.check_ip(caps, addr, port)
+        return Err(Errno::EINVAL);
     };
 
-    if !filter {
+    // Check for access and IP blocklist as necessary.
+    // caps.is_empty() implies accept{,4}(2) here.
+    let (action, filter) = sandbox.check_ip(caps, addr, port);
+
+    if caps.is_empty() && !filter {
+        // accept{,4}
+        let ipv = if addr.is_ipv6() { 6 } else { 4 };
         let sys = op2name(op);
-        let grp = if sys == "bind" { "bind" } else { "connect" };
         if sandbox.verbose {
-            warn!("ctx": "access", "cap": caps, "act": action,
-                "sys": sys, "addr": format!("{addr}!{port}"),
-                "tip": format!("configure `allow/net/{grp}+{addr}!{port}'"),
+            warn!("ctx": "block", "act": action,
+                "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
+                "tip": format!("configure `block-{addr}'"),
                 "req": request);
         } else {
-            warn!("ctx": "access", "cap": caps, "act": action,
-                "sys": sys, "addr": format!("{addr}!{port}"),
-                "tip": format!("configure `allow/net/{grp}+{addr}!{port}'"),
+            warn!("ctx": "block", "act": action,
+                "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
+                "tip": format!("configure `block-{addr}'"),
                 "pid": request.scmpreq.pid);
         }
-    }
-
-    match action {
-        Action::Allow | Action::Warn => Ok(()),
-        Action::Deny | Action::Filter => Err(op2errno(op)),
-        Action::Panic => panic!(),
-        Action::Exit => std::process::exit(op2errno(op) as i32),
-        action => {
-            // Stop|Kill
-            let _ = request.kill(action);
-            Err(op2errno(op))
-        }
-    }
-}
-
-/// Process an `AddressFamily::Inet6` socket address.
-#[allow(clippy::cognitive_complexity)]
-pub(crate) fn sandbox_addr_inet6(
-    request: &UNotifyEventRequest,
-    sandbox: &SandboxGuard,
-    addr: &SockaddrStorage,
-    op: u8,
-    caps: Capability,
-) -> Result<(), Errno> {
-    if sandbox.getcaps(caps).is_empty() {
-        // Sandboxing is off.
-        return Ok(());
-    }
-
-    let addr = addr.as_sockaddr_in6().ok_or(Errno::EINVAL)?;
-    let port = addr.port();
-    // Check if the IPv6 address is a mapped IPv4 address
-    let (addr, ipv) = if let Some(v4addr) = addr.ip().to_ipv4_mapped() {
-        // It's a mapped IPv4 address, convert to IPv4
-        (IpAddr::V4(v4addr), 4)
-    } else {
-        // It's a regular IPv6 address
-        (IpAddr::V6(addr.ip()), 6)
-    };
-
-    let (action, filter) = if matches!(op, 0x5 | 0x12) {
-        // accept{,4}: Check for IP blocklist.
-        sandbox.check_block(addr)
-    } else {
-        // Check for access.
-        sandbox.check_ip(caps, addr, port)
-    };
-
-    if !filter {
+    } else if !filter {
+        // connect, sendto, send{m,}msg
+        let ipv = if addr.is_ipv6() { 6 } else { 4 };
         let sys = op2name(op);
-        let grp = if sys == "bind" { "bind" } else { "connect" };
         if sandbox.verbose {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
-                "tip": format!("configure `allow/net/{grp}+{addr}!{port}'"),
+                "tip": format!("configure `allow/{caps}+{addr}!{port}'"),
                 "req": request);
         } else {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
-                "tip": format!("configure `allow/net/{grp}+{addr}!{port}'"),
+                "tip": format!("configure `allow/{caps}+{addr}!{port}'"),
                 "pid": request.scmpreq.pid);
         }
     }
@@ -736,15 +727,14 @@ pub(crate) fn handle_safe_bind<Fd: AsFd>(
 ) -> Result<(), Errno> {
     let addr = getsockname::<SockaddrStorage>(fd.as_fd().as_raw_fd())?;
 
-    let addr = if let Some(addr) = addr.as_sockaddr_in() {
+    let (addr, port) = if let Some(addr) = addr.as_sockaddr_in() {
         let port = addr.port();
         if port == 0 {
             return Ok(());
         }
 
         // Allow implicit bind with safe_bind.
-        let addr = IpAddr::V4(addr.ip());
-        format!("{addr}!{port}")
+        (IpNet::new_assert(IpAddr::V4(addr.ip()), 32), port)
     } else if let Some(addr) = addr.as_sockaddr_in6() {
         let port = addr.port();
         if port == 0 {
@@ -752,27 +742,27 @@ pub(crate) fn handle_safe_bind<Fd: AsFd>(
         }
 
         // Allow implicit bind with safe_bind.
-        let addr = IpAddr::V6(addr.ip());
-        format!("{addr}!{port}")
+        (IpNet::new_assert(IpAddr::V6(addr.ip()), 128), port)
     } else {
         return Ok(());
     };
 
-    // Configure sandbox, note we remove
-    // and readd the address so repeated
-    // binds to the same address cannot
-    // overflow the vector.
-    let config: &[String] = &[
-        format!("allow/net/connect-{addr}"),
-        format!("allow/net/connect+{addr}"),
-    ];
-
-    // TODO: Log errors!
+    // Configure sandbox:
+    // Remove and re-add the address so repeated binds to the
+    // same address cannot overflow the vector.
+    let rule = CidrRule {
+        act: Action::Allow,
+        cap: Capability::CAP_NET_CONNECT,
+        pat: AddressPattern {
+            addr,
+            port: Some(port..=port),
+        },
+    };
     let mut sandbox = request.get_mut_sandbox();
-    for cmd in config {
-        sandbox.config(cmd)?;
+    if let Some(idx) = sandbox.cidr_rules.iter().position(|r| *r == rule) {
+        sandbox.cidr_rules.remove(idx);
     }
-    drop(sandbox);
+    sandbox.cidr_rules.push_front(rule)?;
 
     Ok(())
 }
