@@ -9,10 +9,7 @@
 use std::{
     fs::File,
     io::BufReader,
-    os::{
-        fd::{AsFd, AsRawFd},
-        unix::ffi::OsStrExt,
-    },
+    os::fd::{AsFd, AsRawFd},
 };
 
 use libseccomp::ScmpNotifResp;
@@ -21,7 +18,7 @@ use nix::{errno::Errno, NixPath};
 use crate::{
     compat::{fstatat64, statx, STATX_BASIC_STATS, STATX_MODE, STATX_TYPE},
     config::{MAGIC_PREFIX, MMAP_MIN_ADDR},
-    confine::scmp_arch_bits,
+    confine::{scmp_arch_bits, EOWNERDEAD},
     fs::{is_sidechannel_device, parse_fd, CanonicalPath, FileInfo, FileType, FsFlags},
     hash::SydHashSet,
     hook::{SysArg, SysFlags, UNotifyEventRequest},
@@ -33,6 +30,11 @@ use crate::{
 pub(crate) fn sys_stat(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
+    // Return EFAULT here for invalid pointers.
+    if req.data.args[1] < *MMAP_MIN_ADDR {
+        return request.fail_syscall(Errno::EFAULT);
+    }
+
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
     let arg = SysArg {
@@ -45,6 +47,13 @@ pub(crate) fn sys_stat(request: UNotifyEventRequest) -> ScmpNotifResp {
 }
 
 pub(crate) fn sys_stat64(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+
+    // Return EFAULT here for invalid pointers.
+    if req.data.args[1] < *MMAP_MIN_ADDR {
+        return request.fail_syscall(Errno::EFAULT);
+    }
+
     let arg = SysArg {
         path: Some(0),
         fsflags: FsFlags::MUST_PATH,
@@ -57,6 +66,11 @@ pub(crate) fn sys_stat64(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_fstat(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
+    // Return EFAULT here for invalid pointers.
+    if req.data.args[1] < *MMAP_MIN_ADDR {
+        return request.fail_syscall(Errno::EFAULT);
+    }
+
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
     let arg = SysArg {
@@ -68,6 +82,13 @@ pub(crate) fn sys_fstat(request: UNotifyEventRequest) -> ScmpNotifResp {
 }
 
 pub(crate) fn sys_fstat64(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+
+    // Return EFAULT here for invalid pointers.
+    if req.data.args[1] < *MMAP_MIN_ADDR {
+        return request.fail_syscall(Errno::EFAULT);
+    }
+
     let arg = SysArg {
         dirfd: Some(0),
         ..Default::default()
@@ -79,6 +100,11 @@ pub(crate) fn sys_fstat64(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_lstat(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
+    // Return EFAULT here for invalid pointers.
+    if req.data.args[1] < *MMAP_MIN_ADDR {
+        return request.fail_syscall(Errno::EFAULT);
+    }
+
     let is32 = scmp_arch_bits(req.data.arch) == 32;
 
     let arg = SysArg {
@@ -91,6 +117,13 @@ pub(crate) fn sys_lstat(request: UNotifyEventRequest) -> ScmpNotifResp {
 }
 
 pub(crate) fn sys_lstat64(request: UNotifyEventRequest) -> ScmpNotifResp {
+    let req = request.scmpreq;
+
+    // Return EFAULT here for invalid pointers.
+    if req.data.args[1] < *MMAP_MIN_ADDR {
+        return request.fail_syscall(Errno::EFAULT);
+    }
+
     let arg = SysArg {
         path: Some(0),
         fsflags: FsFlags::MUST_PATH | FsFlags::NO_FOLLOW_LAST,
@@ -103,6 +136,11 @@ pub(crate) fn sys_lstat64(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_statx(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
+    // Return EFAULT here for invalid pointers.
+    if req.data.args[4] < *MMAP_MIN_ADDR {
+        return request.fail_syscall(Errno::EFAULT);
+    }
+
     let empty = req.data.args[2] & libc::AT_EMPTY_PATH as u64 != 0;
     let follow = req.data.args[2] & libc::AT_SYMLINK_NOFOLLOW as u64 == 0;
 
@@ -131,6 +169,11 @@ pub(crate) fn sys_statx(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_newfstatat(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
+    // Return EFAULT here for invalid pointers.
+    if req.data.args[2] < *MMAP_MIN_ADDR {
+        return request.fail_syscall(Errno::EFAULT);
+    }
+
     let empty = req.data.args[3] & libc::AT_EMPTY_PATH as u64 != 0;
     let follow = req.data.args[3] & libc::AT_SYMLINK_NOFOLLOW as u64 == 0;
     let mut flags = SysFlags::empty();
@@ -180,14 +223,8 @@ fn syscall_stat_handler(
         // If lock is on do not check for magic path.
         let (mut path, magic) = request.read_path(&sandbox, arg, !is_lock)?;
 
-        // SAFETY: For magic calls we allow NULL as stat argument, see syd(2).
-        // For other calls, return EFAULT here for invalid pointers.
-        let is_magic = !is_lock && magic;
-        if !is_magic && req.data.args[arg_stat] < *MMAP_MIN_ADDR {
-            return Err(Errno::EFAULT);
-        }
-
-        if is_magic {
+        let mut ghost = false;
+        if !is_lock && magic {
             drop(sandbox); // release the read-lock.
 
             // Handle magic prefix (ie /dev/syd)
@@ -205,20 +242,14 @@ fn syscall_stat_handler(
             let mut sandbox = request.get_mut_sandbox();
 
             // Execute magic command.
-            match cmd.as_os_str().as_bytes() {
-                b"ghost" => {
-                    // SAFETY: Reset sandbox to ensure no run-away execs.
-                    sandbox.reset()?;
-
-                    // Signal the poll process to exit.
-                    return Err(Errno::EOWNERDEAD);
-                }
-                b"panic" => sandbox.panic()?,
-                _ => {}
-            }
-
             if cmd.is_empty() || cmd.is_equal(b".el") || cmd.is_equal(b".sh") {
                 sandbox.config("")?;
+            } else if cmd.is_equal(b"panic") {
+                sandbox.panic()?;
+            } else if cmd.is_equal(b"ghost") {
+                // SAFETY: Reset sandbox to ensure no run-away execs.
+                sandbox.reset()?;
+                ghost = true;
             } else if let Some(cmd) = cmd.strip_prefix(b"load") {
                 // We handle load specially here as it involves process access.
                 // 1. Attempt to parse as FD, pidfd_getfd and load it.
@@ -257,11 +288,6 @@ fn syscall_stat_handler(
                 return Err(Errno::EINVAL);
             }
             drop(sandbox); // release the write-lock.
-
-            // If the stat buffer is NULL, return immediately.
-            if req.data.args[arg_stat] == 0 {
-                return Ok(request.return_syscall(0));
-            }
         } else {
             // Handle fstat for files with encryption in progress.
             let mut crypt_stat = false;
@@ -466,7 +492,14 @@ fn syscall_stat_handler(
             }
         }
 
-        // stat system call successfully emulated.
+        // Use the pseudo errno(3) EOWNERDEAD to initiate ghost mode.
+        // We only do it here to ensure metadata of /dev/null was
+        // written to sandbox process memory.
+        if ghost {
+            return Ok(ScmpNotifResp::new(0, 0, EOWNERDEAD, 0));
+        }
+
+        // stat(2) system call has been successfully emulated.
         Ok(request.return_syscall(0))
     })
 }
