@@ -177,7 +177,12 @@ fn syscall_utime_handler(
     // 2. nix does not define AT_EMPTY_PATH in `UtimensatFlags`,
     //    so we have to use libc instead.
     Errno::result(unsafe {
-        libc::utimensat(fd, c"".as_ptr().cast(), &times[0], libc::AT_EMPTY_PATH)
+        libc::utimensat(
+            fd,
+            c"".as_ptr().cast(),
+            &raw const times[0],
+            libc::AT_EMPTY_PATH,
+        )
     })
     .map(|_| request.return_syscall(0))
 }
