@@ -118,7 +118,7 @@ impl AccessFs {
     pub fn from_read(abi: ABI) -> Self {
         match abi {
             ABI::Unsupported => AccessFs::EMPTY,
-            ABI::V1 | ABI::V2 | ABI::V3 | ABI::V4 | ABI::V5 | ABI::V6 => {
+            ABI::V1 | ABI::V2 | ABI::V3 | ABI::V4 | ABI::V5 | ABI::V6 | ABI::V7 => {
                 make_bitflags!(AccessFs::{
                     Execute
                     | ReadFile
@@ -148,7 +148,7 @@ impl AccessFs {
             }),
             ABI::V2 => Self::from_write(ABI::V1) | AccessFs::Refer,
             ABI::V3 | ABI::V4 => Self::from_write(ABI::V2) | AccessFs::Truncate,
-            ABI::V5 | ABI::V6 => Self::from_write(ABI::V4) | AccessFs::IoctlDev,
+            ABI::V5 | ABI::V6 | ABI::V7 => Self::from_write(ABI::V4) | AccessFs::IoctlDev,
         }
     }
 
@@ -557,7 +557,7 @@ fn path_beneath_compatibility() {
     path.set_compatibility(CompatLevel::HardRequirement);
 }
 
-// It is useful for documentation generation to explicitely implement Rule for every types, instead
+// It is useful for documentation generation to explicitly implement Rule for every types, instead
 // of doing it generically.
 impl<F> Rule<AccessFs> for PathBeneath<F> where F: AsFd {}
 
@@ -684,8 +684,8 @@ fn path_fd() {
 ///
 /// ```
 /// use syd::landlock::{
-///     path_beneath_rules, Access, AccessFs, Ruleset, RulesetAttr, RulesetCreatedAttr,
-///     RulesetError, RulesetStatus, ABI,
+///     path_beneath_rules, Access, AccessFs, RestrictSelfFlags, Ruleset, RulesetAttr,
+///     RulesetCreatedAttr, RulesetError, RulesetStatus, ABI,
 /// };
 ///
 /// fn restrict_thread() -> Result<(), RulesetError> {
@@ -697,7 +697,7 @@ fn path_fd() {
 ///         .add_rules(path_beneath_rules(&["/usr", "/etc", "/dev"], AccessFs::from_read(abi)))?
 ///         // Read-write access to /home and /tmp.
 ///         .add_rules(path_beneath_rules(&["/home", "/tmp"], AccessFs::from_all(abi)))?
-///         .restrict_self()?;
+///         .restrict_self(RestrictSelfFlags::empty())?;
 ///     match status.ruleset {
 ///         // The FullyEnforced case must be tested by the developer.
 ///         RulesetStatus::FullyEnforced => println!("Fully sandboxed."),
