@@ -9,6 +9,8 @@
 // SAFETY: This module has been liberated from unsafe code!
 #![forbid(unsafe_code)]
 
+use std::sync::LazyLock;
+
 use indexmap::IndexMap;
 use memchr::arch::all::is_equal;
 use nix::{
@@ -16,7 +18,6 @@ use nix::{
     fcntl::{open, OFlag},
     sys::stat::Mode,
 };
-use once_cell::sync::Lazy;
 
 use crate::{
     hash::{SydHashMap, SydHashSet},
@@ -33,7 +34,7 @@ use crate::{
 };
 
 /// landlock(7) filesystem access rights.
-pub static LANDLOCK_ACCESS_FS: Lazy<IndexMap<&str, AccessFs>> = Lazy::new(|| {
+pub static LANDLOCK_ACCESS_FS: LazyLock<IndexMap<&str, AccessFs>> = LazyLock::new(|| {
     IndexMap::from_iter([
         ("read", AccessFs::ReadFile),
         ("write", AccessFs::WriteFile),
@@ -59,17 +60,20 @@ pub static LANDLOCK_ACCESS_FS: Lazy<IndexMap<&str, AccessFs>> = Lazy::new(|| {
         ("dpath", LandlockPolicy::access_fs_from_set("dpath")),
         ("spath", LandlockPolicy::access_fs_from_set("spath")),
         ("tpath", LandlockPolicy::access_fs_from_set("tpath")),
+        ("bnet", LandlockPolicy::access_fs_from_set("bnet")),
     ])
 });
 
 /// landlock(7) network access rights.
-pub static LANDLOCK_ACCESS_NET: Lazy<IndexMap<&str, AccessNet>> = Lazy::new(|| {
+pub static LANDLOCK_ACCESS_NET: LazyLock<IndexMap<&str, AccessNet>> = LazyLock::new(|| {
     IndexMap::from_iter([
         ("bind", AccessNet::BindTcp),
         ("connect", AccessNet::ConnectTcp),
         // Aliases
         ("net", LandlockPolicy::access_net_from_set("net")),
         ("inet", LandlockPolicy::access_net_from_set("inet")),
+        ("bnet", LandlockPolicy::access_net_from_set("bnet")),
+        ("cnet", LandlockPolicy::access_net_from_set("cnet")),
     ])
 });
 
@@ -406,8 +410,10 @@ impl LandlockPolicy {
             AccessFs::MakeFifo | AccessFs::MakeSym
         } else if is_equal(s, b"tpath") {
             AccessFs::MakeDir | AccessFs::RemoveDir
+        } else if is_equal(s, b"bnet") {
+            AccessFs::MakeSock
         } else {
-            panic!("BUG: Invalid landlock(7) filesystem access right {set}");
+            unreachable!("BUG: Invalid landlock(7) filesystem access right {set}, report a bug!");
         }
     }
 
@@ -418,12 +424,14 @@ impl LandlockPolicy {
         let s = set.as_bytes();
         if is_equal(s, b"all") {
             AccessNet::all()
-        } else if is_equal(s, b"net") {
-            AccessNet::BindTcp | AccessNet::ConnectTcp
-        } else if is_equal(s, b"inet") {
+        } else if is_equal(s, b"bnet") {
+            AccessNet::BindTcp
+        } else if is_equal(s, b"cnet") {
+            AccessNet::ConnectTcp
+        } else if is_equal(s, b"net") || is_equal(s, b"inet") {
             AccessNet::BindTcp | AccessNet::ConnectTcp
         } else {
-            panic!("BUG: Invalid landlock(7) network access right {set}");
+            unreachable!("BUG: Invalid landlock(7) network access right {set}, report a bug!");
         }
     }
 
