@@ -9,6 +9,8 @@
 use std::{
     io::Write,
     os::fd::{BorrowedFd, RawFd},
+    sync::OnceLock,
+    thread::ThreadId,
     time::{SystemTime, UNIX_EPOCH},
 };
 
@@ -22,17 +24,102 @@ use serde_json::{Map, Value};
 
 use crate::{
     config::*,
-    fs::{is_active_fd, lock_fd, unlock_fd},
-    proc::*,
+    err::SydResult,
+    fs::{is_active_fd, lock_fd, retry_on_eintr, unlock_fd},
+    proc::{proc_cmdline, proc_comm, proc_cwd, proc_tty},
     syslog::LogLevel,
 };
 
+/// Formatted print which returns Result as Errno rather than panicking.
+#[macro_export]
+macro_rules! printf {
+    () => {{}};
+    ($($arg:tt)*) => {{
+        use ::std::io::Write as _;
+        let __s = format!($($arg)*);
+        ::std::io::stdout().write_all(__s.as_bytes())
+            .map_err(|error| $crate::err::err2no(&error))
+    }};
+}
+
+/// Formatted println which returns Result as Errno rather than panicking.
+#[macro_export]
+macro_rules! printfln {
+    () => {{
+        use ::std::io::Write as _;
+        ::std::io::stdout().write_all(b"\n")
+            .map_err(|error| $crate::err::err2no(&error))
+    }};
+    ($($arg:tt)*) => {{
+        use ::std::io::Write as _;
+        let mut __s = format!($($arg)*);
+        __s.push('\n');
+        ::std::io::stdout().write_all(__s.as_bytes())
+            .map_err(|error| $crate::err::err2no(&error))
+    }};
+}
+
 // Whether we concluded the output is a TTY.
 pub(crate) static LOG_TTY: std::sync::atomic::AtomicBool =
     std::sync::atomic::AtomicBool::new(false);
 // Log destination file descriptor.
 pub(crate) static LOG_FD: std::sync::atomic::AtomicI32 = std::sync::atomic::AtomicI32::new(-42);
 
+// Main thread identifier, used to decide on panic hook.
+pub(crate) static LOG_MAIN_TID: OnceLock<ThreadId> = OnceLock::new();
+
+// Set this in main thread after namespace forks and other dances.
+pub(crate) fn log_init_main() -> SydResult<()> {
+    LOG_MAIN_TID
+        .set(std::thread::current().id())
+        .map_err(|_| Errno::EBUSY.into())
+}
+
+// Sets the global panic hook for Syd threads.
+pub(crate) fn log_set_panic_hook() {
+    // Set a logging panic hook. The default panic
+    // hook calls system calls not permitted by emulators
+    // such as getcwd(2), stat(2) etc.
+    #[allow(clippy::cognitive_complexity)]
+    std::panic::set_hook(Box::new(|info| {
+        let this = std::thread::current();
+        let name = this.name().unwrap_or("?");
+
+        let err = match info.payload().downcast_ref::<&'static str>() {
+            Some(s) => *s,
+            None => match info.payload().downcast_ref::<String>() {
+                Some(s) => &**s,
+                None => "?",
+            },
+        };
+
+        let file = info.location().map(|l| l.file());
+        let line = info.location().map(|l| l.line());
+
+        if log_is_main() {
+            // Main thread panicking isn't recoverable unlike others.
+            // Ensure clean exit right away.
+            crate::alert!("ctx": "panic",
+                "op": "panic_syd_main_thread",
+                "msg": err, "file": file, "line": line);
+            std::process::exit(101);
+        } else {
+            crate::crit!("ctx": "panic",
+                "op": format!("panic_{name}_thread"),
+                "msg": err, "file": file, "line": line);
+        }
+    }));
+}
+
+// Use this to check for main thread in panic handler.
+#[inline]
+fn log_is_main() -> bool {
+    LOG_MAIN_TID
+        .get()
+        .map(|&tid| tid == std::thread::current().id())
+        .unwrap_or(false)
+}
+
 /// emerg! logging macro
 #[macro_export]
 macro_rules! emerg {
@@ -209,7 +296,7 @@ pub(crate) struct LockedWriter<'a> {
 
 impl<'a> LockedWriter<'a> {
     pub(crate) fn new(fd: BorrowedFd<'a>) -> Result<Self, Errno> {
-        lock_fd(fd, true, true)?;
+        retry_on_eintr(|| lock_fd(fd, true, true))?;
         Ok(Self { fd })
     }
 }
@@ -305,8 +392,13 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
     init_global_syslog(logbuflen, fd, level, use_stack)?;
 
     // Finally let's make some noise!
-    info!("ctx": "init", "op": "sing", "chapter": 24,
-        "msg": "Change return success. Going and coming without error. Action brings good fortune.");
+    //
+    // ENV_SING avoids duplicating this message over memfd-reexec.
+    if std::env::var_os(ENV_SING).is_none() {
+        info!("ctx": "init", "op": "sing", "chapter": 24,
+            "msg": "Change return success. Going and coming without error. Action brings good fortune.");
+        std::env::set_var(ENV_SING, "");
+    }
 
     Ok(())
 }
@@ -380,12 +472,18 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
 #[cfg(feature = "log")]
 #[allow(clippy::cognitive_complexity)]
 pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String, Value>) {
+    let sys = if let Some(sys) = crate::syslog::global_syslog() {
+        sys
+    } else {
+        return; // Logging is disabled.
+    };
+
     // For "higher" severities, we add more contextual fields.
     let add_context = level.as_u8() <= crate::syslog::LogLevel::Notice.as_u8();
     let tty = LOG_TTY.load(std::sync::atomic::Ordering::Relaxed);
 
     // If there's a "pid", we might add cmd/cwd, etc.
-    // We remove and readd to reorder for better visibility.
+    // We remove and re-add to reorder for better visibility.
     if let Some(pid_v) = msg.remove("pid").and_then(|v| v.as_i64()) {
         #[allow(clippy::cast_possible_truncation)]
         let pid = Pid::from_raw(pid_v as libc::pid_t);
@@ -406,6 +504,13 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
             } else {
                 msg.insert("cwd".to_string(), Value::Null);
             }
+            if add_context {
+                if let Ok(tty) = proc_tty(pid) {
+                    msg.insert("tty".to_string(), Value::String(tty.to_string()));
+                } else {
+                    msg.insert("tty".to_string(), Value::Null);
+                }
+            }
         }
         msg.insert("pid".to_string(), Value::Number(pid.as_raw().into()));
     }
@@ -454,9 +559,7 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
         None
     };
 
-    if let Some(sys) = crate::syslog::global_syslog() {
-        sys.write_log(level, &msg_data, msg_pretty.as_deref());
-    }
+    sys.write_log(level, &msg_data, msg_pretty.as_deref());
 }
 
 /// Initializes Syslog global object, reading environment variables.
@@ -527,8 +630,15 @@ pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resu
     if let Some(fd) = fd {
         LOG_FD.store(fd, std::sync::atomic::Ordering::Relaxed);
     }
-    info!("ctx": "init", "op": "sing", "chapter": 24,
-        "msg": "Change return success. Going and coming without error. Action brings good fortune.");
+
+    // Finally let's make some noise!
+    //
+    // ENV_SING avoids duplicating this message over memfd-reexec.
+    if std::env::var_os(ENV_SING).is_none() {
+        info!("ctx": "init", "op": "sing", "chapter": 24,
+            "msg": "Change return success. Going and coming without error. Action brings good fortune.");
+        std::env::set_var(ENV_SING, "");
+    }
 
     Ok(())
 }
@@ -546,12 +656,21 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
 #[cfg(not(feature = "log"))]
 #[allow(clippy::cognitive_complexity)]
 pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String, Value>) {
+    // Setting LOG-FD to negative is OK to disable logging.
+    let fd = LOG_FD.load(std::sync::atomic::Ordering::Relaxed);
+    let fd = if fd < 0 {
+        return; // Logging is disabled.
+    } else {
+        // SAFETY: we trust log fd is valid.
+        unsafe { BorrowedFd::borrow_raw(fd) }
+    };
+
     // For "higher" severities, we add more contextual fields.
     let add_context = level.as_u8() <= crate::syslog::LogLevel::Notice.as_u8();
     let tty = LOG_TTY.load(std::sync::atomic::Ordering::Relaxed);
 
     // If there's a "pid", we might add cmd/cwd, etc.
-    // We remove and readd to reorder for better visibility.
+    // We remove and re-add to reorder for better visibility.
     if let Some(pid_v) = msg.remove("pid").and_then(|v| v.as_i64()) {
         #[allow(clippy::cast_possible_truncation)]
         let pid = Pid::from_raw(pid_v as libc::pid_t);
@@ -572,6 +691,13 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
             } else {
                 msg.insert("cwd".to_string(), Value::Null);
             }
+            if add_context {
+                if let Ok(tty) = proc_tty(pid) {
+                    msg.insert("tty".to_string(), Value::String(tty.to_string()));
+                } else {
+                    msg.insert("tty".to_string(), Value::Null);
+                }
+            }
         }
         msg.insert("pid".to_string(), Value::Number(pid.as_raw().into()));
     }
@@ -622,18 +748,13 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
     };
 
     // Finally, log to fd or standard error.
-    let fd = LOG_FD.load(std::sync::atomic::Ordering::Relaxed);
-    if fd >= 0 {
-        // SAFETY: we trust log fd is valid.
-        let fd = unsafe { BorrowedFd::borrow_raw(fd) };
-        if let Ok(mut writer) = LockedWriter::new(fd).map(std::io::BufWriter::new) {
-            if let Some(ref msg) = msg_info {
-                let _ = writer.write_all(msg.as_bytes());
-            } else {
-                let _ = writer.write_all(msg_data.as_bytes());
-            }
-            let _ = writer.write_all(b"\n");
+    if let Ok(mut writer) = LockedWriter::new(fd).map(std::io::BufWriter::new) {
+        if let Some(ref msg) = msg_info {
+            let _ = writer.write_all(msg.as_bytes());
+        } else {
+            let _ = writer.write_all(msg_data.as_bytes());
         }
+        let _ = writer.write_all(b"\n");
     }
 }
 
@@ -660,7 +781,7 @@ fn format_iso8601(timestamp: u64) -> String {
     let mut tm = std::mem::MaybeUninit::uninit();
 
     // SAFETY: gmtime_r returns NULL on error.
-    if unsafe { libc::gmtime_r(&timestamp, tm.as_mut_ptr()) }.is_null() {
+    if unsafe { libc::gmtime_r(&raw const timestamp, tm.as_mut_ptr()) }.is_null() {
         return "error on gmtime_r()".to_string();
     }
 
