@@ -40,6 +40,7 @@ use crate::{
 //
 // Note, this list must be sorted because it's binary searched.
 const NETLINK_FAMILIES: &[&str] = &[
+    "all",
     "audit",
     "connector",
     "crypto",
@@ -113,7 +114,7 @@ pub type PortSet = FixedBitSet;
 pub enum LandlockRule {
     /// landlock(7) filesystem rule
     Fs((AccessFs, String)),
-    /// landlock(7) network fule
+    /// landlock(7) network rule
     Net((AccessNet, PortRange)),
 }
 
@@ -581,12 +582,9 @@ pub(crate) fn parse_scmp_cmd(input: &str) -> Result<ScmpCmd, Errno> {
                 return Err(Errno::EINVAL);
             }
 
-            // Address argument is only valid for Bind and Connect.
-            let maybe_addr = filter
-                .intersects(Capability::CAP_NET_BIND | Capability::CAP_NET_CONNECT)
-                && filter
-                    .difference(Capability::CAP_NET_BIND | Capability::CAP_NET_CONNECT)
-                    .is_empty();
+            // IP address arguments are only valid for the `inet` set.
+            let maybe_addr = filter.intersects(Capability::CAP_INET)
+                && filter.difference(Capability::CAP_INET).is_empty();
 
             // Pattern resolution.
             let pat = if maybe_addr {
