@@ -20,6 +20,7 @@ use std::{
         unix::ffi::{OsStrExt, OsStringExt},
     },
     path::{Path, PathBuf},
+    sync::LazyLock,
 };
 
 use btoi::btoi;
@@ -34,7 +35,6 @@ use nix::{
     unistd::Pid,
     NixPath,
 };
-use once_cell::sync::Lazy;
 
 use crate::{
     config::MAGIC_PREFIX,
@@ -57,7 +57,7 @@ pub const PATH_MAX: usize = 4096;
 pub const PATH_MIN: usize = 64;
 
 // This pointer is confined by seccomp for use with openat(2) for getdir_long().
-static DOTDOT: Lazy<u64> = Lazy::new(|| c"..".as_ptr() as *const libc::c_char as u64);
+static DOTDOT: LazyLock<u64> = LazyLock::new(|| c"..".as_ptr() as *const libc::c_char as u64);
 
 #[inline(always)]
 pub(crate) fn dotdot_with_nul() -> u64 {
@@ -804,7 +804,7 @@ impl XPath {
         self.starts_with(MAGIC_PREFIX)
     }
 
-    /// Check if path is literaly the root path, ie `/`.
+    /// Check if path is literally the root path, ie `/`.
     pub fn is_root(&self) -> bool {
         self.is_equal(b"/")
     }
@@ -863,6 +863,20 @@ impl XPath {
             .unwrap_or(false)
     }
 
+    /// Check if path points to machine-id(5), /etc/hostid or /var/adm/hostid.
+    pub fn is_machine_id(&self) -> bool {
+        for f in [
+            &b"/etc/machine-id"[..],
+            &b"/etc/hostid"[..],
+            &b"/var/adm/hostid"[..],
+        ] {
+            if self.is_equal(f) {
+                return true;
+            }
+        }
+        false
+    }
+
     /// Check if path points to a proc_pid_status(5) file.
     pub fn is_proc_pid_status(&self) -> bool {
         self.is_proc_pid() && self.ends_with(b"/status")
@@ -984,7 +998,7 @@ impl XPath {
 
     /// Determine whether path contains the given substring.
     pub fn contains(&self, sub: &[u8]) -> bool {
-        memmem::find_iter(self.as_bytes(), sub).next().is_some()
+        memmem::find(self.as_bytes(), sub).is_some()
     }
 
     /// Determine whether path contains the given character.
