@@ -42,7 +42,6 @@ use crate::{
     alert,
     config::*,
     confine::ExportMode,
-    crit,
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
     fs::{block_signal, closeexcept, retry_on_eintr, retry_on_intr, seccomp_export_pfc},
@@ -241,23 +240,6 @@ impl ThreadPool {
             .name("syd_mon".to_string())
             .stack_size(MON_STACK_SIZE)
             .spawn(move || {
-                // SAFETY: Default panic hook wont work with stdio(3) closed.
-                std::panic::set_hook(Box::new(|info| {
-                    let err = match info.payload().downcast_ref::<&'static str>() {
-                        Some(s) => *s,
-                        None => match info.payload().downcast_ref::<String>() {
-                            Some(s) => &**s,
-                            None => "?",
-                        },
-                    };
-
-                    let file = info.location().map(|l| l.file());
-                    let line = info.location().map(|l| l.line());
-
-                    crit!("ctx": "panic", "op": "panic_monitor_thread",
-                        "msg": err, "file": file, "line": line);
-                }));
-
                 // Get information from the sandbox about:
                 // 1. Encryption if Crypt sandboxing is enabled.
                 // 2. Transit UIDs and GIDs for SafeSetId.
