@@ -35,6 +35,7 @@ use std::{
         unix::ffi::OsStrExt,
     },
     ptr::NonNull,
+    sync::LazyLock,
 };
 
 use bitflags::bitflags;
@@ -42,7 +43,7 @@ use btoi::{btoi, btoi_radix};
 use indexmap::IndexSet;
 use nix::{
     errno::Errno,
-    fcntl::{open, readlinkat, OFlag},
+    fcntl::{open, readlinkat, OFlag, ResolveFlag},
     libc::pid_t,
     sys::{
         stat::Mode,
@@ -51,7 +52,6 @@ use nix::{
     },
     unistd::{getpid, lseek64, read, write, Gid, Pid, Uid, Whence},
 };
-use once_cell::sync::Lazy;
 use procfs_core::{
     net::UnixNetEntries,
     process::{LimitValue, MMPermissions, MMapPath, MemoryMaps, Namespace, Namespaces},
@@ -362,10 +362,23 @@ pub struct Status {
     pub sig_caught: SydSigSet,
 }
 
-/// Safe `OFlags` to use when opening `/proc` files.
+/// Safe `OFlags` to use when opening proc(5) files.
 pub const OFLAG_PROC: OFlag =
     OFlag::from_bits_retain(libc::O_RDONLY | libc::O_CLOEXEC | libc::O_NOFOLLOW | libc::O_NOCTTY);
 
+/// Safe `ResolveFlag` to use when opening proc(5) files.
+pub const RFLAG_PROC: ResolveFlag = ResolveFlag::from_bits_retain(
+    libc::RESOLVE_NO_MAGICLINKS
+        | libc::RESOLVE_NO_SYMLINKS
+        | libc::RESOLVE_BENEATH
+        | libc::RESOLVE_NO_XDEV,
+);
+
+/// Function to safely open a proc(5) file and return an `OwnedFd`.
+pub fn safe_open_proc(path: &XPath) -> Result<OwnedFd, Errno> {
+    safe_open(PROC_FILE(), path, OFLAG_PROC, RFLAG_PROC)
+}
+
 // major 204 is a mess -- "Low-density serial ports"
 const LOW_DENSITY_NAMES: &[&str] = &[
     "LU0", "LU1", "LU2", "LU3", "FB0", "SA0", "SA1", "SA2", "SC0", "SC1", "SC2", "SC3", "FW0",
@@ -431,7 +444,7 @@ pub fn proc_stat(pid: Pid) -> Result<Stat, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"stat");
 
-    let mut file = safe_open(PROC_FILE(), &path, OFLAG_PROC)
+    let mut file = safe_open_proc(&path)
         .map(File::from)
         .or(Err(Errno::ESRCH))?;
 
@@ -444,7 +457,7 @@ pub fn proc_statm(pid: Pid) -> Result<Statm, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"statm");
 
-    let mut file = safe_open(PROC_FILE(), &path, OFLAG_PROC)
+    let mut file = safe_open_proc(&path)
         .map(File::from)
         .or(Err(Errno::ESRCH))?;
 
@@ -463,7 +476,7 @@ pub fn proc_status_open(pid: Pid) -> Result<OwnedFd, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"status");
 
-    safe_open(PROC_FILE(), &path, OFLAG_PROC).or(Err(Errno::ESRCH))
+    safe_open_proc(&path)
 }
 
 /// Return the status of the given process from the given Fd.
@@ -516,7 +529,7 @@ pub fn proc_stack_pointer(pid: Pid) -> Result<u64, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"syscall");
 
-    let mut file = safe_open(PROC_FILE(), &path, OFLAG_PROC)
+    let mut file = safe_open_proc(&path)
         .map(File::from)
         .map(BufReader::new)
         .or(Err(Errno::ESRCH))?;
@@ -548,7 +561,7 @@ pub fn proc_auxv(pid: Pid) -> Result<SydHashMap<u64, u64>, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"auxv");
 
-    let mut file = safe_open(PROC_FILE(), &path, OFLAG_PROC)
+    let mut file = safe_open_proc(&path)
         .map(File::from)
         .or(Err(Errno::ESRCH))?;
 
@@ -595,7 +608,7 @@ pub fn proc_comm(pid: Pid) -> Result<XPathBuf, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"comm");
 
-    let mut file = safe_open(PROC_FILE(), &path, OFLAG_PROC)
+    let mut file = safe_open_proc(&path)
         .or(Err(Errno::ESRCH))
         .map(File::from)?;
 
@@ -633,7 +646,7 @@ pub fn proc_cmdline(pid: Pid) -> Result<XPathBuf, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"cmdline");
 
-    let mut file = safe_open(PROC_FILE(), &path, OFLAG_PROC)
+    let mut file = safe_open_proc(&path)
         .or(Err(Errno::ESRCH))
         .map(File::from)?;
 
@@ -693,7 +706,7 @@ pub fn proc_maps_open(pid: Pid) -> Result<OwnedFd, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"maps");
 
-    safe_open(PROC_FILE(), &path, OFLAG_PROC).or(Err(Errno::ESRCH))
+    safe_open_proc(&path)
 }
 
 /// Return the memory maps of the given process from the given Fd.
@@ -710,7 +723,7 @@ pub fn proc_smaps(pid: Pid) -> Result<Vec<SydMemoryMap>, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"smaps");
 
-    let reader = safe_open(PROC_FILE(), &path, OFLAG_PROC)
+    let reader = safe_open_proc(&path)
         .map(File::from)
         .map(BufReader::new)
         .or(Err(Errno::ESRCH))?;
@@ -744,11 +757,7 @@ pub fn proc_pipemax() -> Result<libc::c_int, Errno> {
 
 /// Open a FD to /proc/sys/fs/pipe-max-size and return an OwnedFd.
 pub fn proc_pipemax_open() -> Result<OwnedFd, Errno> {
-    safe_open(
-        PROC_FILE(),
-        XPath::from_bytes(b"sys/fs/pipe-max-size"),
-        OFLAG_PROC,
-    )
+    safe_open_proc(XPath::from_bytes(b"sys/fs/pipe-max-size"))
 }
 
 /// Retrieve the system pipe max limit from the given Fd.
@@ -854,6 +863,35 @@ pub fn proc_mmap_min_addr() -> Result<u64, Errno> {
     btoi::<u64>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))
 }
 
+/// Retrieves the value of /proc/sys/kernel/randomize_va_space
+#[allow(clippy::disallowed_methods)]
+pub fn proc_kernel_randomize_va_space() -> Result<u8, Errno> {
+    // SAFETY: Informational command,
+    // runs before syd::config::proc_init.
+    #[allow(clippy::disallowed_methods)]
+    let fd = nix::fcntl::openat(
+        AT_BADFD,
+        XPath::from_bytes(b"/proc/sys/kernel/randomize_va_space"),
+        OFLAG_PROC,
+        Mode::empty(),
+    )?;
+
+    // Read up to 1 byte + 1 byte for \n.
+    let mut data = [0u8; 2];
+    let mut nread = 0;
+    while nread < data.len() {
+        #[allow(clippy::arithmetic_side_effects)]
+        match read(&fd, &mut data[nread..]) {
+            Ok(0) => break,
+            Ok(n) => nread += n,
+            Err(Errno::EINTR) => continue,
+            Err(_) => return Err(Errno::ESRCH),
+        }
+    }
+
+    btoi::<u8>(trim_ascii(&data[..nread])).or(Err(Errno::EINVAL))
+}
+
 /// Reads the tty number from /proc/[pid]/stat and figures out the corresponding /dev/tty device node path.
 #[allow(clippy::arithmetic_side_effects)]
 #[allow(clippy::cast_sign_loss)]
@@ -1299,7 +1337,7 @@ pub fn proc_environ_open(pid: Pid) -> Result<OwnedFd, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"environ");
 
-    safe_open(PROC_FILE(), &path, OFLAG_PROC).or(Err(Errno::ESRCH))
+    safe_open_proc(&path)
 }
 
 /// Parses /proc/pid/environ from given Fd and returns a HashMap.
@@ -1391,7 +1429,7 @@ pub fn proc_crypto() -> Result<CryptoTable, Errno> {
 
 /// Open a FD to _proc_crypto_(5) and return an OwnedFd.
 pub fn proc_crypto_open() -> Result<OwnedFd, Errno> {
-    safe_open(PROC_FILE(), XPath::from_bytes(b"crypto"), OFLAG_PROC)
+    safe_open_proc(XPath::from_bytes(b"crypto"))
 }
 
 /// Parses /proc/crypto from given Fd and returns the table of crypto algorithms.
@@ -1402,9 +1440,7 @@ pub fn proc_crypto_read(fd: OwnedFd) -> Result<CryptoTable, Errno> {
 /// Parses /proc/net/unix and returns the inodes of all UNIX domain sockets.
 pub fn proc_unix_get_inodes() -> Result<SydHashSet<u64>, Errno> {
     let path = XPath::from_bytes(b"net/unix");
-    let file = safe_open(PROC_FILE(), path, OFLAG_PROC)
-        .map(File::from)
-        .map(BufReader::new)?;
+    let file = safe_open_proc(path).map(File::from).map(BufReader::new)?;
     Ok(UnixNetEntries::from_buf_read(file)
         .map(|e| e.0)
         .or(Err(Errno::EPERM))?
@@ -1441,10 +1477,10 @@ pub fn proc_unix_get_inodes() -> Result<SydHashSet<u64>, Errno> {
 pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
     // This function requires CONFIG_CROSS_MEMORY_ATTACH,
     // unless Syd is explicitly configured to fallback.
-    static FORCE_PROC: Lazy<bool> =
-        Lazy::new(|| std::env::var_os(ENV_NO_CROSS_MEMORY_ATTACH).is_some());
-    static PROC_MEM_F: Lazy<bool> =
-        Lazy::new(|| std::env::var_os(ENV_PROC_PID_MEM_FALLBACK).is_some());
+    static FORCE_PROC: LazyLock<bool> =
+        LazyLock::new(|| std::env::var_os(ENV_NO_CROSS_MEMORY_ATTACH).is_some());
+    static PROC_MEM_F: LazyLock<bool> =
+        LazyLock::new(|| std::env::var_os(ENV_PROC_PID_MEM_FALLBACK).is_some());
     proc_set_at_secure2(
         pid,
         elf_type,
@@ -1472,7 +1508,7 @@ pub fn proc_set_at_secure2(
         let mut flags = OFLAG_PROC;
         flags.remove(OFlag::O_RDONLY);
         flags.insert(OFlag::O_RDWR);
-        match safe_open(PROC_FILE(), &path, flags) {
+        match safe_open(PROC_FILE(), &path, flags, RFLAG_PROC) {
             Ok(fd) => Some(File::from(fd)),
             Err(_) => return Err(Errno::EACCES),
         }
@@ -1759,7 +1795,12 @@ pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
     let mut dir = XPathBuf::from_pid(pid);
     dir.push(b"fd");
 
-    let dir = safe_open(PROC_FILE(), &dir, OFLAG_PROC | OFlag::O_DIRECTORY)?;
+    let dir = safe_open(
+        PROC_FILE(),
+        &dir,
+        OFLAG_PROC | OFlag::O_DIRECTORY,
+        RFLAG_PROC,
+    )?;
 
     let mut dot = 0u8;
     let mut res = vec![];
@@ -1791,7 +1832,7 @@ pub fn proc_max_open_files(pid: Pid) -> Result<LimitValue, Errno> {
     let mut path = XPathBuf::from_pid(pid);
     path.push(b"limits");
 
-    let mut file = safe_open(PROC_FILE(), &path, OFLAG_PROC)
+    let mut file = safe_open_proc(&path)
         .map(File::from)
         .or(Err(Errno::ESRCH))?;
 
@@ -2278,4 +2319,9 @@ mod tests {
             );
         }
     }
+
+    #[test]
+    fn test_proc_kernel_randomize_va_space() {
+        matches!(proc_kernel_randomize_va_space(), Ok(0 | 1 | 2));
+    }
 }
