@@ -21,9 +21,10 @@ use std::{
     ptr,
     ptr::NonNull,
     result::Result,
+    sync::LazyLock,
 };
 
-use ahash::AHashMap;
+use ahash::HashMapExt;
 use libc::c_void;
 use libseccomp::ScmpSyscall;
 use nix::{
@@ -34,14 +35,13 @@ use nix::{
     },
     unistd::{sysconf, SysconfVar},
 };
-use once_cell::sync::Lazy;
 
-use crate::config::HAVE_MADV_GUARD;
+use crate::{config::HAVE_MADV_GUARD, hash::SydHashMap};
 
 // Note mseal may not be available,
 // and libc::SYS_mseal may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_MSEAL: Lazy<Option<libc::c_long>> = Lazy::new(|| {
+static SYS_MSEAL: LazyLock<Option<libc::c_long>> = LazyLock::new(|| {
     ScmpSyscall::from_name("mseal")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -481,8 +481,8 @@ impl<K: Ord, V> SealBoxSlice<(K, V)> {
         Self::from_slice(&vec)
     }
 
-    /// Allocate an `AHashMap<K, V>` as a sorted slice of `(K, V)`.
-    pub fn from_ahashmap(map: AHashMap<K, V>) -> Result<Self, Errno>
+    /// Allocate an `SydHashMap<K, V>` as a sorted slice of `(K, V)`.
+    pub fn from_sydhashmap(map: SydHashMap<K, V>) -> Result<Self, Errno>
     where
         K: Clone,
         V: Clone,
@@ -540,9 +540,9 @@ impl<K: Ord + Clone, V: Clone> SealableSlice<(K, V)> for HashMap<K, V> {
     }
 }
 
-impl<K: Hash + Eq + Ord + Clone, V: Clone> SealableSlice<(K, V)> for AHashMap<K, V> {
+impl<K: Hash + Eq + Ord + Clone, V: Clone> SealableSlice<(K, V)> for SydHashMap<K, V> {
     fn seal(self, vma_name: Option<&CStr>) -> Result<SealedSlice<(K, V)>, Errno> {
-        SealBoxSlice::from_ahashmap(self)?.seal(vma_name)
+        SealBoxSlice::from_sydhashmap(self)?.seal(vma_name)
     }
 }
 
@@ -1423,23 +1423,23 @@ where
     }
 }
 
-/// A AHashMap-like container that can be sealed into a read-only mmap of sorted entries.
-pub enum SealableAHashMap<K, V> {
-    /// The unsealed, growable `AHashMap<K, V>` state.
-    Unsealed(AHashMap<K, V>),
+/// A HashMap-like container that can be sealed into a read-only mmap of sorted entries.
+pub enum SealableSydHashMap<K, V> {
+    /// The unsealed, growable `SydHashMap<K, V>` state.
+    Unsealed(SydHashMap<K, V>),
     /// The sealed, read-only slice state of sorted `(K, V)` pairs.
     Sealed(SealedSlice<(K, V)>),
 }
 
-impl<K, V> SealableAHashMap<K, V> {
-    /// Create from an existing `HashMap<K, V>`.
-    pub fn new(map: AHashMap<K, V>) -> Self {
-        SealableAHashMap::Unsealed(map)
+impl<K, V> SealableSydHashMap<K, V> {
+    /// Create from an existing `SydHashMap<K, V>`.
+    pub fn new(map: SydHashMap<K, V>) -> Self {
+        SealableSydHashMap::Unsealed(map)
     }
 
     /// Create with a specified capacity.
     pub fn with_capacity(cap: usize) -> Self {
-        SealableAHashMap::Unsealed(AHashMap::with_capacity(cap))
+        SealableSydHashMap::Unsealed(SydHashMap::with_capacity(cap))
     }
 
     /// Seal into a read-only mmap of sorted `(K, V)` pairs.
@@ -1451,19 +1451,19 @@ impl<K, V> SealableAHashMap<K, V> {
         V: Clone,
     {
         match self {
-            SealableAHashMap::Unsealed(map) => {
-                let sealed = SealBoxSlice::from_ahashmap(map)?.seal(vma_name)?;
-                Ok(SealableAHashMap::Sealed(sealed))
+            SealableSydHashMap::Unsealed(map) => {
+                let sealed = SealBoxSlice::from_sydhashmap(map)?.seal(vma_name)?;
+                Ok(SealableSydHashMap::Sealed(sealed))
             }
-            SealableAHashMap::Sealed(_) => Err(Errno::EPERM),
+            SealableSydHashMap::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
     /// Returns the number of elements.
     pub fn len(&self) -> usize {
         match self {
-            SealableAHashMap::Unsealed(m) => m.len(),
-            SealableAHashMap::Sealed(s) => s.len(),
+            SealableSydHashMap::Unsealed(m) => m.len(),
+            SealableSydHashMap::Sealed(s) => s.len(),
         }
     }
 
@@ -1481,8 +1481,8 @@ impl<K, V> SealableAHashMap<K, V> {
         Q: Ord + Eq + Hash + ?Sized,
     {
         match self {
-            SealableAHashMap::Unsealed(m) => m.get(key),
-            SealableAHashMap::Sealed(s) => s
+            SealableSydHashMap::Unsealed(m) => m.get(key),
+            SealableSydHashMap::Sealed(s) => s
                 .binary_search_by(|(k, _)| k.borrow().cmp(key))
                 .ok()
                 .map(|idx| &s[idx].1),
@@ -1507,8 +1507,8 @@ impl<K, V> SealableAHashMap<K, V> {
         K: Eq + Hash,
     {
         match self {
-            SealableAHashMap::Unsealed(m) => Ok(m.insert(key, value)),
-            SealableAHashMap::Sealed(_) => Err(Errno::EPERM),
+            SealableSydHashMap::Unsealed(m) => Ok(m.insert(key, value)),
+            SealableSydHashMap::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1521,14 +1521,14 @@ impl<K, V> SealableAHashMap<K, V> {
         Q: Eq + Hash + ?Sized,
     {
         match self {
-            SealableAHashMap::Unsealed(m) => m.remove(key),
-            SealableAHashMap::Sealed(_) => None,
+            SealableSydHashMap::Unsealed(m) => m.remove(key),
+            SealableSydHashMap::Sealed(_) => None,
         }
     }
 
     /// Clears the map if unsealed; does nothing if sealed.
     pub fn clear(&mut self) {
-        if let SealableAHashMap::Unsealed(m) = self {
+        if let SealableSydHashMap::Unsealed(m) = self {
             m.clear();
         }
     }
@@ -1541,11 +1541,11 @@ impl<K, V> SealableAHashMap<K, V> {
         K: Eq + Hash,
     {
         match self {
-            SealableAHashMap::Unsealed(m) => {
+            SealableSydHashMap::Unsealed(m) => {
                 m.reserve(additional);
                 Ok(())
             }
-            SealableAHashMap::Sealed(_) => Err(Errno::EPERM),
+            SealableSydHashMap::Sealed(_) => Err(Errno::EPERM),
         }
     }
 
@@ -1553,8 +1553,8 @@ impl<K, V> SealableAHashMap<K, V> {
     /// for sealed, the number of entries.
     pub fn capacity(&self) -> usize {
         match self {
-            SealableAHashMap::Unsealed(m) => m.capacity(),
-            SealableAHashMap::Sealed(s) => s.len(),
+            SealableSydHashMap::Unsealed(m) => m.capacity(),
+            SealableSydHashMap::Sealed(s) => s.len(),
         }
     }
 
@@ -1564,26 +1564,26 @@ impl<K, V> SealableAHashMap<K, V> {
         F: FnMut(&K, &mut V) -> bool,
     {
         match self {
-            SealableAHashMap::Unsealed(m) => {
+            SealableSydHashMap::Unsealed(m) => {
                 m.retain(f);
                 Ok(())
             }
-            SealableAHashMap::Sealed(_) => Err(Errno::EPERM),
+            SealableSydHashMap::Sealed(_) => Err(Errno::EPERM),
         }
     }
 }
 
-impl<K, V> Default for SealableAHashMap<K, V> {
+impl<K, V> Default for SealableSydHashMap<K, V> {
     fn default() -> Self {
-        SealableAHashMap::Unsealed(AHashMap::new())
+        SealableSydHashMap::Unsealed(SydHashMap::new())
     }
 }
 
-impl<K: fmt::Debug, V: fmt::Debug> fmt::Debug for SealableAHashMap<K, V> {
+impl<K: fmt::Debug, V: fmt::Debug> fmt::Debug for SealableSydHashMap<K, V> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            SealableAHashMap::Unsealed(m) => m.fmt(f),
-            SealableAHashMap::Sealed(s) => {
+            SealableSydHashMap::Unsealed(m) => m.fmt(f),
+            SealableSydHashMap::Sealed(s) => {
                 let mut dm = f.debug_map();
                 for pair in s.iter() {
                     dm.entry(&pair.0, &pair.1);
@@ -1594,20 +1594,20 @@ impl<K: fmt::Debug, V: fmt::Debug> fmt::Debug for SealableAHashMap<K, V> {
     }
 }
 
-impl<K: Ord + Eq + Hash, V: PartialEq> PartialEq for SealableAHashMap<K, V> {
+impl<K: Ord + Eq + Hash, V: PartialEq> PartialEq for SealableSydHashMap<K, V> {
     fn eq(&self, other: &Self) -> bool {
         if self.len() != other.len() {
             return false;
         }
         match self {
-            SealableAHashMap::Unsealed(m) => {
+            SealableSydHashMap::Unsealed(m) => {
                 for (k, v) in m {
                     if other.get(k) != Some(v) {
                         return false;
                     }
                 }
             }
-            SealableAHashMap::Sealed(s) => {
+            SealableSydHashMap::Sealed(s) => {
                 for (k, v) in s.iter() {
                     if other.get(k) != Some(v) {
                         return false;
@@ -1619,15 +1619,15 @@ impl<K: Ord + Eq + Hash, V: PartialEq> PartialEq for SealableAHashMap<K, V> {
     }
 }
 
-impl<K: Ord + Eq + Hash, V: Eq> Eq for SealableAHashMap<K, V> {}
+impl<K: Ord + Eq + Hash, V: Eq> Eq for SealableSydHashMap<K, V> {}
 
-impl<K, V> From<AHashMap<K, V>> for SealableAHashMap<K, V> {
-    fn from(m: AHashMap<K, V>) -> Self {
-        SealableAHashMap::Unsealed(m)
+impl<K, V> From<SydHashMap<K, V>> for SealableSydHashMap<K, V> {
+    fn from(m: SydHashMap<K, V>) -> Self {
+        SealableSydHashMap::Unsealed(m)
     }
 }
 
-impl<'a, K, V> IntoIterator for &'a SealableAHashMap<K, V>
+impl<'a, K, V> IntoIterator for &'a SealableSydHashMap<K, V>
 where
     K: Borrow<K> + fmt::Debug + Ord + Eq + Hash,
 {
@@ -1636,8 +1636,8 @@ where
 
     fn into_iter(self) -> Self::IntoIter {
         match self {
-            SealableAHashMap::Unsealed(m) => Box::new(m.iter()),
-            SealableAHashMap::Sealed(s) => Box::new(s.iter().map(|(k, v)| (k, v))),
+            SealableSydHashMap::Unsealed(m) => Box::new(m.iter()),
+            SealableSydHashMap::Sealed(s) => Box::new(s.iter().map(|(k, v)| (k, v))),
         }
     }
 }
@@ -1683,9 +1683,9 @@ unsafe impl<K: Send, V: Send> Send for SealableHashMap<K, V> {}
 unsafe impl<K: Send, V: Sync> Sync for SealableHashMap<K, V> {}
 
 // SAFETY: ditto.
-unsafe impl<K: Send, V: Send> Send for SealableAHashMap<K, V> {}
+unsafe impl<K: Send, V: Send> Send for SealableSydHashMap<K, V> {}
 // SAFETY: ditto.
-unsafe impl<K: Send, V: Sync> Sync for SealableAHashMap<K, V> {}
+unsafe impl<K: Send, V: Sync> Sync for SealableSydHashMap<K, V> {}
 
 #[cfg(test)]
 mod tests {
@@ -1838,14 +1838,14 @@ mod tests {
     }
 
     #[test]
-    fn test_seal_ahashmap() {
+    fn test_seal_sydhashmap() {
         if !check_mseal_support() {
             return;
         }
-        let mut m = AHashMap::new();
+        let mut m = SydHashMap::new();
         m.insert("x", 100);
         m.insert("y", 200);
-        let sb: SealBoxSlice<(&str, i32)> = SealBoxSlice::from_ahashmap(m).unwrap();
+        let sb: SealBoxSlice<(&str, i32)> = SealBoxSlice::from_sydhashmap(m).unwrap();
         let s = sb.seal(None).unwrap();
         assert_eq!(&*s, &[("x", 100), ("y", 200)]);
         let (ptr, size) = s.mapping();
@@ -2123,28 +2123,28 @@ mod tests {
     }
 
     #[test]
-    fn test_ahashmap_default_and_new() {
+    fn test_sydhashmap_default_and_new() {
         if !check_mseal_support() {
             return;
         }
-        let m: SealableAHashMap<i32, i32> = SealableAHashMap::default();
+        let m: SealableSydHashMap<i32, i32> = SealableSydHashMap::default();
         assert!(m.is_empty());
         assert_eq!(m.len(), 0);
 
-        let mut base = AHashMap::new();
+        let mut base = SydHashMap::new();
         base.insert(1, 10);
-        let m2 = SealableAHashMap::new(base.clone());
+        let m2 = SealableSydHashMap::new(base.clone());
         assert!(!m2.is_empty());
         assert_eq!(m2.len(), 1);
         assert_eq!(m2.get(&1), Some(&10));
     }
 
     #[test]
-    fn test_ahashmap_unsealed_insert_get_remove() {
+    fn test_sydhashmap_unsealed_insert_get_remove() {
         if !check_mseal_support() {
             return;
         }
-        let mut m = SealableAHashMap::with_capacity(4);
+        let mut m = SealableSydHashMap::with_capacity(4);
         assert!(m.is_empty());
 
         assert_eq!(m.insert(1, 100).unwrap(), None);
@@ -2158,11 +2158,13 @@ mod tests {
     }
 
     #[test]
-    fn test_ahashmap_unsealed_clear_reserve_capacity() {
+    fn test_sydhashmap_unsealed_clear_reserve_capacity() {
         if !check_mseal_support() {
             return;
         }
-        let mut m = SealableAHashMap::new(AHashMap::from([(2, 20), (3, 30)]));
+        let mut m = SydHashMap::new();
+        m.extend([(2, 20), (3, 30)]);
+        let mut m = SealableSydHashMap::from(m);
         let old_cap = m.capacity();
         m.reserve(10).unwrap();
         assert!(m.capacity() >= old_cap);
@@ -2173,11 +2175,11 @@ mod tests {
     }
 
     #[test]
-    fn test_ahashmap_seal_empty_and_sealed_properties() {
+    fn test_sydhashmap_seal_empty_and_sealed_properties() {
         if !check_mseal_support() {
             return;
         }
-        let m: SealableAHashMap<&str, i32> = SealableAHashMap::default();
+        let m: SealableSydHashMap<&str, i32> = SealableSydHashMap::default();
         let sealed = m.seal(None).unwrap();
 
         assert!(sealed.is_empty());
@@ -2188,9 +2190,10 @@ mod tests {
     }
 
     #[test]
-    fn test_ahashmap_seal_and_readonly_behavior() {
-        let base = AHashMap::from([("a", 1), ("b", 2), ("c", 3)]);
-        let m = SealableAHashMap::new(base);
+    fn test_sydhashmap_seal_and_readonly_behavior() {
+        let mut base = SydHashMap::new();
+        base.extend([("a", 1), ("b", 2), ("c", 3)]);
+        let m = SealableSydHashMap::new(base);
         let mut sealed = m.seal(None).unwrap();
 
         assert_eq!(sealed.len(), 3);
@@ -2207,18 +2210,18 @@ mod tests {
     }
 
     #[test]
-    fn test_ahashmap_iter_unsealed_and_sealed() {
+    fn test_sydhashmap_iter_unsealed_and_sealed() {
         if !check_mseal_support() {
             return;
         }
         let entries = vec![(10, "x"), (5, "y"), (20, "z")];
-        let mut map = AHashMap::new();
+        let mut map = SydHashMap::new();
         for &(k, v) in &entries {
             map.insert(k, v);
         }
 
-        let unsealed = SealableAHashMap::new(map.clone());
-        let sealed = SealableAHashMap::new(map).seal(None).unwrap();
+        let unsealed = SealableSydHashMap::new(map.clone());
+        let sealed = SealableSydHashMap::new(map).seal(None).unwrap();
 
         let mut u: Vec<_> = unsealed.into_iter().map(|(&k, &v)| (k, v)).collect();
         let mut s: Vec<_> = sealed.into_iter().map(|(&k, &v)| (k, v)).collect();
@@ -2233,24 +2236,24 @@ mod tests {
     }
 
     #[test]
-    fn test_ahashmap_partial_eq_and_eq() {
+    fn test_sydhashmap_partial_eq_and_eq() {
         if !check_mseal_support() {
             return;
         }
-        let mut h1 = AHashMap::new();
+        let mut h1 = SydHashMap::new();
         h1.insert("k1", 100);
         h1.insert("k2", 200);
 
-        let m1 = SealableAHashMap::new(h1.clone());
-        let m2 = SealableAHashMap::new(h1.clone());
+        let m1 = SealableSydHashMap::new(h1.clone());
+        let m2 = SealableSydHashMap::new(h1.clone());
         assert_eq!(m1, m2);
 
-        let sealed1 = SealableAHashMap::new(h1.clone()).seal(None).unwrap();
+        let sealed1 = SealableSydHashMap::new(h1.clone()).seal(None).unwrap();
         assert_eq!(m2, sealed1);
 
         let mut h3 = h1.clone();
         h3.insert("k3", 300);
-        let m3 = SealableAHashMap::new(h3);
+        let m3 = SealableSydHashMap::new(h3);
         assert_ne!(m3, sealed1);
     }
 
@@ -2315,10 +2318,12 @@ mod tests {
         assert_eq!(err, Errno::EPERM);
     }
 
-    // Tests for SealableAHashMap::retain
+    // Tests for SealableSydHashMap::retain
     #[test]
-    fn test_ahashmap_retain_removes_small_values() {
-        let mut sahm = SealableAHashMap::new(AHashMap::from([(10, 1), (20, 2), (30, 3), (40, 4)]));
+    fn test_sydhashmap_retain_removes_small_values() {
+        let mut sahm = SydHashMap::new();
+        sahm.extend([(10, 1), (20, 2), (30, 3), (40, 4)]);
+        let mut sahm = SealableSydHashMap::new(sahm);
         // Keep only entries with value > 2
         sahm.retain(|_, v| *v > 2).unwrap();
         let mut collected: Vec<_> = sahm.into_iter().map(|(&k, &v)| (k, v)).collect();
@@ -2328,13 +2333,15 @@ mod tests {
 
     #[ignore]
     #[test]
-    fn test_ahashmap_retain_mutates_keys_and_values() {
+    fn test_sydhashmap_retain_mutates_keys_and_values() {
         // Note: mutation applies only to values
-        let mut sahm = SealableAHashMap::new(AHashMap::from([
+        let mut sahm = SydHashMap::new();
+        sahm.extend([
             ("one".to_string(), 1),
             ("two".to_string(), 2),
             ("three".to_string(), 3),
-        ]));
+        ]);
+        let mut sahm = SealableSydHashMap::new(sahm);
         // For entries with even length keys, add 10 to the value and keep
         sahm.retain(|k, v| {
             if k.len() % 2 == 0 {
@@ -2350,8 +2357,10 @@ mod tests {
     }
 
     #[test]
-    fn test_ahashmap_retain_all_and_none() {
-        let mut sahm = SealableAHashMap::new(AHashMap::from([(1, 100)]));
+    fn test_sydhashmap_retain_all_and_none() {
+        let mut sahm = SydHashMap::new();
+        sahm.extend([(1, 100)]);
+        let mut sahm = SealableSydHashMap::new(sahm);
         sahm.retain(|_, _| true).unwrap();
         assert_eq!(sahm.len(), 1);
         sahm.retain(|_, _| false).unwrap();
@@ -2359,15 +2368,17 @@ mod tests {
     }
 
     #[test]
-    fn test_ahashmap_retain_on_empty_map() {
-        let mut sahm: SealableAHashMap<u8, u8> = SealableAHashMap::default();
+    fn test_sydhashmap_retain_on_empty_map() {
+        let mut sahm: SealableSydHashMap<u8, u8> = SealableSydHashMap::default();
         sahm.retain(|_, _| unreachable!()).unwrap();
         assert!(sahm.is_empty());
     }
 
     #[test]
-    fn test_ahashmap_sealed_retain_error() {
-        let sahm = SealableAHashMap::new(AHashMap::from([(0, 0)]));
+    fn test_sydhashmap_sealed_retain_error() {
+        let mut sahm = SydHashMap::new();
+        sahm.extend([(0, 0)]);
+        let sahm = SealableSydHashMap::new(sahm);
         let mut sealed = sahm.seal(None).unwrap();
         let err = sealed.retain(|_, _| true).unwrap_err();
         assert_eq!(err, Errno::EPERM);
