@@ -283,7 +283,7 @@ impl RandTimer {
         let repr = self.proc();
         let data = repr.as_bytes();
 
-        let fd = create_memfd(b"syd-upt\0", *SAFE_MFD_FLAGS)?;
+        let fd = create_memfd(b"syd-proc-uptime\0", *SAFE_MFD_FLAGS)?;
         let mut file = File::from(fd);
 
         file.write_all(data).or(Err(Errno::EIO))?;
