@@ -6,13 +6,10 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-#![allow(clippy::disallowed_types)]
-
 use std::{
     env,
     fs::remove_dir_all,
     ops::Range,
-    path::{Path, PathBuf},
     process::{Command, ExitCode},
 };
 
@@ -20,9 +17,14 @@ use nix::{
     errno::Errno,
     fcntl::AT_FDCWD,
     sys::stat::{fchmodat, umask, FchmodatFlags, Mode},
-    unistd::{chdir, mkdir, mkdtemp},
+    unistd::{chdir, getcwd, mkdir, mkdtemp},
+};
+use syd::{
+    err::SydResult,
+    path::{XPath, XPathBuf},
+    syslog::LogLevel,
+    wildmatch::inamematch,
 };
-use syd::{err::SydResult, syslog::LogLevel, wildmatch::inamematch};
 
 use crate::util::shuffle_vec;
 
@@ -32,15 +34,15 @@ use test::*;
 
 #[derive(Debug)]
 struct TempDir {
-    path: PathBuf,
+    path: XPathBuf,
     keep: bool,
 }
 
 impl TempDir {
-    fn new<P: AsRef<Path>>(path: P) -> Self {
+    fn new(path: &XPath) -> Self {
         TempDir {
-            path: path.as_ref().to_path_buf(),
-            keep: false,
+            path: path.to_owned(),
+            keep: std::env::var_os("SYD_TEST_KEEP").is_some(),
         }
     }
 }
@@ -119,7 +121,7 @@ fn main() -> SydResult<ExitCode> {
     umask(Mode::from_bits_truncate(0o077));
 
     println!("# syd-test: Welcome to the Machine!");
-    println!("# usage: syd-test [-hlq] [<name-regex>|<number>|<number>..<number>]..");
+    println!("# usage: syd-test [-hlq] [<name-glob>|<number>|<number>..<number>]..");
 
     let args = std::env::args().skip(1).collect::<Vec<_>>();
     let mut args_is_empty = args.is_empty();
@@ -161,10 +163,10 @@ fn main() -> SydResult<ExitCode> {
             "{}/syd_test_XXXXXX",
             env::var("SYD_TEST_TMPDIR").unwrap_or(".".to_string())
         );
-        match mkdtemp(Path::new(&tmp)) {
+        match mkdtemp(XPath::new(&tmp)) {
             Ok(path) => {
                 // HOME must be canonicalized!
-                let path = path.canonicalize()?;
+                let path = path.canonicalize().map(XPathBuf::from)?;
 
                 match chdir(&path) {
                     Ok(_) => {
@@ -172,7 +174,7 @@ fn main() -> SydResult<ExitCode> {
                         println!("# Use SYD_TEST_TMPDIR to override.");
                         //Setting HOME here breaks podman!
                         //env::set_var("HOME", &path);
-                        Some(TempDir::new(path))
+                        Some(TempDir::new(&path))
                     }
                     Err(error) => {
                         println!("# chdir failed: {error}.");
@@ -295,7 +297,12 @@ fn main() -> SydResult<ExitCode> {
         return Ok(ExitCode::from(0));
     }
     let ntest = test_indices.len();
-    println!("1..{ntest}");
+    let ptest = if env::var_os("SYD_TEST_REEXEC").is_none() {
+        ""
+    } else {
+        "#    "
+    };
+    println!("{ptest}1..{ntest}");
 
     let exp_test = env::var_os("SYD_TEST_EXPENSIVE").is_some();
     let mut fail_hard = 0;
@@ -315,6 +322,7 @@ fn main() -> SydResult<ExitCode> {
         idx += 1;
         #[allow(clippy::disallowed_methods)]
         let name = name.strip_prefix("test_syd_").unwrap();
+        env::set_var("SYD_TEST_NAME", name);
         if exp_test && !name.starts_with("exp_") {
             println!("# ok {idx} - {name} # SKIP not expensive, unset SYD_TEST_EXPENSIVE to run");
             skip += 1;
@@ -365,14 +373,21 @@ fn main() -> SydResult<ExitCode> {
                     fail_soft += 1;
                     soft_fails.push(name.to_string());
                     std::env::remove_var("SYD_TEST_SOFT_FAIL");
-                    println!("ok {idx} - {name} # TODO");
+                    println!("{ptest}ok {idx} - {name} # TODO");
                 } else {
-                    println!("ok {idx} - {name}");
+                    println!("{ptest}ok {idx} - {name}");
+                }
+                if tmpdir.as_ref().map(|t| t.keep).unwrap_or(false) {
+                    let cwd = getcwd()
+                        .map(XPathBuf::from)
+                        .unwrap_or_else(|_| XPathBuf::from("?"));
+                    eprintln!("# Keeping test directory \"{cwd}\"");
+                } else {
+                    let _ = remove_dir_all(name);
                 }
-                let _ = remove_dir_all(name);
             }
             Err(error) => {
-                println!("not ok {idx} - {name} - FAIL: {error}");
+                println!("{ptest}not ok {idx} - {name} - FAIL: {error}");
                 fail_hard += 1;
                 fail_names.push(name.to_string());
                 if fail_quick {
