@@ -20,6 +20,7 @@ use std::{
     os::fd::RawFd,
     path::{Path, PathBuf},
     process::{Child, Command, ExitStatus, Output, Stdio},
+    sync::LazyLock,
     time::Duration,
 };
 
@@ -30,7 +31,6 @@ use nix::{
         utsname::uname,
     },
 };
-use once_cell::sync::Lazy;
 use syd::config::*;
 
 #[derive(Debug)]
@@ -196,12 +196,19 @@ macro_rules! assert_status_bad_message {
 }
 
 #[macro_export]
-macro_rules! assert_status_denied {
+macro_rules! assert_status_access_denied {
     ($status:expr) => {
         assert_status_code!($status, libc::EACCES);
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_permission_denied {
+    ($status:expr) => {
+        assert_status_code!($status, libc::EPERM);
+    };
+}
+
 #[macro_export]
 macro_rules! assert_status_hidden {
     ($status:expr) => {
@@ -341,6 +348,17 @@ macro_rules! skip_if_root {
     }};
 }
 
+#[macro_export]
+macro_rules! skip_unless_iproute2 {
+    () => {{
+        if !check_iproute2() {
+            eprintln!("ip is not iproute2, cannot run test, skipping!");
+            std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+            return Ok(());
+        }
+    }};
+}
+
 #[macro_export]
 macro_rules! skip_unless_linux {
     ($need_major:expr, $need_minor:expr) => {{
@@ -392,13 +410,62 @@ macro_rules! skip_unless_stdout_is_a_tty {
 macro_rules! skip_unless_kernel_crypto_is_supported {
     () => {{
         let key = syd::hash::Key::random().unwrap();
-        match syd::hash::aes_ctr_setup(key.as_ref()) {
+        let key_id = match syd::hash::add_key(
+            "user",
+            "SYD-3-CRYPT-TEST",
+            key.as_ref(),
+            syd::hash::KEY_SPEC_USER_KEYRING,
+        ) {
+            Ok(key_id) => key_id,
+            Err(nix::errno::Errno::EAFNOSUPPORT | nix::errno::Errno::ENOSYS) => {
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(nix::errno::Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(errno) => {
+                return Err(TestError(format!(
+                    "Failed to test for Linux keyrings(7) API: {errno}"
+                )));
+            }
+        };
+        match syd::hash::aes_ctr_setup(key_id) {
             Ok(fd) => drop(fd),
             Err(nix::errno::Errno::EAFNOSUPPORT) => {
                 eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
                 std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
                 return Ok(());
             }
+            Err(nix::errno::Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(errno) => {
+                return Err(TestError(format!(
+                    "Failed to test for Linux Kernel Cryptography API: {errno}"
+                )));
+            }
+        }
+        match syd::hash::hmac_sha256_setup(key_id) {
+            Ok(fd) => drop(fd),
+            Err(nix::errno::Errno::EAFNOSUPPORT) => {
+                eprintln!("Test requires Linux Kernel Cryptography API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
+            Err(nix::errno::Errno::EACCES) => {
+                eprintln!("Is your session keyring attached to your user keyring?");
+                eprintln!("Test requires Linux keyrings(7) API, skipping!");
+                std::env::set_var("SYD_TEST_SOFT_FAIL", "1");
+                return Ok(());
+            }
             Err(errno) => {
                 return Err(TestError(format!(
                     "Failed to test for Linux Kernel Cryptography API: {errno}"
@@ -647,122 +714,133 @@ macro_rules! skip_unless_at_execve_check_is_supported {
 
 /// Holds a `String` to run `syd`.
 /// Honours CARGO_BIN_EXE_syd environment variable.
-pub static SYD: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd").unwrap_or("syd".to_string()));
+pub static SYD: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd").unwrap_or("syd".to_string()));
 
 /// Holds a `String` to run `syd-aux`.
 /// Honours CARGO_BIN_EXE_syd-aux environment variable.
-pub static SYD_AUX: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-aux").unwrap_or("syd-aux".to_string()));
+pub static SYD_AUX: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-aux").unwrap_or("syd-aux".to_string()));
 
 /// Holds a `String` to run `syd-bit`.
 /// Honours CARGO_BIN_EXE_syd-aux environment variable.
-pub static SYD_BIT: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-bit").unwrap_or("syd-bit".to_string()));
+pub static SYD_BIT: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-bit").unwrap_or("syd-bit".to_string()));
 
 /// Holds a `String` to run `syd-cap`.
 /// Honours CARGO_BIN_EXE_syd-cap environment variable.
-pub static SYD_CAP: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-cap").unwrap_or("syd-cap".to_string()));
+pub static SYD_CAP: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-cap").unwrap_or("syd-cap".to_string()));
 
 /// Holds a `String` to run `syd-dns`.
 /// Honours CARGO_BIN_EXE_syd-dns environment variable.
-pub static SYD_DNS: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-dns").unwrap_or("syd-dns".to_string()));
+pub static SYD_DNS: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-dns").unwrap_or("syd-dns".to_string()));
 
 /// Holds a `String` to run `syd-env`.
 /// Honours CARGO_BIN_EXE_syd-env environment variable.
-pub static SYD_ENV: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-env").unwrap_or("syd-env".to_string()));
+pub static SYD_ENV: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-env").unwrap_or("syd-env".to_string()));
 
 /// Holds a `String` to run `syd-exec`.
 /// Honours CARGO_BIN_EXE_syd-exec environment variable.
-pub static SYD_EXEC: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-exec").unwrap_or("syd-exec".to_string()));
+pub static SYD_EXEC: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-exec").unwrap_or("syd-exec".to_string()));
 
 /// Holds a `String` to run `syd-aes`.
 /// Honours CARGO_BIN_EXE_syd-aes environment variable.
-pub static SYD_AES: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-aes").unwrap_or("syd-aes".to_string()));
+pub static SYD_AES: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-aes").unwrap_or("syd-aes".to_string()));
 
 /// Holds a `String` to run `syd-key`.
 /// Honours CARGO_BIN_EXE_syd-key environment variable.
-pub static SYD_KEY: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-key").unwrap_or("syd-key".to_string()));
+pub static _SYD_KEY: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-key").unwrap_or("syd-key".to_string()));
 
 /// Holds a `String` to run `syd-elf`.
 /// Honours CARGO_BIN_EXE_syd-elf environment variable.
-pub static SYD_ELF: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-elf").unwrap_or("syd-elf".to_string()));
+pub static SYD_ELF: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-elf").unwrap_or("syd-elf".to_string()));
 
 /// Holds a `String` to run `syd-cpu`.
 /// Honours CARGO_BIN_EXE_syd-cpu environment variable.
-pub static SYD_CPU: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-cpu").unwrap_or("syd-cpu".to_string()));
+pub static SYD_CPU: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-cpu").unwrap_or("syd-cpu".to_string()));
 
 /// Holds a `String` to run `syd-hex`.
 /// Honours CARGO_BIN_EXE_syd-hex environment variable.
-pub static SYD_HEX: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-hex").unwrap_or("syd-hex".to_string()));
+pub static SYD_HEX: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-hex").unwrap_or("syd-hex".to_string()));
 
 /// Holds a `String` to run `syd-info`.
 /// Honours CARGO_BIN_EXE_syd-info environment variable.
-pub static SYD_INFO: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-info").unwrap_or("syd-info".to_string()));
+pub static SYD_INFO: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-info").unwrap_or("syd-info".to_string()));
 
 /// Holds a `String` to run `syd-lock`.
 /// Honours CARGO_BIN_EXE_syd-lock environment variable.
-pub static SYD_LOCK: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-lock").unwrap_or("syd-lock".to_string()));
+pub static SYD_LOCK: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-lock").unwrap_or("syd-lock".to_string()));
 
 /// Holds a `String` to run `syd-mdwe`.
 /// Honours CARGO_BIN_EXE_syd-mdwe environment variable.
-pub static SYD_MDWE: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-mdwe").unwrap_or("syd-mdwe".to_string()));
+pub static SYD_MDWE: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-mdwe").unwrap_or("syd-mdwe".to_string()));
+
+/// Holds a `String` to run `syd-ofd`.
+/// Honours CARGO_BIN_EXE_syd-ofd environment variable.
+pub static SYD_OFD: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-ofd").unwrap_or("syd-ofd".to_string()));
+
+/// Holds a `String` to run `syd-pause`.
+/// Honours CARGO_BIN_EXE_syd-pause environment variable.
+pub static SYD_PAUSE: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-pause").unwrap_or("syd-pause".to_string()));
 
 /// Holds a `String` to run `syd-pds`.
 /// Honours CARGO_BIN_EXE_syd-pds environment variable.
-pub static SYD_PDS: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-pds").unwrap_or("syd-pds".to_string()));
+pub static SYD_PDS: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-pds").unwrap_or("syd-pds".to_string()));
 
 /// Holds a `String` to run `syd-size`.
 /// Honours CARGO_BIN_EXE_syd-size environment variable.
-pub static SYD_SIZE: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-size").unwrap_or("syd-size".to_string()));
+pub static SYD_SIZE: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-size").unwrap_or("syd-size".to_string()));
 
 /// Holds a `String` to run `syd-test-do`.
 /// Honours CARGO_BIN_EXE_syd-test-do environment variable.
 /// This path is a canonicalized for ease of use in sandboxing tests.
-pub static SYD_DO: Lazy<String> = Lazy::new(|| match env::var("CARGO_BIN_EXE_syd-test-do") {
-    Ok(var) => Path::new(&var)
-        .canonicalize()
-        .expect("CARGO_BIN_EXE_syd-test-do")
-        .to_string_lossy()
-        .into_owned(),
-    Err(_) => which(if env::var("SYD_TEST_32").is_ok() {
-        "syd-test-do32"
-    } else {
-        "syd-test-do"
-    })
-    .expect("syd-test-do"),
-});
+pub static SYD_DO: LazyLock<String> =
+    LazyLock::new(|| match env::var("CARGO_BIN_EXE_syd-test-do") {
+        Ok(var) => Path::new(&var)
+            .canonicalize()
+            .expect("CARGO_BIN_EXE_syd-test-do")
+            .to_string_lossy()
+            .into_owned(),
+        Err(_) => which(if env::var("SYD_TEST_32").is_ok() {
+            "syd-test-do32"
+        } else {
+            "syd-test-do"
+        })
+        .expect("syd-test-do"),
+    });
 
 /// Holds a `String` to run `syd-x`.
 /// Honours CARGO_BIN_EXE_syd-x environment variable.
-pub static SYD_X: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-x").unwrap_or("syd-x".to_string()));
+pub static SYD_X: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-x").unwrap_or("syd-x".to_string()));
 
 #[cfg(feature = "oci")]
 /// Holds a `String` to run `syd-oci`.
 /// Honours CARGO_BIN_EXE_syd-oci environment variable.
-pub static SYD_OCI: Lazy<String> =
-    Lazy::new(|| env::var("CARGO_BIN_EXE_syd-oci").unwrap_or("syd-oci".to_string()));
+pub static SYD_OCI: LazyLock<String> =
+    LazyLock::new(|| env::var("CARGO_BIN_EXE_syd-oci").unwrap_or("syd-oci".to_string()));
 
 /// A boolean which specifies if we're running under SourceHut CI.
-pub static CI_BUILD: Lazy<bool> = Lazy::new(|| env::var("JOB_ID").ok().is_some());
+pub static CI_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("JOB_ID").ok().is_some());
 
 /// A boolean which specifies if we're running under GitLab CI.
-pub static GL_BUILD: Lazy<bool> = Lazy::new(|| env::var("CI_PROJECT_ID").ok().is_some());
+pub static GL_BUILD: LazyLock<bool> = LazyLock::new(|| env::var("CI_PROJECT_ID").ok().is_some());
 
 pub struct Syd(Command);
 
@@ -796,6 +874,11 @@ impl Syd {
         self
     }
 
+    pub fn current_dir<S: ToString>(&mut self, arg: S) -> &mut Self {
+        self.0.current_dir(arg.to_string());
+        self
+    }
+
     pub fn log(&mut self, value: &str) -> &mut Self {
         self.0
             .env(ENV_LOG, env::var(ENV_LOG).unwrap_or(value.to_string()));
@@ -905,11 +988,13 @@ impl Syd {
 /// Returns a `Command` to run `syd`.
 /// Honours CARGO_BIN_EXE_syd environment variable.
 pub fn syd() -> Syd {
-    static USE_PERF: Lazy<bool> = Lazy::new(|| env::var_os("SYD_TEST_PERF").is_some());
-    static USE_TRACE: Lazy<bool> = Lazy::new(|| env::var_os("SYD_TEST_TRACE").is_some());
-    static USE_STRACE: Lazy<bool> = Lazy::new(|| env::var_os("SYD_TEST_STRACE").is_some());
-    static USE_VALGRIND: Lazy<bool> = Lazy::new(|| env::var_os("SYD_TEST_VALGRIND").is_some());
-    static USE_HELGRIND: Lazy<bool> = Lazy::new(|| env::var_os("SYD_TEST_HELGRIND").is_some());
+    static USE_PERF: LazyLock<bool> = LazyLock::new(|| env::var_os("SYD_TEST_PERF").is_some());
+    static USE_TRACE: LazyLock<bool> = LazyLock::new(|| env::var_os("SYD_TEST_TRACE").is_some());
+    static USE_STRACE: LazyLock<bool> = LazyLock::new(|| env::var_os("SYD_TEST_STRACE").is_some());
+    static USE_VALGRIND: LazyLock<bool> =
+        LazyLock::new(|| env::var_os("SYD_TEST_VALGRIND").is_some());
+    static USE_HELGRIND: LazyLock<bool> =
+        LazyLock::new(|| env::var_os("SYD_TEST_HELGRIND").is_some());
     let mut cmd = Syd::new("timeout");
     if check_timeout_foreground() {
         cmd.arg("--foreground");
@@ -927,6 +1012,7 @@ pub fn syd() -> Syd {
         cmd.arg("--");
     } else if *USE_STRACE {
         cmd.arg("strace");
+        cmd.arg("-yyY");
 
         if env::var_os("SYD_TEST_STRACE_NOFORK").is_none() {
             cmd.arg("-f");
@@ -1064,6 +1150,8 @@ int main(int argc, char *argv[])
     // Compile the C code using the C compiler
     let compile_status = Command::new("cc")
         .arg("-Wall")
+        .arg("-Wextra")
+        .arg("-zexecstack") // required on ppc64
         .arg("nested.c")
         .arg("-o")
         .arg("nested")
@@ -1327,6 +1415,18 @@ pub fn check_timeout_foreground() -> bool {
         .arg("-sKILL")
         .arg("60s")
         .arg("true")
+        .stdout(Stdio::null())
+        .stderr(Stdio::null())
+        .status()
+        .map(|status| status.success())
+        .unwrap_or(false)
+}
+
+/// Check if the `ip` command is of `iproute2` (not `busybox`).
+pub fn check_iproute2() -> bool {
+    Command::new("sh")
+        .arg("-cex")
+        .arg("ip -V | grep -iq iproute2")
         .status()
         .map(|status| status.success())
         .unwrap_or(false)
@@ -1341,10 +1441,7 @@ pub fn current_dir(base: bool) -> std::io::Result<PathBuf> {
         // Extract the basename
         let basename = current_dir
             .file_name()
-            .ok_or(std::io::Error::new(
-                std::io::ErrorKind::Other,
-                "Failed to get the basename",
-            ))
+            .ok_or(std::io::Error::other("Failed to get the basename"))
             .map(PathBuf::from)?;
 
         Ok(basename)
