@@ -7,28 +7,30 @@
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
-    os::{fd::AsRawFd, unix::ffi::OsStrExt},
+    os::unix::ffi::OsStrExt,
     process::ExitCode,
     time::{Duration, Instant},
 };
 
+use btoi::btoi;
 use data_encoding::HEXLOWER_PERMISSIVE;
 use nix::{
+    errno::Errno,
     fcntl::{splice, OFlag, SpliceFFlags},
-    sys::socket::{sendmsg, AlgAddr, ControlMessage, MsgFlags},
     unistd::{isatty, pipe2},
 };
 use syd::{
     config::PIPE_BUF,
     err::SydResult,
     hash::{
-        aes_ctr_dec, aes_ctr_enc, aes_ctr_flush, aes_ctr_init, aes_ctr_setup, Key, BLOCK_SIZE, IV,
+        aes_ctr_dec, aes_ctr_enc, aes_ctr_flush, aes_ctr_init, aes_ctr_setup, KeySerial,
+        BLOCK_SIZE, IV,
     },
 };
 use zeroize::Zeroizing;
 
-fn process_data(encrypting: bool, key: Key, iv: IV, verbose: bool) -> SydResult<()> {
-    let fd = aes_ctr_setup(key.as_ref())?;
+fn process_data(encrypting: bool, key_id: KeySerial, iv: IV, verbose: bool) -> SydResult<()> {
+    let fd = aes_ctr_setup(key_id)?;
     let fd = aes_ctr_init(&fd, false)?;
 
     if encrypting {
@@ -111,15 +113,20 @@ fn process_data(encrypting: bool, key: Key, iv: IV, verbose: bool) -> SydResult<
         }
     }
 
-    // Signal the AF_ALG socket the end of the process (No MSG_MORE in flags).
-    let cmsgs = [ControlMessage::AlgSetOp(if encrypting {
-        &nix::libc::ALG_OP_ENCRYPT
-    } else {
-        &nix::libc::ALG_OP_DECRYPT
-    })];
-    sendmsg::<AlgAddr>(fd.as_raw_fd(), &[], &cmsgs, MsgFlags::empty(), None)?;
-
     if nread > 0 {
+        // Finalize {en,de}cryption to flush final batch with `false`.
+        //
+        // Some kernel versions may incorrectly return EINVAL here.
+        // Gracefully handle this errno and move on.
+        match if encrypting {
+            aes_ctr_enc(&fd, &[], None, false)
+        } else {
+            aes_ctr_dec(&fd, &[], None, false)
+        } {
+            Ok(_) | Err(Errno::EINVAL) => {}
+            Err(errno) => return Err(errno.into()),
+        }
+
         // {En,De}crypt the final batch.
         // SAFETY: Zero-out memory if decrypting.
         aes_ctr_flush(&fd, std::io::stdout(), buf, nread)?;
@@ -153,9 +160,8 @@ syd::main! {
 
     // Parse CLI options.
     let mut opt_encrypt = None;
-    let mut opt_key_hex = None;
+    let mut opt_key_id = None;
     let mut opt_iv_hex = None;
-    let mut opt_tag = None;
     let mut opt_verbose = false;
 
     let mut parser = lexopt::Parser::from_env();
@@ -168,34 +174,33 @@ syd::main! {
             Short('v') => opt_verbose = true,
             Short('e') => opt_encrypt = Some(true),
             Short('d') => opt_encrypt = Some(false),
-            Short('k') => opt_key_hex = Some(parser.value()?.parse::<String>()?),
+            Short('k') => opt_key_id = Some(btoi::<KeySerial>(parser.value()?.as_bytes())?),
             Short('i') => opt_iv_hex = Some(parser.value()?.parse::<String>()?),
-            Short('t') => opt_tag = Some(parser.value()?),
             _ => return Err(arg.unexpected().into()),
         }
     }
 
-    if opt_encrypt.is_none() || opt_key_hex.is_none() || opt_iv_hex.is_none() {
-        eprintln!("syd-aes: Error: -e or -d and -k, -i options are required.");
+    let is_enc = if let Some(is_enc) = opt_encrypt {
+        is_enc
+    } else {
+        eprintln!("syd-aes: Error: -e or -d options are required.");
+        help();
+        return Ok(ExitCode::FAILURE);
+    };
+
+    let key_id = if let Some(key_id) = opt_key_id {
+        key_id
+    } else {
+        eprintln!("syd-aes: Error: -k option is required.");
+        help();
+        return Ok(ExitCode::FAILURE);
+    };
+
+    if opt_iv_hex.is_none() {
+        eprintln!("syd-aes: Error: -i option is required.");
         help();
         return Ok(ExitCode::FAILURE);
     }
-
-    let mut key = match opt_key_hex
-        .and_then(|hex| HEXLOWER_PERMISSIVE.decode(hex.as_bytes()).ok())
-        .and_then(|vec| vec.as_slice().try_into().ok())
-    {
-        Some(key) => Key::new(key),
-        None => {
-            eprintln!("syd-aes: Error: Key must be valid hex, and 512 bits (64 bytes) in length!");
-            return Ok(ExitCode::FAILURE);
-        }
-    };
-
-    if let Some(info) = opt_tag {
-        key = key.derive(None, info.as_bytes());
-    }
-
     let iv = match opt_iv_hex
         .and_then(|hex| HEXLOWER_PERMISSIVE.decode(hex.as_bytes()).ok())
         .and_then(|vec| vec.as_slice().try_into().ok())
@@ -207,19 +212,18 @@ syd::main! {
         }
     };
 
-    #[allow(clippy::disallowed_methods)]
-    process_data(opt_encrypt.unwrap(), key, iv, opt_verbose).map(|_| ExitCode::SUCCESS)
+    process_data(is_enc, key_id, iv, opt_verbose).map(|_| ExitCode::SUCCESS)
 }
 
 fn help() {
-    println!("Usage: syd-aes [-h] -e|-d -k <key-hex> -i <iv-hex> -t <tag-str>");
+    println!("Usage: syd-aes [-h] -e|-d -k <key-serial> -i <iv-hex>");
     println!("AES-CTR Encryption and Decryption Utility");
     println!("Reads from standard input and writes to standard output.");
     println!("  -h        Print this help message and exit.");
     println!("  -v        Enable verbose mode.");
     println!("  -e        Encrypt the input data.");
     println!("  -d        Decrypt the input data.");
-    println!("  -k <key>  Hex-encoded key (256 bits)");
+    println!("  -k <key>  Key serial ID for keyrings(7) (32-bit integer)");
+    println!("            Key must have search permission.");
     println!("  -i <iv>   Hex-encoded IV (128 bits)");
-    println!("  -t <tag>  Information tag for HKDF (optional)");
 }
