@@ -38,8 +38,10 @@ syd::main! {
         CapSet::Effective,
     ] {
         let mut vec = Vec::new();
-        for cap in syd::caps::all() {
-            #[allow(clippy::disallowed_methods)]
+
+        #[allow(clippy::disallowed_methods)]
+        for flag in syd::caps::Capabilities::all() {
+            let cap: syd::caps::Capability = flag.try_into().unwrap();
             if syd::caps::has_cap(None, set, cap).expect("syd::caps::has_cap") {
                 vec.push(serde_json::Value::String(cap.to_string()));
             }
