@@ -8,13 +8,12 @@
 
 use std::{
     os::{fd::AsRawFd, unix::process::CommandExt},
-    process::{Command, ExitCode},
+    process::Command,
 };
 
 use nix::{fcntl::OFlag, sys::stat::Mode, unistd::Uid};
-use syd::err::SydResult;
 
-fn main() -> SydResult<ExitCode> {
+syd::main! {
     syd::set_sigpipe_dfl()?;
 
     // Determine HOME directory.
