@@ -80,10 +80,12 @@ use syd::{
     compat::{waitid, WaitStatus},
     err::{SydError, SydResult},
     hook::Supervisor,
+    ignore_signals,
     log::log_init,
     path::XPath,
     sandbox::Sandbox,
     syslog::LogLevel,
+    IgnoreSignalOpts,
 };
 use tabwriter::TabWriter;
 use tracing_subscriber::layer::SubscriberExt;
@@ -179,7 +181,11 @@ impl Executor for SydExecutor {
 
         // Ignore all signals except SIG{KILL,STOP,PIPE,CHLD,Core}.
         // This is used to ensure we can deny {rt_,}sigreturn(2) to mitigate SROP.
-        syd::ignore_signals().or(Err(ExecutorError::CantHandle("Failed to ignore signals!")))?;
+        let mut opts = IgnoreSignalOpts::SkipIgnoreAlarm;
+        if sandbox.flags.allow_unsafe_prlimit() {
+            opts.insert(IgnoreSignalOpts::SkipIgnoreCoreDump);
+        }
+        ignore_signals(opts).or(Err(ExecutorError::CantHandle("Failed to ignore signals!")))?;
 
         // Run command under the Syd sandbox and exit with the return code.
         #[allow(clippy::disallowed_methods)]
@@ -2353,7 +2359,7 @@ fn get_cgroup_path(cgroups_path: &Option<PathBuf>, container_id: &str) -> PathBu
 
 // Function to query and return capabilities
 fn query_caps() -> SydResult<Vec<String>> {
-    Ok(syd::caps::all()
+    Ok(syd::caps::Capabilities::all()
         .iter()
         .map(|cap| format!("{cap:?}"))
         .collect())
