@@ -49,7 +49,7 @@ syd::main! {
     #[allow(clippy::disallowed_methods)]
     let mut opt_path = env::var("PATH").unwrap_or("/usr/bin:/bin".to_string());
     let mut opt_func = HashAlgorithm::Sha512;
-    let mut opt_action = "kill";
+    let mut opt_action = None;
     let mut opt_limit = 0usize; // 0 means no limit.
     let mut elf_set: HashSet<Filter, RandomState> = HashSet::default();
 
@@ -67,8 +67,8 @@ syd::main! {
             Short('c') => opt_func = HashAlgorithm::Crc64,
             Short('C') => opt_func = HashAlgorithm::Crc32,
             Short('m') => opt_func = HashAlgorithm::Md5,
-            Short('k') => opt_action = "kill",
-            Short('w') => opt_action = "warn",
+            Short('k') => opt_action = Some("kill"),
+            Short('w') => opt_action = Some("warn"),
             Short('p') => opt_path = parser.value()?.parse::<String>()?,
             Short('l') => opt_limit = parser.value()?.parse::<usize>()?,
             Short('s') => opt_dsyd = true,
@@ -209,14 +209,13 @@ syd::main! {
                                     // Write Integrity Force Rules.
                                     let reader = BufReader::new(file);
                                     if let Ok(key) = syd::hash::hash(reader, opt_func) {
-                                        let prefix = if opt_dsyd { "/dev/syd/" } else { "" };
-                                        println!(
-                                            "{}force+{}:{}:{}",
-                                            prefix,
-                                            path,
-                                            HEXLOWER.encode(&key),
-                                            opt_action
-                                        );
+                                        let key = HEXLOWER.encode(&key);
+                                        let pre = if opt_dsyd { "/dev/syd/" } else { "" };
+                                        if let Some(act) = opt_action {
+                                            println!("{pre}force+{path}:{key}:{act}");
+                                        } else {
+                                            println!("{pre}force+{path}:{key}");
+                                        }
                                         if opt_limit > 0 {
                                             count += 1;
                                             if count >= opt_limit {
