@@ -12,7 +12,6 @@ use std::{
     process::{exit, ExitCode},
 };
 
-use libc::{TCGETS, TCSETS, TCSETSF, TCSETSW, TIOCGWINSZ, TIOCSWINSZ};
 use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
 use nix::{
     errno::Errno,
@@ -21,6 +20,7 @@ use nix::{
     sched::{unshare, CloneFlags},
     sys::{
         epoll::{Epoll, EpollCreateFlags, EpollEvent, EpollFlags},
+        prctl::set_dumpable,
         signal::{signal, sigprocmask, SigHandler, SigmaskHow, Signal},
         signalfd::{SfdFlags, SigSet, SignalFd},
         termios::{cfmakeraw, tcgetattr, tcsetattr, OutputFlags, SetArg, Termios},
@@ -29,9 +29,16 @@ use nix::{
 };
 use syd::{
     compat::epoll_ctl_safe,
+    confine::{confine_mdwe, extend_ioctl},
     err::SydResult,
-    fs::{duprand, set_nonblock, winsize_get, winsize_set},
+    fs::{closeexcept, duprand, set_exclusive, set_nonblock, winsize_get, winsize_set},
+    ignore_signals,
+    ioctl::{
+        TCGETS, TCGETS2, TCSETS, TCSETS2, TCSETSF, TCSETSF2, TCSETSW, TCSETSW2, TIOCGWINSZ,
+        TIOCSWINSZ,
+    },
     landlock_policy::LandlockPolicy,
+    IgnoreSignalOpts,
 };
 
 // This is from <linux/tty.h>
@@ -41,34 +48,29 @@ const N_TTY_BUF_SIZE: usize = 4096;
 // syd::config::PIPE_BUF may be larger...
 const PIPE_BUF: usize = N_TTY_BUF_SIZE;
 
-#[allow(clippy::disallowed_methods)]
-fn main() -> SydResult<ExitCode> {
+syd::main! {
     // Parse options.
     let opts = parse_options()?;
 
-    // Ignore all signals except SIG{KILL,STOP,PIPE,CHLD,Core}.
+    // Ignore all signals except SIG{KILL,STOP,PIPE,CHLD}.
     // This is used to ensure we can deny {rt_,}sigreturn(2) to mitigate SROP.
-    syd::ignore_signals()?;
+    ignore_signals(IgnoreSignalOpts::empty())?;
 
     // Close all file descriptors, except:
     // 1. Standard input, output, and error.
     // 2. The PID fd and the PTY fd passed by the Syd process.
-    let max_fd = std::cmp::max(opts.fpid.as_raw_fd(), opts.fpty.as_raw_fd());
-    for fd in 3..=max_fd {
-        if fd == opts.fpid.as_raw_fd() {
-            continue;
-        }
-
-        if fd == opts.fpty.as_raw_fd() {
-            continue;
-        }
-
-        // SAFETY: Nothing can be done on close errors.
-        let _ = close(fd);
+    // Nothing can be done on closeexcept errors.
+    // We do it early here so FD randomization doesn't effect performance.
+    #[allow(clippy::cast_sign_loss)]
+    {
+        let fd1 = opts.fpid.as_raw_fd() as libc::c_uint;
+        let fd2 = opts.fpty.as_raw_fd() as libc::c_uint;
+        let _ = closeexcept(&if fd1 < fd2 {
+            [0, 1, 2, fd1, fd2]
+        } else {
+            [0, 1, 2, fd2, fd1]
+        });
     }
-    // SAFETY: nix does not have a close_range wrapper yet.
-    // Nothing can be done on close_range errors.
-    let _ = unsafe { libc::syscall(libc::SYS_close_range, max_fd + 1, RawFd::MAX, 0) };
 
     let PtyOpts {
         fpid,
@@ -143,6 +145,9 @@ fn main() -> SydResult<ExitCode> {
         unsafe { OwnedFd::from_raw_fd(fd) }
     })?;
 
+    // SAFETY: Set PTY to exclusive mode to harden against sniffing.
+    set_exclusive(&fpty, true)?;
+
     // Set PTY fd non-blocking.
     set_nonblock(&fpty, true)?;
 
@@ -422,21 +427,41 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
     )?;
 
     // Allow ioctl(2) requests:
-    // 1. TCGETS, aka tcgetattr(3)
-    // 2. TCSETS, aka tcsetattr(3) with TCSANOW
-    // 3. TCSETSW, aka tcsetattr(3) with TCSADRAIN
-    // 4. TCSETSF, aka tcsetattr(3) with TCSAFLUSH
+    // 1. TCGETS{,2}, aka tcgetattr(3)
+    // 2. TCSETS{,2}, aka tcsetattr(3) with TCSANOW
+    // 3. TCSETSW{,2}, aka tcsetattr(3) with TCSADRAIN
+    // 4. TCSETSF{,2}, aka tcsetattr(3) with TCSAFLUSH
     // 5. TIOCGWINSZ, aka winsize_get
     // 6. TIOCSWINSZ, aka winsize_set
+    //
+    // For *2, we use hardcoded versions because not all libcs define them.
+    // They are portable as `struct termios2` has the same size
+    // across 32-bit and 64-bit architectures.
     #[allow(clippy::disallowed_methods)]
     let syscall = ScmpSyscall::from_name("ioctl").unwrap();
     #[allow(clippy::unnecessary_cast)]
-    for request in [TCGETS, TCSETS, TCSETSW, TCSETSF, TIOCGWINSZ, TIOCSWINSZ] {
-        ctx.add_rule_conditional(
-            ScmpAction::Allow,
-            syscall,
-            &[scmp_cmp!($arg1 == request as u64)],
-        )?;
+    for request in [
+        *TCGETS,
+        *TCGETS2,
+        *TCSETS,
+        *TCSETS2,
+        *TCSETSW,
+        *TCSETSW2,
+        *TCSETSF,
+        *TCSETSF2,
+        *TIOCGWINSZ,
+        *TIOCSWINSZ,
+    ] {
+        let request = if let Some(request) = request {
+            request as u64
+        } else {
+            continue;
+        };
+
+        ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg1 == request)])?;
+        if let Some(request) = extend_ioctl(request) {
+            ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg1 == request)])?;
+        }
     }
 
     // Allow safe fcntl(2) utility calls.
@@ -456,7 +481,7 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
     chdir("/proc/self/fdinfo")?;
 
     if !dry_run {
-        // SAFETY: Default panic hook wont play well with seccomp.
+        // SAFETY: Default panic hook won't play well with seccomp.
         std::panic::set_hook(Box::new(|_| {}));
 
         // Set up namespace isolation for all available namespaces.
@@ -488,14 +513,10 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
 
         // Set up Memory-Deny-Write-Execute protections.
         // Ignore errors as PR_SET_MDWE may not be supported.
-        const PR_SET_MDWE: libc::c_int = 65;
-        const PR_MDWE_REFUSE_EXEC_GAIN: libc::c_ulong = 1;
-        // SAFETY: nix has no wrapper for PR_SET_MDWE.
-        let _ = unsafe { libc::prctl(PR_SET_MDWE, PR_MDWE_REFUSE_EXEC_GAIN, 0, 0, 0) };
+        let _ = confine_mdwe(false);
 
         // Set the process dumpable attribute to not-dumpable.
-        // SAFETY: Our nix version does not have a wrapper for set_dumpable yet.
-        let _ = unsafe { libc::prctl(libc::PR_SET_DUMPABLE, 0, 0, 0, 0) };
+        let _ = set_dumpable(false);
 
         // Deny reading the timestamp counter (x86 only).
         // SAFETY: Our nix version does not have a wrapper for SET_TSC yet.
