@@ -34,8 +34,8 @@ use nix::{
         prctl::set_dumpable,
         resource::{getrlimit, setrlimit, Resource},
         socket::{
-            accept4, connect, listen, shutdown, socket, AddressFamily, Backlog, Shutdown, SockFlag,
-            SockType, SockaddrLike, SockaddrStorage, UnixAddr,
+            accept4, listen, shutdown, AddressFamily, Backlog, Shutdown, SockFlag, SockType,
+            SockaddrLike, SockaddrStorage, UnixAddr,
         },
         stat::Mode,
     },
@@ -46,15 +46,21 @@ use syd::{
     compat::epoll_ctl_safe,
     config::PIPE_BUF,
     confine::{confine_mdwe, seccomp_native_has_socketcall},
+    cookie::{
+        safe_connect, safe_socket, CONNECT_COOKIE_ARG3, CONNECT_COOKIE_ARG4, CONNECT_COOKIE_ARG5,
+        SOCKET_COOKIE_ARG3, SOCKET_COOKIE_ARG4, SOCKET_COOKIE_ARG5,
+    },
     err::{SydError, SydResult},
     fs::{closeexcept, duprand, fillrandom, randint, retry_on_eintr},
     hash::SydHashMap,
+    ignore_signals,
     landlock_policy::LandlockPolicy,
     parsers::sandbox::PortSet,
     path::XPathBuf,
     sealbox::{
         check_madvise_guard_support, madvise_guard_install, mprotect_none, mprotect_readonly, mseal,
     },
+    IgnoreSignalOpts,
 };
 
 // Pong reply to Syd for debugging.
@@ -67,7 +73,7 @@ syd::main! {
 
     // Ignore all signals except SIG{KILL,STOP,PIPE,CHLD,Core}.
     // This is used to ensure we can deny {rt_,}sigreturn(2) to mitigate SROP.
-    syd::ignore_signals()?;
+    ignore_signals(IgnoreSignalOpts::empty())?;
 
     // SAFETY: We leak the UNIX socket fd on purpose,
     // and leave it open for process lifetime.
@@ -401,16 +407,18 @@ impl<'a> Proxy<'a> {
         }
 
         // Socket filtering only works if there's no multiplexing socketcall.
+        #[allow(clippy::useless_conversion)]
         if seccomp_native_has_socketcall() {
             for sysname in ["socket", "connect"] {
                 #[allow(clippy::disallowed_methods)]
                 filter.add_rule(ScmpAction::Allow, ScmpSyscall::from_name(sysname).unwrap())?;
             }
         } else {
-            // Restrict socket to the given domain, type and protocol.
+            // Restrict socket(2) to the given domain, type and protocol.
+            // Restrict socket(2) using syscall argument cookies.
             let sock_domain = domain as u64;
             let sock_type = (libc::SOCK_STREAM | libc::SOCK_NONBLOCK | libc::SOCK_CLOEXEC) as u64;
-            let sock_protocol = libc::IPPROTO_IP as u64;
+            let sock_protocol = 0u64;
             #[allow(clippy::disallowed_methods)]
             filter.add_rule_conditional(
                 ScmpAction::Allow,
@@ -419,15 +427,25 @@ impl<'a> Proxy<'a> {
                     scmp_cmp!($arg0 == sock_domain),
                     scmp_cmp!($arg1 == sock_type),
                     scmp_cmp!($arg2 == sock_protocol),
+                    scmp_cmp!($arg3 == (*SOCKET_COOKIE_ARG3).into()),
+                    scmp_cmp!($arg4 == (*SOCKET_COOKIE_ARG4).into()),
+                    scmp_cmp!($arg5 == (*SOCKET_COOKIE_ARG5).into()),
                 ],
             )?;
 
-            // Restrict connect to a single safe pointer.
+            // Restrict connect(2) to a single safe pointer.
+            // Restrict connect(2) using syscall argument cookies.
             #[allow(clippy::disallowed_methods)]
             filter.add_rule_conditional(
                 ScmpAction::Allow,
                 ScmpSyscall::from_name("connect").unwrap(),
-                &[scmp_cmp!($arg1 == addr_ptr), scmp_cmp!($arg2 == addr_len)],
+                &[
+                    scmp_cmp!($arg1 == addr_ptr),
+                    scmp_cmp!($arg2 == addr_len),
+                    scmp_cmp!($arg3 == (*CONNECT_COOKIE_ARG3).into()),
+                    scmp_cmp!($arg4 == (*CONNECT_COOKIE_ARG4).into()),
+                    scmp_cmp!($arg5 == (*CONNECT_COOKIE_ARG5).into()),
+                ],
             )?;
         }
 
@@ -456,7 +474,7 @@ impl<'a> Proxy<'a> {
         chdir(safe_cwd)?;
 
         if !self.debug {
-            // SAFETY: Default panic hook wont play well with seccomp.
+            // SAFETY: Default panic hook won't play well with seccomp.
             std::panic::set_hook(Box::new(|_| {}));
 
             // Set up namespace isolation.
@@ -722,17 +740,16 @@ impl Client {
         let family = extaddr
             .family()
             .ok_or::<SydError>(Errno::EAFNOSUPPORT.into())?;
-        let fd_ext = socket(
-            family,
-            SockType::Stream,
-            SockFlag::SOCK_NONBLOCK | SockFlag::SOCK_CLOEXEC,
-            None,
+        let fd_ext = safe_socket(
+            family as i32,
+            (SockType::Stream as i32) | (SockFlag::SOCK_NONBLOCK | SockFlag::SOCK_CLOEXEC).bits(),
+            0,
         )?;
 
         // Set socket options on the external fd.
         set_socket_options(&fd_ext, None);
 
-        retry_on_eintr(|| connect(fd_ext.as_raw_fd(), extaddr)).or_else(|e| {
+        retry_on_eintr(|| safe_connect(&fd_ext, extaddr)).or_else(|e| {
             if matches!(e, Errno::EAGAIN | Errno::EINPROGRESS) {
                 Ok(())
             } else {
