@@ -16,19 +16,29 @@ use std::{
     thread,
 };
 
-use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
+use libseccomp::{scmp_cmp, RawSyscall, ScmpAction, ScmpFilterContext, ScmpSyscall};
 use nix::{
     errno::Errno,
     fcntl::{splice, tee, OFlag, SpliceFFlags},
     unistd::{lseek64, pipe2, write, Gid, Uid, Whence},
 };
 
+#[cfg(target_arch = "x86")]
+use crate::cookie::FTRUNCATE64_COOKIE_ARG3;
 use crate::{
     config::*,
-    confine::{scmp_add_setid_rules, ExportMode},
+    confine::{
+        confine_scmp_clone, confine_scmp_clone3, confine_scmp_write, scmp_add_setid_rules,
+        ExportMode,
+    },
+    cookie::{
+        safe_ftruncate64, ACCEPT4_COOKIE_ARG4, ACCEPT4_COOKIE_ARG5, FTRUNCATE64_COOKIE_ARG4,
+        FTRUNCATE64_COOKIE_ARG5, FTRUNCATE_COOKIE_ARG2, FTRUNCATE_COOKIE_ARG3,
+        FTRUNCATE_COOKIE_ARG4, FTRUNCATE_COOKIE_ARG5, SYS_ACCEPT4,
+    },
     err::{err2no, SydJoinHandle, SydResult},
     error,
-    fs::{ftruncate64, lock_fd, retry_on_eintr, retry_on_intr, seal_memfd, FileInfo},
+    fs::{lock_fd, retry_on_eintr, retry_on_intr, seal_memfd, FileInfo},
     hash::{
         aes_ctr_enc, aes_ctr_init, hmac_sha256_feed, hmac_sha256_fini, hmac_sha256_init,
         SydHashMap, BLOCK_SIZE, HMAC_TAG_SIZE, IV, IV_SIZE,
@@ -119,13 +129,13 @@ impl AesWorker {
                     let safe_setid = self
                         .flags
                         .intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
+
+                    // Log before load, log after will kill.
+                    // TODO: Ensure syd_aes threads can write to log fd.
                     info!("ctx": "confine", "op": "confine_aes_thread",
                         "msg": format!("AES thread confined with{} SROP mitigation",
                             if safe_setid { "out" } else { "" }));
 
-                    // SAFETY: Default panic hook wont play well with seccomp.
-                    std::panic::set_hook(Box::new(|_| {}));
-
                     ctx.load()?;
                 } else {
                     error!("ctx": "confine", "op": "confine_aes_thread",
@@ -220,7 +230,7 @@ impl AesWorker {
                     // Wait until we take a write lock on the encrypted fd.
                     // This will succeed once all fds owned by the sandbox
                     // process are closed.
-                    lock_fd(&crypt_fd, true, true)?;
+                    retry_on_eintr(|| lock_fd(&crypt_fd, true, true))?;
 
                     // All good, sync contents to disk.
                     let result = Self::sync(fdalg, &files, memfd, &crypt_fd, &crypt_path);
@@ -275,7 +285,7 @@ impl AesWorker {
         #[allow(clippy::cast_sign_loss)]
         let data_size = lseek64(&crypt_fd, 0, Whence::SeekEnd).unwrap() as u64;
         if data_size == 0 {
-            retry_on_eintr(|| ftruncate64(&enc_fd, 0)).unwrap();
+            retry_on_eintr(|| safe_ftruncate64(&enc_fd, 0)).unwrap();
             return Ok(());
         }
 
@@ -324,7 +334,7 @@ impl AesWorker {
             if last_block_offset < file_size {
                 // Truncate the encrypted file to remove the partial block.
                 let truncate_offset = header_size + last_block_offset;
-                retry_on_eintr(|| ftruncate64(&enc_fd, truncate_offset.try_into().unwrap()))?;
+                retry_on_eintr(|| safe_ftruncate64(&enc_fd, truncate_offset.try_into().unwrap()))?;
 
                 // Adjust crypt_fd to read from the last full block offset.
                 #[allow(clippy::cast_possible_wrap)]
@@ -382,7 +392,7 @@ impl AesWorker {
             if file_size > 0 {
                 // Remove previous content,
                 // SAFETY: wipe IV to avoid reuse.
-                retry_on_eintr(|| ftruncate64(&enc_fd, 0))?;
+                retry_on_eintr(|| safe_ftruncate64(&enc_fd, 0))?;
                 lseek64(&enc_fd, 0, Whence::SeekSet)?;
             }
 
@@ -558,12 +568,18 @@ impl AesWorker {
             }
         }
 
-        // Finalize encryption with `false`.
-        aes_ctr_enc(&sock_enc, &[], None, false)?;
-
         // Flush the final batch.
         #[allow(clippy::cast_possible_truncation)]
         while nflush > 0 {
+            // Finalize encryption with `false`.
+            //
+            // Some kernel versions may incorrectly return EINVAL here.
+            // Gracefully handle this errno and move on.
+            match aes_ctr_enc(&sock_enc, &[], None, false) {
+                Ok(_) | Err(Errno::EINVAL) => {}
+                Err(errno) => return Err(errno.into()),
+            }
+
             let len = nflush.min(PIPE_BUF_ALG);
             let n = retry_on_eintr(|| {
                 splice(
@@ -717,20 +733,119 @@ impl AesWorker {
             }
         }
 
-        // Allow reads/writes up to HMAC & IV size.
+        // Allow reads up to MAX(HMAC | IV) bytes.
         // See the note in sync_file().
-        for sysname in ["read", "write"] {
-            match ScmpSyscall::from_name(sysname) {
+        let rw_max = u64::try_from(HMAC_TAG_SIZE.max(IV_SIZE))?;
+        match ScmpSyscall::from_name("read") {
+            Ok(syscall) => {
+                ctx.add_rule_conditional(
+                    ScmpAction::Allow,
+                    syscall,
+                    &[scmp_cmp!($arg2 <= rw_max)],
+                )?;
+            }
+            Err(_) => {
+                info!("ctx": "confine", "op": "allow_aes_syscall",
+                    "msg": "invalid or unsupported syscall read");
+            }
+        }
+
+        // Allow writes to log-fd OR up to MAX(HMAC | IV) bytes.
+        // No proc_pid_mem(5) access required here.
+        confine_scmp_write(&mut ctx, Some(rw_max), false)?;
+
+        // Allow clones without namespace flags.
+        confine_scmp_clone(&mut ctx)?;
+        // Deny clone3 with ENOSYS for compatibility.
+        confine_scmp_clone3(&mut ctx)?;
+
+        // ftruncate{,64}(2) may be used only with syscall argument cookies.
+        let sysname = "ftruncate";
+        #[allow(clippy::useless_conversion)]
+        match ScmpSyscall::from_name(sysname) {
+            Ok(syscall) => {
+                // Secure using syscall argument cookies.
+                let mut rules = vec![];
+                if !flags.allow_unsafe_nocookie() {
+                    rules.extend(&[
+                        scmp_cmp!($arg2 == (*FTRUNCATE_COOKIE_ARG2).into()),
+                        scmp_cmp!($arg3 == (*FTRUNCATE_COOKIE_ARG3).into()),
+                        scmp_cmp!($arg4 == (*FTRUNCATE_COOKIE_ARG4).into()),
+                        scmp_cmp!($arg5 == (*FTRUNCATE_COOKIE_ARG5).into()),
+                    ]);
+                }
+
+                if rules.is_empty() {
+                    ctx.add_rule(ScmpAction::Allow, syscall)?;
+                } else {
+                    ctx.add_rule_conditional(ScmpAction::Allow, syscall, &rules)?;
+                }
+            }
+            Err(_) => {
+                info!("ctx": "confine", "op": "allow_emu_syscall",
+                    "msg": format!("invalid or unsupported syscall {sysname}"));
+            }
+        }
+
+        let sysname = "ftruncate64";
+        #[allow(clippy::useless_conversion)]
+        match ScmpSyscall::from_name(sysname) {
+            Ok(syscall) => {
+                // Secure using syscall argument cookies.
+                let mut rules = vec![];
+                if !flags.allow_unsafe_nocookie() {
+                    rules.extend(&[
+                        #[cfg(target_arch = "x86")]
+                        scmp_cmp!($arg3 == (*FTRUNCATE64_COOKIE_ARG3).into()),
+                        scmp_cmp!($arg4 == (*FTRUNCATE64_COOKIE_ARG4).into()),
+                        scmp_cmp!($arg5 == (*FTRUNCATE64_COOKIE_ARG5).into()),
+                    ]);
+                }
+
+                if rules.is_empty() {
+                    ctx.add_rule(ScmpAction::Allow, syscall)?;
+                } else {
+                    ctx.add_rule_conditional(ScmpAction::Allow, syscall, &rules)?;
+                }
+            }
+            Err(_) => {
+                info!("ctx": "confine", "op": "allow_emu_syscall",
+                    "msg": format!("invalid or unsupported syscall {sysname}"));
+            }
+        }
+
+        // accept4(2) may be used only with syscall argument cookies.
+        //
+        // We only enforce this on architectures where the system call is direct,
+        // and there's no socketcall(2) multiplexer indirection.
+        #[allow(clippy::cast_possible_truncation)]
+        #[allow(clippy::cast_sign_loss)]
+        #[allow(clippy::useless_conversion)]
+        #[allow(deprecated)]
+        if let Some(syscall) = SYS_ACCEPT4.map(|n| ScmpSyscall::from_raw_syscall(n as RawSyscall)) {
+            // Secure using syscall argument cookies.
+            let mut rules = vec![];
+            if !flags.allow_unsafe_nocookie() {
+                rules.extend(&[
+                    scmp_cmp!($arg4 == (*ACCEPT4_COOKIE_ARG4).into()),
+                    scmp_cmp!($arg5 == (*ACCEPT4_COOKIE_ARG5).into()),
+                ]);
+            }
+
+            if rules.is_empty() {
+                ctx.add_rule(ScmpAction::Allow, syscall)?;
+            } else {
+                ctx.add_rule_conditional(ScmpAction::Allow, syscall, &rules)?;
+            }
+        } else {
+            match ScmpSyscall::from_name("accept4") {
                 Ok(syscall) => {
-                    ctx.add_rule_conditional(
-                        ScmpAction::Allow,
-                        syscall,
-                        &[scmp_cmp!($arg2 <= HMAC_TAG_SIZE.max(IV_SIZE) as u64)],
-                    )?;
+                    // Allow socketcall(2).
+                    ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_aes_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                    info!("ctx": "confine", "op": "allow_emu_syscall",
+                        "msg": "invalid or unsupported syscall accept4");
                 }
             }
         }
