@@ -8,95 +8,199 @@
 
 use std::{
     env,
-    os::fd::RawFd,
+    os::fd::{FromRawFd, OwnedFd, RawFd},
     sync::{
         atomic::{AtomicBool, Ordering},
         Arc,
     },
     thread,
-    thread::JoinHandle,
 };
 
 use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
 use nix::{
     errno::Errno,
-    unistd::{getpid, lseek64, Pid, Whence},
+    sched::{unshare, CloneFlags},
+    unistd::{getpid, write, Gid, Pid, Uid},
 };
+use serde::{ser::SerializeMap, Serialize, Serializer};
 
 use crate::{
+    alert,
     cache::SysInterrupt,
     config::*,
-    err::{err2no, SydResult},
+    confine::{confine_scmp_write, scmp_add_setid_rules, ExportMode},
+    cookie::{OPENAT2_COOKIE_ARG4, OPENAT2_COOKIE_ARG5},
+    err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
-    fs::seccomp_notify_id_valid,
+    fs::{closeexcept, retry_on_eintr, seccomp_notify_id_valid, tgkill},
     info,
-    landlock_policy::LandlockPolicy,
-    proc::proc_status_read,
+    proc::proc_status,
     sandbox::Flags,
+    sigset::SydSigSet,
     workers::WorkerCache,
-    ExportMode,
 };
 
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+enum InterruptSource {
+    Signal(SydSigSet),
+    ProcessInvalid(Errno),
+}
+
+impl Serialize for InterruptSource {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut map = serializer.serialize_map(Some(2))?;
+        match self {
+            Self::Signal(set) => {
+                map.serialize_entry("name", "signal")?;
+                map.serialize_entry("set", set)?;
+            }
+            Self::ProcessInvalid(errno) => {
+                let err = *errno as i32;
+                map.serialize_entry("name", "process")?;
+                map.serialize_entry("err", &err)?;
+            }
+        }
+        map.end()
+    }
+}
+
 #[derive(Clone)]
 pub(crate) struct Interrupter {
-    scmp: RawFd,
+    seccomp_fd: RawFd,
     flags: Flags,
+
+    transit_uids: Vec<(Uid, Uid)>,
+    transit_gids: Vec<(Gid, Gid)>,
+
     should_exit: Arc<AtomicBool>,
     cache: Arc<WorkerCache<'static>>,
 }
 
 impl Interrupter {
     pub(crate) fn new(
-        scmp: RawFd,
+        seccomp_fd: RawFd,
         flags: Flags,
+        transit_uids: &[(Uid, Uid)],
+        transit_gids: &[(Gid, Gid)],
         should_exit: Arc<AtomicBool>,
         cache: Arc<WorkerCache<'static>>,
     ) -> Self {
         Self {
-            scmp,
             flags,
+            seccomp_fd,
             should_exit,
             cache,
+            transit_uids: transit_uids.to_vec(),
+            transit_gids: transit_gids.to_vec(),
         }
     }
 
     #[allow(clippy::cognitive_complexity)]
-    pub(crate) fn try_spawn(self) -> Result<JoinHandle<()>, Errno> {
+    pub(crate) fn try_spawn(self, notif_pipe: (RawFd, RawFd)) -> Result<SydJoinHandle<()>, Errno> {
         thread::Builder::new()
             .name("syd_int".to_string())
             .stack_size(INT_STACK_SIZE)
             .spawn(move || {
+                // SAFETY: We use exit_group(2) here to bail,
+                // because this unsharing is a critical safety feature.
+                if let Err(errno) = unshare(CloneFlags::CLONE_FS | CloneFlags::CLONE_FILES) {
+                    alert!("ctx": "boot", "op": "unshare_interrupt_thread",
+                        "msg": format!("failed to unshare(CLONE_FS|CLONE_FILES): {errno}"),
+                        "err": errno as i32);
+                    std::process::exit(101);
+                }
+
+                // SAFETY: notif_pipe points to valid FDs.
+                let (pipe_rd, pipe_wr) = unsafe {
+                    (
+                        OwnedFd::from_raw_fd(notif_pipe.0),
+                        OwnedFd::from_raw_fd(notif_pipe.1),
+                    )
+                };
+                drop(pipe_rd);
+                let buf = [42u8; 1];
+                #[allow(clippy::disallowed_methods)]
+                match retry_on_eintr(|| write(&pipe_wr, &buf)).unwrap() {
+                    0 => return Err(Errno::EIO.into()), // Syd died before reading.
+                    1 => {}
+                    n => unreachable!("BUG: invalid pipe write of size {n}!"),
+                }
+
+                // Close the notification pipe.
+                drop(pipe_wr);
+
+                // SAFETY: The Interrupt thread needs to inherit the following FDs:
+                // 1. Seccomp-notify FD.
+                // 2. Static FD of procfs(5).
+                // 3. Log FD.
+                // We have to sort the set as the FDs are randomized.
+                #[allow(clippy::cast_sign_loss)]
+                let mut set = vec![
+                    self.seccomp_fd as libc::c_uint,
+                    PROC_FD() as libc::c_uint,
+                    crate::log::LOG_FD.load(Ordering::Relaxed) as libc::c_uint,
+                ];
+                set.sort_unstable();
+                if let Err(errno) = closeexcept(&set) {
+                    alert!("ctx": "boot", "op": "close_range_interrupt_thread",
+                        "msg": format!("failed to close range: {errno}"),
+                        "err": errno as i32);
+                    std::process::exit(101);
+                }
+                drop(set);
+
                 // To be used by tgkill when signaling threads.
                 let tgid = getpid();
 
                 // Honour dry-run when exporting.
                 let dry_run =
                     env::var_os(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
-                if !dry_run {
-                    // SAFETY: Default panic hook wont play well with seccomp.
-                    std::panic::set_hook(Box::new(|_| {}));
-                }
 
-                // SAFETY: Logging will kill us after seccomp.
-                let safe_setid = self
-                    .flags
-                    .intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
-                info!("ctx": "confine", "op": "confine_interrupt_thread",
-                    "msg": format!("interrupt thread confined with{} SROP mitigation",
-                        if safe_setid { "out" } else { "" }));
-
-                // SAFETY: Panic if we cannot prepare the seccomp filter.
-                // Note, prepare_confine also applies Landlock for !dry_run.
-                #[allow(clippy::disallowed_methods)]
-                let ctx = Self::prepare_confine(self.scmp, tgid, self.flags, dry_run)
-                    .expect("prepare interrupt thread confinement");
-                #[allow(clippy::disallowed_methods)]
+                // Confine `syd_int' thread.
                 if !dry_run {
-                    // SAFETY: Panic if we cannot confine the thread.
-                    ctx.load().expect("confine interrupt thread");
+                    // SAFETY: We use exit_group(2) here to bail,
+                    // because this confinement is a critical safety feature.
+                    let ctx = match Self::prepare_confine(
+                        self.seccomp_fd,
+                        tgid,
+                        self.flags,
+                        &self.transit_uids,
+                        &self.transit_gids,
+                        false,
+                    ) {
+                        Ok(ctx) => ctx,
+                        Err(error) => {
+                            let errno = error.errno().unwrap_or(Errno::ENOSYS);
+                            alert!("ctx": "boot", "op": "confine_interrupt_thread",
+                                "msg": format!("failed to confine: {error}"),
+                                "err": errno as i32);
+                            std::process::exit(101);
+                        }
+                    };
+
+                    // Load seccomp(2) BPF into the kernel.
+                    // SAFETY: We use exit_group(2) here to bail,
+                    // because this confinement is a critical safety feature.
+                    if let Err(error) = ctx.load() {
+                        let errno = scmp2no(&error).unwrap_or(Errno::ENOSYS);
+                        alert!("ctx": "boot", "op": "confine_interrupt_thread",
+                            "msg": format!("failed to confine: {error}"),
+                            "err": errno as i32);
+                        std::process::exit(101);
+                    }
+
+                    let safe_setid = self
+                        .flags
+                        .intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
+                    info!("ctx": "confine", "op": "confine_interrupt_thread",
+                        "msg": format!("interrupt thread confined with{} SROP mitigation",
+                            if safe_setid { "out" } else { "" }));
                 } else {
-                    error!("ctx": "confine", "op": "confine_interrupt_thread",
-                        "msg": "interrupter thread is running unconfined in debug mode");
+                    error!("ctx": "confine", "op": "confine_int_thread",
+                        "msg": "interrupt thread is running unconfined in debug mode");
                 }
 
                 // Enter main loop.
@@ -105,7 +209,7 @@ impl Interrupter {
             .map_err(|err| err2no(&err))
     }
 
-    fn main(self, tgid: Pid) {
+    fn main(self, tgid: Pid) -> SydResult<()> {
         loop {
             // Wait for one cycle.
             std::thread::sleep(INT_CYCLE_TIME);
@@ -115,7 +219,8 @@ impl Interrupter {
                 let (ref lock, ref cvar) = *self.cache.sysint_map.sys_block;
                 let mut map = lock.lock().unwrap_or_else(|err| err.into_inner());
 
-                // As long as the map is empty, we wait for an insert notification.
+                // As long as the map is empty,
+                // we wait for an insert notification.
                 map = cvar
                     .wait_while(map, |map| map.is_empty())
                     .unwrap_or_else(|err| err.into_inner());
@@ -129,40 +234,31 @@ impl Interrupter {
                 break;
             }
         }
+
+        Ok(())
     }
 
     fn handle_interrupt(&self, tgid: Pid, interrupt: &SysInterrupt) -> bool {
-        // Check if request is still valid.
-        if !self.is_valid(interrupt.request.id) {
-            // Request no longer valid:
-            // Interrupt syd_emu thread and remove entry.
-            Self::interrupt(tgid, interrupt.handler);
-            return false;
-        }
-
         // Check pending signals for the thread.
         //
         // SAFETY:
         // 1. We want to wake the respective syd_emu thread in case the
         //    process is no longer valid otherwise we may end up with a
         //    deadlock: See miniupnpc tests, thx kepstin!
-        // 2. Because we preopen the status-fd and seek here, there's
-        //    no concern for PID-reuse, therefore we don't do a post
-        //    seccomp-id validation.
-        let status = if lseek64(&interrupt.status_fd, 0, Whence::SeekSet).is_ok() {
-            proc_status_read(&interrupt.status_fd).ok()
-        } else {
-            None
-        };
-
-        let status = if let Some(status) = status {
-            // SAFETY: seccomp-id validated, proc status is valid.
-            status
-        } else {
-            // Process no longer valid:
-            // Interrupt syd_emu thread and remove entry.
-            Self::interrupt(tgid, interrupt.handler);
-            return false;
+        // 2. To prevent PID reuse vectors we validate the request ID.
+        #[allow(clippy::cast_possible_wrap)]
+        let pid = Pid::from_raw(interrupt.request.pid as libc::pid_t);
+        let status = match proc_status(pid) {
+            Ok(status) if self.is_valid(interrupt.request.id) => {
+                // SAFETY: seccomp-id validated, proc_pid_status(5) is valid.
+                status
+            }
+            Err(errno) if self.is_valid(interrupt.request.id) => {
+                let source = InterruptSource::ProcessInvalid(errno);
+                Self::interrupt(tgid, interrupt, source);
+                return false;
+            }
+            _ => return false,
         };
 
         // Check for per-{thread,process} pending signals:
@@ -205,7 +301,7 @@ impl Interrupter {
                 .sig_restart
                 .lock()
                 .unwrap_or_else(|err| err.into_inner())
-                .get(&interrupt.tgid)
+                .get(&Pid::from_raw(status.pid))
             {
                 sigset.del_set(*sigset_restart);
 
@@ -217,27 +313,25 @@ impl Interrupter {
         }
 
         // Interrupt syd_emu thread and remove entry.
-        Self::interrupt(tgid, interrupt.handler);
+        Self::interrupt(tgid, interrupt, InterruptSource::Signal(sigset));
         false
     }
 
     // Interrupt the respective `syd_emu` thread.
-    #[inline]
-    fn interrupt(syd: Pid, syd_emu: Pid) {
-        // SAFETY: There's no libc wrapper for tgkill.
-        match Errno::result(unsafe {
-            libc::syscall(
-                libc::SYS_tgkill,
-                syd.as_raw(),
-                syd_emu.as_raw(),
-                libc::SIGALRM,
-            )
-        }) {
-            Ok(_) | Err(Errno::ESRCH) => {}
+    #[allow(clippy::cognitive_complexity)]
+    fn interrupt(tgid: Pid, interrupt: &SysInterrupt, source: InterruptSource) {
+        match tgkill(tgid, interrupt.handler, libc::SIGALRM) {
+            Ok(_) | Err(Errno::ESRCH) => {
+                info!("ctx": "int", "op": "interrupt_emulator",
+                    "msg": "interrupted emulator thread",
+                    "src": source, "int": interrupt);
+            }
             Err(errno) => {
-                // SAFETY: Inter-thread signaling does not work.
-                // This is seriously wrong, exit ASAP.
-                unsafe { libc::_exit(errno as i32) };
+                alert!("ctx": "int", "op": "interrupt_emulator",
+                    "msg": format!("failed to interrupt emulator: {errno}"),
+                    "err": errno as i32,
+                    "src": source, "int": interrupt);
+                std::process::exit(101);
             }
         }
     }
@@ -246,32 +340,23 @@ impl Interrupter {
     fn is_valid(&self, id: u64) -> bool {
         // EAGAIN|EINTR is handled.
         // ENOENT means child died mid-way.
-        seccomp_notify_id_valid(self.scmp, id).is_ok()
+        seccomp_notify_id_valid(self.seccomp_fd, id).is_ok()
     }
 
     /// Confine Interrupter thread.
     #[allow(clippy::cognitive_complexity)]
     pub(crate) fn prepare_confine(
-        scmp: RawFd,
+        seccomp_fd: RawFd,
         tgid: Pid,
         flags: Flags,
-        dry_run: bool,
+        transit_uids: &[(Uid, Uid)],
+        transit_gids: &[(Gid, Gid)],
+        _dry_run: bool,
     ) -> SydResult<ScmpFilterContext> {
-        if !dry_run {
-            // SAFETY: Set up a Landlock sandbox to disallow:
-            // 1. All read, write, exec, network access.
-            // 2. Scoped UNIX sockets.
-            // We cannot enable scoped signals because we
-            // want to signal Syd syscall handler threads
-            // that are going to be outside this Landlock
-            // sandbox.
-            let abi = crate::landlock::ABI::new_current();
-            let policy = LandlockPolicy {
-                scoped_abs: true,
-                ..Default::default()
-            };
-            let _ = policy.restrict_self(abi);
-        }
+        // Note, we cannot confine `syd_int` with a per-thread
+        // Landlock filter here, because it requires access to
+        // proc_pid_status(5) which in turn requires ptrace
+        // rights and Landlock unconditionally limits that.
 
         // Create seccomp filter with default action.
         let mut ctx = ScmpFilterContext::new(ScmpAction::KillProcess)?;
@@ -282,7 +367,7 @@ impl Interrupter {
 
         // Disable Speculative Store Bypass mitigations
         // with trace/allow_unsafe_spec_exec:1
-        ctx.set_ctl_ssb(flags.contains(Flags::FL_ALLOW_UNSAFE_SPEC_EXEC))?;
+        ctx.set_ctl_ssb(flags.allow_unsafe_spec_exec())?;
 
         // DO NOT synchronize filter to all threads.
         // Other threads will self-confine.
@@ -315,7 +400,7 @@ impl Interrupter {
                 )?;
             }
             Err(_) => {
-                info!("ctx": "confine", "op": "allow_syscall",
+                info!("ctx": "confine", "op": "allow_int_syscall",
                     "msg": format!("invalid or unsupported syscall {sysname}"));
             }
         }
@@ -331,36 +416,51 @@ impl Interrupter {
                     ScmpAction::Allow,
                     syscall,
                     &[
-                        scmp_cmp!($arg0 == scmp as u64),
+                        scmp_cmp!($arg0 == seccomp_fd as u64),
                         scmp_cmp!($arg1 == crate::fs::SECCOMP_IOCTL_NOTIF_ID_VALID as u64),
                     ],
                 )?;
             }
             Err(_) => {
-                info!("ctx": "confine", "op": "allow_syscall",
+                info!("ctx": "confine", "op": "allow_int_syscall",
                     "msg": format!("invalid or unsupported syscall {sysname}"));
             }
         }
 
-        // Deny open and stat family with ENOSYS rather than KillProcess.
+        // Allow openat2 with the static procfs(5) fd only.
+        // Apply system call argument cookies.
+        let sysname = "openat2";
+        #[allow(clippy::cast_sign_loss)]
+        #[allow(clippy::useless_conversion)]
+        match ScmpSyscall::from_name(sysname) {
+            Ok(syscall) => {
+                ctx.add_rule_conditional(
+                    ScmpAction::Allow,
+                    syscall,
+                    &[
+                        scmp_cmp!($arg0 == PROC_FD() as u64),
+                        scmp_cmp!($arg4 == (*OPENAT2_COOKIE_ARG4).into()),
+                        scmp_cmp!($arg5 == (*OPENAT2_COOKIE_ARG5).into()),
+                    ],
+                )?;
+            }
+            Err(_) => {
+                info!("ctx": "confine", "op": "allow_int_syscall",
+                    "msg": format!("invalid or unsupported syscall {sysname}"));
+            }
+        }
+
+        // Deny rest of open and stat family with ENOSYS rather than KillProcess.
         // We need this because std::thread::spawn has unwanted
         // side-effects such as opening /sys/devices/system/cpu/online
         // on some architectures.
-        for sysname in [
-            "open",
-            "openat",
-            "openat2",
-            "stat",
-            "lstat",
-            "statx",
-            "newfstatat",
-        ] {
+        for sysname in ["open", "openat", "stat", "lstat", "statx", "newfstatat"] {
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule(ScmpAction::Errno(Errno::ENOSYS as i32), syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_int_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
@@ -371,7 +471,7 @@ impl Interrupter {
             let syscall = match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => syscall,
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_int_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                     continue;
                 }
@@ -382,6 +482,28 @@ impl Interrupter {
             }
         }
 
+        // Prevent executable memory.
+        const PROT_EXEC: u64 = libc::PROT_EXEC as u64;
+        for sysname in ["mmap", "mmap2", "mprotect"] {
+            match ScmpSyscall::from_name(sysname) {
+                Ok(syscall) => {
+                    ctx.add_rule_conditional(
+                        ScmpAction::Allow,
+                        syscall,
+                        &[scmp_cmp!($arg2 & PROT_EXEC == 0)],
+                    )?;
+                }
+                Err(_) => {
+                    info!("ctx": "confine", "op": "allow_int_syscall",
+                        "msg": format!("invalid or unsupported syscall {sysname}"));
+                }
+            }
+        }
+
+        // Allow writes to the log-fd.
+        // No proc_pid_mem(5) access required here.
+        confine_scmp_write(&mut ctx, None, false)?;
+
         // Allow safe system calls.
         for sysname in INT_SYSCALLS {
             match ScmpSyscall::from_name(sysname) {
@@ -389,7 +511,7 @@ impl Interrupter {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_int_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
@@ -402,7 +524,7 @@ impl Interrupter {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_int_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
@@ -415,42 +537,24 @@ impl Interrupter {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_int_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
         }
 
-        if flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID) {
-            // SAFETY: Main thread confines these further.
-            // As these system calls as per-process,
-            // the main thread's seccomp rules will apply
-            // to us even without TSYNC.
-            for sysname in SET_ID_SYSCALLS {
-                match ScmpSyscall::from_name(sysname) {
-                    Ok(syscall) => {
-                        ctx.add_rule(ScmpAction::Allow, syscall)?;
-                    }
-                    Err(_) => {
-                        info!("ctx": "confine", "op": "allow_syscall",
-                            "msg": format!("invalid or unsupported syscall {sysname}"));
-                    }
-                }
-            }
-
-            // SAFETY:
-            // Signal system calls are necessary to handle reserved signals.
-            for sysname in ["sigreturn", "rt_sigreturn"] {
-                match ScmpSyscall::from_name(sysname) {
-                    Ok(syscall) => {
-                        ctx.add_rule(ScmpAction::Allow, syscall)?;
-                    }
-                    Err(_) => {
-                        info!("ctx": "confine", "op": "allow_syscall",
-                            "msg": format!("invalid or unsupported syscall {sysname}"));
-                    }
-                }
-            }
+        // Allow UID/GID changing system calls as necessary.
+        let safe_setuid = flags.allow_safe_setuid();
+        let safe_setgid = flags.allow_safe_setgid();
+        if safe_setuid || safe_setgid {
+            scmp_add_setid_rules(
+                "int",
+                &mut ctx,
+                safe_setuid,
+                safe_setgid,
+                transit_uids,
+                transit_gids,
+            )?;
         }
 
         Ok(ctx)
