@@ -23,6 +23,8 @@ const SYD_AES_EXE: &'static str = env!("CARGO_BIN_EXE_syd-aes");
 const SYD_KEY_EXE: &'static str = env!("CARGO_BIN_EXE_syd-key");
 const SYD_LOCK_EXE: &'static str = env!("CARGO_BIN_EXE_syd-lock");
 const SYD_MDWE_EXE: &'static str = env!("CARGO_BIN_EXE_syd-mdwe");
+const SYD_OFD_EXE: &'static str = env!("CARGO_BIN_EXE_syd-ofd");
+const SYD_PAUSE_EXE: &'static str = env!("CARGO_BIN_EXE_syd-pause");
 const SYD_PDS_EXE: &'static str = env!("CARGO_BIN_EXE_syd-pds");
 const SYD_ELF_EXE: &'static str = env!("CARGO_BIN_EXE_syd-elf");
 const SYD_CPU_EXE: &'static str = env!("CARGO_BIN_EXE_syd-cpu");
@@ -50,6 +52,8 @@ fn syd_test() {
     env::set_var("CARGO_BIN_EXE_syd-key", SYD_KEY_EXE);
     env::set_var("CARGO_BIN_EXE_syd-lock", SYD_LOCK_EXE);
     env::set_var("CARGO_BIN_EXE_syd-mdwe", SYD_MDWE_EXE);
+    env::set_var("CARGO_BIN_EXE_syd-ofd", SYD_OFD_EXE);
+    env::set_var("CARGO_BIN_EXE_syd-pause", SYD_PAUSE_EXE);
     env::set_var("CARGO_BIN_EXE_syd-pds", SYD_PDS_EXE);
     env::set_var("CARGO_BIN_EXE_syd-elf", SYD_ELF_EXE);
     env::set_var("CARGO_BIN_EXE_syd-cpu", SYD_CPU_EXE);
